package com.r3chess;

//
// Fruit 2.1 chess engine by Fabien Letouzey, 2004-2005.
// At http://www.fruitchess.com,  http://wbec-ridderkerk.nl/
//
// Java port by http://chessforeva.blogspot.com, 2013
//
// The idea was to get not too heavy java AI for small MIDlet.
//
// Small 5Kb opening book.
//
// Samples and all the usage is obvious down at the end.
// Free usage and much thanks to Fabien!
//

import java.util.*;

class Fruit21
{
	// main constants

	final static int FileNb = 16;   // const int
	final static int RankNb = 16;   // const int
	final static int SquareNb = FileNb*RankNb;   // const int
	final static int PieceNb = 256; // const int

	final static int DepthMax = 16;     // was 64 const int
	final static int HeightMax = 256;   // const int
	final static int TransDepth = 1;    // const int
	
	final static int ColourNone = -1; // const int
	final static int White = 0;       // const int
	final static int Black = 1;       // const int
	final static int ColourNb = 2;    // const int

	final static int TRUE = 1;
	final static int FALSE = 0;

	final static boolean UseTable = true;   // const bool
	final static int MaterialTableSize = 64 * 256;         // const size of Material hashing (array elements to use)
	final static int PawnTableSize = 64 * 256;             // const size of Pawn hashing (array elements to use)

	final static boolean UseTrans = true;   // const bool
	final static int TransSize = 64 * 256;                 // const size of transp-table (array elements to use)
	// it is not a memory hash size, because there is no memory allocation at all

	public String bestmv = "";      // string contains best move
	public String bestmv2 = "";      // string contains pgn-format of the move 

	private boolean ShowInfo = false;  // set true to show thinking!
	private boolean DispStats = false;  // stats of hashing

	final static int WhiteFlag = (1 << White);   // const int
	final static int BlackFlag = (1 << Black);   // const int
	final static int WxorB = (White ^ Black);
	final static int bnot1 = (~1);
	final static int bnot3 = (~3);
	final static int bnotx77 = (~0x77);
	final static int bnotxF = (~0xF);

	final static int V07777 = 4095;          // const int
	final static int bnotV07777  = (~V07777);   // const int
	final static int StackSize = 4096; // const int


	final static int ListSize = 256;   // const int
	final static boolean UseStrict = true;   // const bool

	final static int SearchNormal = 0;  // const int
	final static int SearchShort = 1;  // const int

	final static int SearchUnknown = 0; // const int
	final static int SearchUpper = 1; // const int
	final static int SearchLower = 2; // const int
	final static int SearchExact = 3; // const int

	final static boolean UseShortSearch = true;    // const bool
	final static int ShortSearchDepth = 1;     // const int

	final static boolean DispBest = true;          // const bool
	final static boolean DispDepthStart = true;    // const bool
	final static boolean DispDepthEnd = true;      // const bool
	final static boolean DispRoot = true;          // const bool
	final static boolean DispStat = true;          // const bool

	final static boolean UseEasy = true;           // const bool  singular move
	final static int EasyThreshold = 150;      // const int
	final static int EasyRatio = 20;         // const

	final static boolean UseEarly = true;          // const bool  early iteration end
	final static int EarlyRatio = 60;        // const

	final static boolean UseBad = true;            // const bool
	final static int BadThreshold = 50;        // const int
	final static boolean UseExtension = true;      // const bool

	final static int DateSize = 16;            // const int
	final static int DepthNone = -128;         // const int
	final static int ClusterSize = 4;          // const int, not a hash size

	final static int IncNone = 0;          // const int
	final static int IncNb = (2*17) + 1;   // const int
	final static int IncOffset = 17;       // const int

	final static int DeltaNone = 0;           // const int
	final static int DeltaNb = (2*119) + 1;   // const int
	final static int DeltaOffset = 119;       // const int

	// classes for objects
	
	public class alist_t
	{
	    public int size = 0;          // int
	    public int[] square = new int[15];
	}
	public class alists_t
	{
	    public alist_t[] alist = { new alist_t(), new alist_t() };	//alist_t [ColourNb][1]
	}
	public class attack_t
	{
	    public int dn = 0;   // int
	    public int[] ds = { 0, 0, 0 }; // int[2+1];
	    public int[] di = { 0, 0, 0 }; // int[2+1];
	}
	public class board_t
	{

	    public int[] square = new int[SquareNb];  // int[SquareNb]
	    public int[] pos = new int[SquareNb];     // int[SquareNb]

	    public int piece[][] = new int[ColourNb][32];   // int[ColourNb][32] only 17 are needed

	    public int[] piece_size = new int[ColourNb];  // int[ColourNb]

	    public int[][] pawn = new int[ColourNb][16];       // int[ColourNb][16] only 9 are needed

	    public int[] pawn_size = new int[ColourNb];  // int[ColourNb]

	    public int piece_nb = 0;   // int
	    public int[] number = new int[16];    // int[16] only 12 are needed

	    public int[][] pawn_file = new int[ColourNb][FileNb]; // int[ColourNb][FileNb];

	    public int turn = 0;       // int
	    public int flags = 0;      // int
	    public int ep_square = 0;  // int
	    public int ply_nb = 0;     // int
	    public int sp = 0;         // int

	    public int cap_sq = 0;     // int

	    public int opening = 0;    // int
	    public int endgame = 0;    // int

	    public int key = 0;           // uint64
	    public int pawn_key = 0;      // uint64
	    public int material_key = 0;  // uint64

	    public int[] stack = new int[StackSize];        // uint64[StackSize];
	    public int movenumb = 0;     // int

	}
	public class endgame_t
	{
	    public int v = 0;
	}
	public class entry_t
	{
	    public int lock = 0;        // uint32
	    public int move = 0;        // uint16
	    public int depth = 0;       // sint8
	    public int date = 0;        // uint8
	    public int move_depth = 0;  // sint8
	    public int flags = 0;       // uint8
	    public int min_depth = 0;   // sint8
	    public int max_depth = 0;   // sint8
	    public int min_value = 0;   // sint16
	    public int max_value = 0;   // sint16
	}
	public class int_t
	{
	    public int v = 0;
	}
	public class list_t
	{
	    public int size = 0;      // int
	    public int[] move = new int[ListSize];  // int[ListSize]
	    public int[] value = new int[ListSize];  // short int[ListSize]
	}
	public class material_info_t
	{
	    public int lock = 0;	     // uint32
	    public int recog = 0;            // uint8
	    public int flags = 0;            // uint8
	    public int[] cflags = new int[ColourNb]; // uint8[ColourNb]
	    public int[] mul = new int[ColourNb];     // uint8[ColourNb]
	    public int phase = 0;           // sint16
	    public int opening = 0;         // sint16
	    public int endgame = 0;         // sint16
	}
	public class material_t
	{
	    public material_info_t[] table = new material_info_t[MaterialTableSize];        // entry_t*
	    public int size = 0;             // uint32
	    public int mask = 0;             // uint32
	    public int used = 0;             // uint32

	    public int read_nb = 0;          // sint64
	    public int read_hit = 0;         // sint64
	    public int write_nb = 0;         // sint64
	    public int write_collision = 0;  // sint64
	}
	public class my_timer_t
	{
	    public long start_real = 0;	// double
	    public long elapsed_real = 0;	// double
	    public boolean running = false;    // bool
	}
	public class opening_t
	{
	    public int v = 0;
	}
	public class opt_t_def
	{
	    public String vary = "";         // string
	    public boolean decl = false;     // bool
	    public String init = "";         // string
	    public String type = "";         // string
	    public String extra = "";        // string
	    public String val = "";          // string
	}
	public class pawn_info_t
	{
	    public int lock = 0;                // uint32
	    public int opening = 0;             // sint16
	    public int endgame = 0;             // sint16
	    public int[] flags = new int[ColourNb];        // uint8[ColourNb]
	    public int[] passed_bits = new int[ColourNb];  // uint8[ColourNb]
	    public int[] single_file = new int[ColourNb];  // uint8[ColourNb]
	    public int pad = 0;                 // uint16
	} 
	public class pawn_t
	{
	    public pawn_info_t[] table = new pawn_info_t[PawnTableSize];       // entry_t*
	    public int size = 0;             // uint32
	    public int mask = 0;             // uint32
	    public int used = 0;             // uint32

	    public int read_nb = 0;          // sint64
	    public int read_hit = 0;         // sint64
	    public int write_nb = 0;         // sint64
	    public int write_collision = 0;  // sint64
	}
	public class search_best_t
	{
	    public int move = 0;    // int
	    public int value = 0;   // int
	    public int flags = 0;   // int
	    public int depth = 0;   // int
	    public int[] pv = new int[HeightMax];    // int[HeightMax];
	}
	public class search_current_t
	{
	    public board_t board = new board_t();       // board_t[1]
	    public my_timer_t timer = new my_timer_t();    // my_timer_t[1]
	    public int mate = 0;         // int
	    public int depth = 0;        // int
	    public int max_depth = 0;    // int
	    public int node_nb = 0;      // sint64
	    public long time = 0;       // double
	    public long speed = 0;      // double
	}
	public class search_info_t
	{
	    public boolean can_stop = false;   // bool
	    public boolean stop  = false;       // bool
	    public int check_nb = 10000;  // was 100000;       // int
	    public int check_inc = 10000;  // was 100000
	    public long last_time = 0;    // double
	}
	public class search_input_t
	{
	    public board_t board = new board_t();          // board_t[1]
	    public list_t list = new list_t();            // list_t[1]
	    public boolean infinite = false;           // bool
	    public boolean depth_is_limited = false;   // bool
	    public int depth_limit = 0;            // int
	    public boolean time_is_limited = false;    // bool
	    public long time_limit_1 = 0;         // double
	    public long time_limit_2 = 0;         // double
	}
	public class search_root_t
	{
	    public list_t list = new list_t();  // list_t[1]
	    public int depth = 0;      // int
	    public int move = 0;       // int
	    public int move_pos = 0;   // int
	    public int move_nb = 0;    // int
	    public int last_value = 0; // int
	    public boolean bad_1 = false;  // bool
	    public boolean bad_2 = false;  // bool
	    public boolean change = false; // bool
	    public boolean easy = false;   // bool
	    public boolean flag = false;   // bool
	}
	public class sort_t
	{
	    public int depth = 0;         // int
	    public int height = 0;        // int
	    public int trans_killer = 0;  // int
	    public int killer_1 = 0;      // int
	    public int killer_2 = 0;      // int
	    public int gen = 0;           // int
	    public int test = 0;          // int
	    public int pos = 0;           // int
	    public int value = 0;         // int
	    public board_t board = null;        // board_t *
	    public attack_t attack = null;      // const attack_t *
	    public list_t list = new list_t();   // list_t[1]
	    public list_t bad = new list_t();    // list_t[1]
	}
	public class string_t
	{
	    public String v = "";
	}
	public class trans_rtrv
	{
	    public int trans_move = 0;        // int
	    public int trans_min_depth = 0;   // int
	    public int trans_max_depth = 0;   // int
	    public int trans_min_value = 0;   // int
	    public int trans_max_value = 0;   // int
	}
	public class trans_t
	{
	    public entry_t[] table = new entry_t[TransSize];        // entry_t*
	    public int size = 0;             // uint32
	    public int mask = 0;             // uint32
	    public int date = 0;             // int
	    public int[] age = new int[DateSize];       // int[DateSize]
	    public int used = 0;             // uint32
	    public int read_nb = 0;          // sint64
	    public int read_hit = 0;         // sint64
	    public int write_nb = 0;         // sint64
	    public int write_hit = 0;        // sint64
	    public int write_collision = 0;  // sint64
	} 
	public class undo_t
	{
	    public boolean capture = false;   // bool

	    public int capture_square = 0;  // int
	    public int capture_piece = 0;   // int
	    public int capture_pos = 0;     // int

	    public int pawn_pos = 0;        // int

	    public int turn = 0;      // int
	    public int flags = 0;     // int
	    public int ep_square = 0; // int
	    public int ply_nb = 0;    // int

	    public int cap_sq = 0;    // int

	    public int opening = 0;   // int
	    public int endgame = 0;   // int

	    public int key = 0;           // uint64
	    public int pawn_key = 0;      // uint64
	    public int material_key = 0;  // uint64
	}
	

	// some internal funcs
		 
	// should be true, or error otherwise

// --for debugging	
//	private void ASSERT(int id,boolean logic) { 
//	if( ! logic ) {
//	print2out("ASSERT FAIL on id=" + string_from_int(id));
//	}
//	}

	private String  string_from_int( int a ) {
	return Integer.toString(a);
	}


	private void print2out( String s ) {
//
// Be silent in JavaME
//
//	System.out.println(s);
	}
	
	private long millisecs = 0;
	
	private long os_clock() {
	// standard java
	long ms = System.currentTimeMillis();
	long r = 0;
	if(millisecs == 0) millisecs = ms;
	else r = (ms-millisecs);
	return r;
	
	/*
	// javaME
	return (new Date()).getTime()/1000;    // in seconds in java.util.*
	*/
	}

	

	// colour.h


	// my_fatal()

	private void my_fatal( String errmess ) {  // for error case 
	print2out( "my-error: "+errmess );
	}

	private boolean  COLOUR_IS_OK(int colour) { 
	return ((colour & bnot1)==0);
	}

	private boolean  COLOUR_IS_WHITE(int colour) { 
	return (colour==White);
	}

	private boolean  COLOUR_IS_BLACK(int colour) { 
	return (colour!=White);
	}

	private int  COLOUR_FLAG(int colour) { 
	return (colour+1);
	}

	private boolean  COLOUR_IS(int piece,int colour) { 
	return (FLAG_IS(piece,colour+1));
	}

	private boolean  FLAG_IS(int piece, int flag) { 
	return ((piece & flag)!=0);
	}

	private int  COLOUR_OPP(int colour) { 
	return (colour ^ WxorB);
	}

	// end of colour.h

	// piece.h

	// constants

	final static int WhitePawnFlag = (1 << 2);   // const int
	final static int BlackPawnFlag = (1 << 3);   // const int
	final static int KnightFlag = (1 << 4);   // const int
	final static int BishopFlag = (1 << 5);   // const int
	final static int RookFlag = (1 << 6);   // const int
	final static int KingFlag = (1 << 7);   // const int

	final static int PawnFlags = (WhitePawnFlag | BlackPawnFlag);   // const int
	final static int QueenFlags = (BishopFlag | RookFlag);          // const int

	final static int PieceNone64 = 0; // const int
	final static int WhitePawn64 = WhitePawnFlag;  // const int
	final static int BlackPawn64 = BlackPawnFlag;  // const int
	final static int Knight64 = KnightFlag;     // const int
	final static int Bishop64 = BishopFlag;     // const int
	final static int Rook64 = RookFlag;       // const int
	final static int Queen64 = QueenFlags;     // const int
	final static int King64 = KingFlag;       // const int

	final static int PieceNone256 = 0; // const int
	final static int WhitePawn256 =  (WhitePawn64 | WhiteFlag);  // const int
	final static int BlackPawn256 =  (BlackPawn64 | BlackFlag);  // const int
	final static int WhiteKnight256 =  (Knight64 | WhiteFlag);     // const int
	final static int BlackKnight256 =  (Knight64 | BlackFlag);     // const int
	final static int WhiteBishop256 =  (Bishop64 | WhiteFlag);     // const int
	final static int BlackBishop256 =  (Bishop64 | BlackFlag);     // const int
	final static int WhiteRook256 =  (Rook64 | WhiteFlag);       // const int
	final static int BlackRook256 =  (Rook64 | BlackFlag);       // const int
	final static int WhiteQueen256 =  (Queen64 | WhiteFlag);      // const int
	final static int BlackQueen256 =  (Queen64 | BlackFlag);      // const int
	final static int WhiteKing256 =  (King64 | WhiteFlag);       // const int
	final static int BlackKing256 =  (King64 | BlackFlag);       // const int


	final static int WhitePawn12 =  0; // const int
	final static int BlackPawn12 =  1; // const int
	final static int WhiteKnight12 =  2; // const int
	final static int BlackKnight12 =  3; // const int
	final static int WhiteBishop12 =  4; // const int
	final static int BlackBishop12 =  5; // const int
	final static int WhiteRook12 =  6; // const int
	final static int BlackRook12 =  7; // const int
	final static int WhiteQueen12 =  8; // const int
	final static int BlackQueen12 =  9; // const int
	final static int WhiteKing12  = 10; // const int
	final static int BlackKing12 = 11; // const int

	// "constants"

	final static int[] PawnMake = { WhitePawn256, BlackPawn256 };   // const int[ColourNb]

	final static int[] PieceFrom12 = {  WhitePawn256, BlackPawn256, WhiteKnight256, BlackKnight256,
	WhiteBishop256, BlackBishop256, WhiteRook256, BlackRook256,
	WhiteQueen256,  BlackQueen256, WhiteKing256, BlackKing256 };  // const int[12]

	final static String PieceString = "PpNnBbRrQqKk";   // const char[12+1]

	final static int[] PawnMoveInc = { 16, -16 };  // const int[ColourNb]

	final static int[] KnightInc = { -33, -31, -18, -14, 14, 18, 31, 33, 0 };  // const int[8+1]

	final static int[] BishopInc = { -17, -15, 15, 17, 0 };  // const int[4+1]

	final static int[] RookInc = { -16, -1, 1, 16, 0 };  // const int[4+1]

	final static int[] QueenInc = { -17, -16, -15, -1, 1, 15, 16, 17, 0 };  // const[8+1]

	final static int[] KingInc = { -17, -16, -15, -1, 1, 15, 16, 17, 0 };  // const[8+1]


	// variables

	private int[] PieceTo12 = new int[PieceNb];    // int[PieceNb]
	private int[] PieceOrder = new int[PieceNb];   // int[PieceNb]
	private int[][] PieceInc = new int[PieceNb][];     // const

	// macros

	private int  PAWN_OPP(int pawn) { 
	return (pawn ^ (WhitePawn256 ^ BlackPawn256));
	}

	private int  PIECE_COLOUR(int piece) { 
	return ((piece & 3)-1);
	}

	private int  PIECE_TYPE(int piece) { 
	return (piece & bnot3);
	}

	private boolean  PIECE_IS_PAWN(int piece) { 
	return ((piece & PawnFlags)!=0);
	}
/*
	private boolean  PIECE_IS_KNIGHT(int piece) { 
	return ((piece & KnightFlag)!=0);
	}
*/
	
	private boolean  PIECE_IS_BISHOP(int piece) { 
	return ((piece & QueenFlags)==BishopFlag);
	}
/*
	private boolean  PIECE_IS_ROOK(int piece) { 
	return ((piece & QueenFlags)==RookFlag);
	}
*/
	
	private boolean  PIECE_IS_QUEEN(int piece) { 
	return ((piece & QueenFlags)==QueenFlags);
	}

	private boolean  PIECE_IS_KING(int piece) { 
	return ((piece & KingFlag)!=0);
	}

	private boolean  PIECE_IS_SLIDER(int piece) { 
	return ((piece & QueenFlags)!=0);
	}


	// end of piece.h



	// square.h


	final static int FileInc = 1;   // const int
	final static int RankInc = 16;  // const int

	final static int FileNone = 0;   // const int

	final static int FileA = 0x4;   // const int
	final static int FileB = 0x5;   // const int
	final static int FileC = 0x6;   // const int
	final static int FileD = 0x7;   // const int
	final static int FileE = 0x8;   // const int
	final static int FileF = 0x9;   // const int
	final static int FileG = 0xA;   // const int
	final static int FileH = 0xB;   // const int

	final static int RankNone = 0;   // const int

	final static int Rank1 = 0x4;   // const int
	final static int Rank2 = 0x5;   // const int
	final static int Rank3 = 0x6;   // const int
	final static int Rank4 = 0x7;   // const int
	final static int Rank5 = 0x8;   // const int
	final static int Rank6 = 0x9;   // const int
	final static int Rank7 = 0xA;   // const int
	final static int Rank8 = 0xB;   // const int

	final static int SquareNone = 0;   // const int

	final static int A1=0x44; final static int B1=0x45; final static int C1=0x46;
	 final static int D1=0x47; final static int E1=0x48; final static int F1=0x49;
	 final static int G1=0x4A; final static int H1=0x4B;   // const int
	final static int A2=0x54; final static int B2=0x55; final static int C2=0x56;
	 final static int D2=0x57; final static int E2=0x58; final static int F2=0x59;
	 final static int G2=0x5A; final static int H2=0x5B;   // const int
	final static int A3=0x64; final static int B3=0x65; final static int C3=0x66;
	 final static int D3=0x67; final static int E3=0x68; final static int F3=0x69;
	 final static int G3=0x6A; final static int H3=0x6B;   // const int
	final static int A4=0x74; final static int B4=0x75; final static int C4=0x76;
	 final static int D4=0x77; final static int E4=0x78; final static int F4=0x79;
	 final static int G4=0x7A; final static int H4=0x7B;   // const int
	final static int A5=0x84; final static int B5=0x85; final static int C5=0x86;
	 final static int D5=0x87; final static int E5=0x88; final static int F5=0x89;
	 final static int G5=0x8A; final static int H5=0x8B;   // const int
	final static int A6=0x94; final static int B6=0x95; final static int C6=0x96;
	 final static int D6=0x97; final static int E6=0x98; final static int F6=0x99;
	 final static int G6=0x9A; final static int H6=0x9B;   // const int
	final static int A7=0xA4; final static int B7=0xA5; final static int C7=0xA6;
	 final static int D7=0xA7; final static int E7=0xA8; final static int F7=0xA9;
	 final static int G7=0xAA; final static int H7=0xAB;   // const int
	final static int A8=0xB4; final static int B8=0xB5; final static int C8=0xB6;
	 final static int D8=0xB7; final static int E8=0xB8; final static int F8=0xB9;
	 final static int G8=0xBA; final static int H8=0xBB;   // const int


	final static int Dark = 0;   // const int
	final static int Light = 1;   // const int

	// variables

	private int[] SquareTo64 = new int[SquareNb];        		// int[SquareNb]
	private boolean[] SquareIsPromote = new boolean[SquareNb];	// bool[SquareNb]


	// "constants"

	final static int[] SquareFrom64 = {
	A1, B1, C1, D1, E1, F1, G1, H1,
	A2, B2, C2, D2, E2, F2, G2, H2,
	A3, B3, C3, D3, E3, F3, G3, H3,
	A4, B4, C4, D4, E4, F4, G4, H4,
	A5, B5, C5, D5, E5, F5, G5, H5,
	A6, B6, C6, D6, E6, F6, G6, H6,
	A7, B7, C7, D7, E7, F7, G7, H7,
	A8, B8, C8, D8, E8, F8, G8, H8,
	};   // const int[64]

	final static int[] RankMask = {0, 0xF};            // const int[ColourNb]
	final static int[] PromoteRank = { 0xB0, 0x40 };   // const int[ColourNb]

	// macros

	private boolean  SQUARE_IS_OK(int square) { 
	return ((square-0x44 & bnotx77)==0);
	}

	private int  SQUARE_MAKE(int file,int rank) { 
	return ((rank << 4) | file);
	}

	private int  SQUARE_FILE(int square) { 
	return (square & 0xF);
	}

	private int  SQUARE_RANK(int square) { 
	return (square >> 4);
	}

	private int  SQUARE_EP_DUAL(int square) { 
	return (square ^ 16);
	}

	private int  SQUARE_COLOUR(int square) { 
	return ( (square ^ (square >> 4)) & 1);
	}

	private int  SQUARE_FILE_MIRROR(int square) { 
	return (square ^ 0x0F);
	}

	private int  SQUARE_RANK_MIRROR(int square) { 
	return (square ^ 0xF0);
	}
/*
	private int  FILE_OPP(int file) { 
	return (file ^ 0xF);
	}
*/
	
	private int  RANK_OPP(int rank) { 
	return (rank ^ 0xF);
	}

	private int  PAWN_RANK(int square,int colour) { 
	return (SQUARE_RANK(square) ^ RankMask[colour]);
	}

	private int  PAWN_PROMOTE(int square,int colour) { 
	return (PromoteRank[colour] | (square & 0xF));
	}


	// end of square.h


	// board.h

	// constants

	final static int Empty = 0;       // const int
	final static int Edge = Knight64; // const int   HACK: uncoloured knight

	final static int WP = WhitePawn256;   // const int
	final static int WN = WhiteKnight256; // const int
	final static int WB = WhiteBishop256; // const int
	final static int WR = WhiteRook256;   // const int
	final static int WQ = WhiteQueen256;  // const int
	final static int WK = WhiteKing256;   // const int

	final static int BP = BlackPawn256;   // const int
	final static int BN = BlackKnight256; // const int
	final static int BB = BlackBishop256; // const int
	final static int BR  = BlackRook256;   // const int
	final static int BQ = BlackQueen256;  // const int
	final static int BK = BlackKing256;   // const int

	final static int FlagsNone = 0;   // const int
	final static int FlagsWhiteKingCastle = (1 << 0) ;   // const int
	final static int FlagsWhiteQueenCastle = (1 << 1) ;   // const int
	final static int FlagsBlackKingCastle = (1 << 2) ;   // const int
	final static int FlagsBlackQueenCastle = (1 << 3) ;   // const int


	// macros

	private int  KING_POS(board_t board,int colour) { 
	return board.piece[colour][0];
	}


	// end of board.h


	// move.h

	// constants

	final static int MoveNone = 0;  // const int   HACK: a1a1 cannot be a legal move
	final static int Movenull = 11; // const int   HACK: a1d2 cannot be a legal move

	final static int MoveNormal =  (0 << 14);   // const int
	final static int MoveCastle =  (1 << 14);   // const int
	final static int MovePromote =  (2 << 14);   // const int
	final static int MoveEnPassant =  (3 << 14);   // const int
	final static int MoveFlags =  (3 << 14);   // const int

	final static int MovePromoteKnight =  (MovePromote | (0 << 12));   // const int
	final static int MovePromoteBishop =  (MovePromote | (1 << 12));   // const int
	final static int MovePromoteRook =  (MovePromote | (2 << 12));   // const int
	final static int MovePromoteQueen =  (MovePromote | (3 << 12));   // const int

	final static int MoveAllFlags = (0xF << 12);   // const int

	private int[] PromotePiece = { Knight64, Bishop64, Rook64, Queen64 };   // int[4]

	// macros

	private int  MOVE_MAKE(int from, int to) { 
	return ( (this.SquareTo64[from] << 6) | this.SquareTo64[to]);
	}

	private int  MOVE_MAKE_FLAGS(int from, int to, int flags) {
	   return ( (this.SquareTo64[from] << 6) | (this.SquareTo64[to] | flags));
	}

	private int  MOVE_FROM(int move) { 
	return SquareFrom64[ ((move >> 6) & 63)];
	}

	private int  MOVE_TO(int move) { 
	return SquareFrom64[ (move & 63)];
	}

	private boolean  MOVE_IS_SPECIAL(int move) { 
	return ( (move & MoveFlags)!=MoveNormal );
	}

	private boolean  MOVE_IS_PROMOTE(int move) { 
	return ( (move & MoveFlags)==MovePromote );
	}

	private boolean  MOVE_IS_EN_PASSANT(int move) { 
	return ( (move & MoveFlags)==MoveEnPassant );
	}

	private boolean  MOVE_IS_CASTLE(int move) { 
	return ( (move & MoveFlags)==MoveCastle );
	}

	private int  MOVE_PIECE(int move,board_t board) { 
	return (board.square[MOVE_FROM(move)]);
	}


	// end of move.h



	// attack.h


	// variables

	private int[] DeltaIncLine = new int[DeltaNb];      // int[DeltaNb]
	private int[] DeltaIncAll = new int[DeltaNb];       // int[DeltaNb]

	private int[] DeltaMask = new int[DeltaNb];         // int[DeltaNb]
	private int[] IncMask = new int[IncNb];           // int[IncNb]

	private int[] PieceCode = new int[PieceNb];         // int[PieceNb]
	private int[][] PieceDeltaSize = new int[4][256];    // int[4][256]      4kB
	private int[][][] PieceDeltaDelta = new int[4][256][4];   // int[4][256][4]  16kB


	// macros

	private boolean  IS_IN_CHECK(board_t board, int colour) { 
	return is_attacked(board,KING_POS(board,colour),COLOUR_OPP(colour));
	}

	private int  DELTA_INC_LINE(int delta) { 
	return this.DeltaIncLine[DeltaOffset+delta];
	}

	private int  DELTA_INC_ALL(int delta) { 
	return this.DeltaIncAll[DeltaOffset+delta];
	}

	private int  DELTA_MASK(int delta) { 
	return this.DeltaMask[DeltaOffset+delta];
	}

	private int  INC_MASK(int inc) { 
	return this.IncMask[IncOffset+inc];
	}

	private boolean  PSEUDO_ATTACK(int piece, int delta) { 
	return ((piece & DELTA_MASK(delta))!=0);
	}

	private boolean  PIECE_ATTACK(board_t board, int piece, int from, int to) { 
	return PSEUDO_ATTACK(piece,to-from) && line_is_empty(board,from,to);
	}


	private boolean  SLIDER_ATTACK(int piece,int inc) { 
	return ((piece & INC_MASK(inc))!=0);
	}

	private boolean  ATTACK_IN_CHECK(attack_t attack) { 
	return (attack.dn!=0);
	}


	// end of attack.h


	// trans.h


	// variables

	private trans_t Trans  = new trans_t();      // trans_t [1]
	private trans_rtrv TransRv = new trans_rtrv();  // retriever

	// end of trans.h




	// hash.h

	// macros

	private int  uint32(int i) { 
	return (i & 0xFFFFFFFF);
	}

	private int  KEY_INDEX(int key) { 
	return uint32(key);
	}

	private int  KEY_LOCK(int key) {       // no 64 bits, so, we use the original key
	return key;                                       // uint32((key >> 32));
	}

	// constants

	final static int RandomPiece      =   0; // 12 * 64   const int
	final static int RandomCastle     = 768; // 4         const int
	final static int RandomEnPassant  = 772; // 8         const int
	final static int RandomTurn       = 780; // 1         const int


	// end of hash.h


	// list.h


	// macros

	private void LIST_ADD(list_t list,int mv) { 
	list.move[list.size]=mv;
	list.size++;
	}

	private void LIST_CLEAR(list_t list) {
	int i = list.size;	
	while(i>0)
		{
		list.move[i] = 0;
		list.value[i] = 0;
		i--;
		}
	list.size = 0;
	}


	// end of list.h


	// material.h

	// constants

	final static int MAT_NONE =0; final static int MAT_KK =1; final static int MAT_KBK =2;
	final static int MAT_KKB =3; final static int MAT_KNK =4; final static int MAT_KKN =5;
	final static int MAT_KPK =6; final static int MAT_KKP =7; final static int MAT_KQKQ =8;
	final static int MAT_KQKP =9; final static int MAT_KPKQ =10; final static int MAT_KRKR =11;
	final static int MAT_KRKP =12; final static int MAT_KPKR =13; final static int MAT_KBKB =14;
	final static int MAT_KBKP =15; final static int MAT_KPKB =16; final static int MAT_KBPK =17;
	final static int MAT_KKBP =18; final static int MAT_KNKN =19; final static int MAT_KNKP =20;
	final static int MAT_KPKN =21; final static int MAT_KNPK =22; final static int MAT_KKNP =23;
	final static int MAT_KRPKR =24; final static int MAT_KRKRP =25; final static int MAT_KBPKB =26;
	final static int MAT_KBKBP =27; final static int MAT_NB =28;

	final static int DrawNodeFlag    =  (1 << 0);  // const int
	final static int DrawBishopFlag  =  (1 << 1);  // const int
	final static int MatRookPawnFlag =  (1 << 0);  // const int
	final static int MatBishopFlag   =  (1 << 1);  // const int
	final static int MatKnightFlag   =  (1 << 2);  // const int
	final static int MatKingFlag     =  (1 << 3);  // const int


	// constants

	final static int PawnPhase   = 0;   // const int
	final static int KnightPhase = 1;   // const int
	final static int BishopPhase = 1;   // const int
	final static int RookPhase   = 2;   // const int
	final static int QueenPhase  = 4;   // const int
	final static int TotalPhase = (PawnPhase * 16) + (KnightPhase * 4) +
	(BishopPhase * 4) + RookPhase * 4 + (QueenPhase * 2);   // const int

	// constants and variables

	private int MaterialWeight = 256; // 100% const int

	final static int PawnOpening   = 80;    // was 100 const int
	final static int PawnEndgame   = 90;    // was 100 const int
	final static int KnightOpening = 325;   // const int
	final static int KnightEndgame = 325;   // const int
	final static int BishopOpening = 325;   // const int
	final static int BishopEndgame = 325;   // const int
	final static int RookOpening   = 500;   // const int
	final static int RookEndgame   = 500;   // const int
	final static int QueenOpening  = 1000;  // const int
	final static int QueenEndgame  = 1000;  // const int

	final static int BishopPairOpening = 50;   // const int
	final static int BishopPairEndgame = 50;   // const int


	// variables

	private material_t Material = new material_t();   // material_t[1]

	// material_info_copy ()

	private void material_info_copy (material_info_t dst, material_info_t src) { 

	dst.lock = src.lock;
	dst.recog = src.recog;

	dst.cflags[0] = src.cflags[0];
	dst.cflags[1] = src.cflags[1];

	dst.mul[0] = src.mul[0];
	dst.mul[1] = src.mul[1];

	dst.phase = src.phase;
	dst.opening = src.opening;
	dst.endgame = src.endgame;

	dst.flags = src.flags;

	}


	// end of material.h



	// move_do.h

	// variables

	private int[] CastleMask = new int[SquareNb];   // int[SquareNb]

	// end of move_do.h



	// pawn.h


	// constants

	final static int BackRankFlag =  (1 << 0);   // const int


	// pawn_info_copy ()

	private void pawn_info_copy ( pawn_info_t dst, pawn_info_t src ) { 
	dst.lock = src.lock;
	dst.opening = src.opening;
	dst.endgame = src.endgame;
	dst.flags[0] = src.flags[0];
	dst.flags[1] = src.flags[1];
	dst.passed_bits[0] = src.passed_bits[0];
	dst.passed_bits[1] = src.passed_bits[1];
	dst.single_file[0] = src.single_file[0];
	dst.single_file[1] = src.single_file[1];
	dst.pad  = src.pad ;
	}

	// constants and variables

	private pawn_t Pawn = new pawn_t();          // pawn_t[1]

	final static int doubledOpening = 10;       // const int
	final static int doubledEndgame = 20;       // const int

	final static int IsolatedOpening = 10;      // const int
	final static int IsolatedOpeningOpen = 20;  // const int
	final static int IsolatedEndgame = 20;      // const int

	final static int BackwardOpening = 8;       // const int
	final static int BackwardOpeningOpen = 16;  // const int
	final static int BackwardEndgame = 10;      // const int

	final static int CandidateOpeningMin = 5;   // const int
	final static int CandidateOpeningMax = 55;  // const int
	final static int CandidateEndgameMin = 10;  // const int
	final static int CandidateEndgameMax = 110; // const int

	private int[] Bonus = new int[RankNb];   // int[RankNb]

	// variables

	private int[] BitEQ = new int[16];   // int[16]
	private int[] BitLT = new int[16];   // int[16]
	private int[] BitLE = new int[16];   // int[16]
	private int[] BitGT = new int[16];   // int[16]
	private int[] BitGE = new int[16];   // int[16]

	private int[] BitFirst = new int[0x100];  // int[0x100]
	private int[] BitLast = new int[0x100];   // int[0x100]
	private int[] BitCount = new int[0x100];  // int[0x100]
	private int[] BitRev = new int[0x100];    // int[0x100]


	private int[] BitRank1 = new int[RankNb];  // int[RankNb]
	private int[] BitRank2 = new int[RankNb];  // int[RankNb]
	private int[] BitRank3 = new int[RankNb];  // int[RankNb]


	// end of pawn.h


	// pst.h

	// constants

	final static int Opening = 0;   // const int
	final static int Endgame = 1;   // const int
	final static int StageNb = 2;   // const int

	// constants

	final static int pA1=0; final static int pB1=1; final static int pC1=2;
	 final static int pD1=3; final static int pE1=4; final static int pF1=5;
	 final static int pG1=6; final static int pH1=7;  // const int
	final static int pA2=8; final static int pB2=9; final static int pC2=10;
	 final static int pD2=11; final static int pE2=12; final static int pF2=13;
	 final static int pG2=14; final static int pH2=15;  // const int
	final static int pA3=16; final static int pB3=17; final static int pC3=18;
	 final static int pD3=19; final static int pE3=20; final static int pF3=21;
	 final static int pG3=22; final static int pH3=23;  // const int
	final static int pA4=24; final static int pB4=25; final static int pC4=26;
	 final static int pD4=27; final static int pE4=28; final static int pF4=29;
	 final static int pG4=30; final static int pH4=31;  // const int
	final static int pA5=32; final static int pB5=33; final static int pC5=34;
	 final static int pD5=35; final static int pE5=36; final static int pF5=37;
	 final static int pG5=38; final static int pH5=39;  // const int
	final static int pA6=40; final static int pB6=41; final static int pC6=42;
	 final static int pD6=43; final static int pE6=44; final static int pF6=45;
	 final static int pG6=46; final static int pH6=47;  // const int
	final static int pA7=48; final static int pB7=49; final static int pC7=50;
	 final static int pD7=51; final static int pE7=52; final static int pF7=53;
	 final static int pG7=54; final static int pH7=55;  // const int
	final static int pA8=56; final static int pB8=57; final static int pC8=58;
	 final static int pD8=59; final static int pE8=60; final static int pF8=61;
	 final static int pG8=62; final static int pH8=63;  // const int

	// constants and variables

	private int PieceActivityWeight = 256; // 100%   const int
	private int KingSafetyWeight = 256;    // 100%  const int
	private int PassedPawnWeight = 256;    // 100%  const int
	private int PawnStructureWeight = 256; // 100%  const int

	final static int PawnFileOpening = 5;        // const int
	final static int KnightCentreOpening = 5;    // const int
	final static int KnightCentreEndgame = 5;    // const int
	final static int KnightRankOpening = 5;      // const int
	final static int KnightBackRankOpening = 0;  // const int
	final static int KnightTrapped = 100;        // const int
	final static int BishopCentreOpening = 2;    // const int
	final static int BishopCentreEndgame = 3;    // const int
	final static int BishopBackRankOpening = 10; // const int
	final static int BishopDiagonalOpening = 4;  // const int
	final static int RookFileOpening = 3;        // const int
	final static int QueenCentreOpening = 0;     // const int
	final static int QueenCentreEndgame = 4;     // const int
	final static int QueenBackRankOpening = 5;   // const int
	final static int KingCentreEndgame = 12;     // const int
	final static int KingFileOpening = 10;       // const int
	final static int KingRankOpening = 10;       // const int

	// "constants"

	final static int[] PawnFile = { -3, -1, 0, 1, 1, 0, -1, -3 };      // const int[8]

	final static int[] KnightLine = { -4, -2, 0, 1, 1, 0, -2, -4 };    // const int[8]

	final static int[] KnightRank = { -2, -1, 0, 1, 2, 3, 2, 1 };    // const int[8]

	final static int[] BishopLine = { -3, -1, 0, 1, 1, 0, -1, -3 };    // const int[8]

	final static int[] RookFile = { -2, -1, 0, 1, 1, 0, -1, -2 };      // const int[8]

	final static int[] QueenLine = { -3, -1, 0, 1, 1, 0, -1, -3 };     // const int[8]

	final static int[] KingLine = { -3, -1, 0, 1, 1, 0, -1, -3 };      // const int[8]

	final static int[] KingFile = { 3, 4, 2, 0, 0, 2, 4, 3 };      // const int[8]

	final static int[] KingRank = { 1, 0, -2, -3, -4, -5, -6, -7 };      // const int[8]

	// variables

	private int[][][] Pst = new int[12][64][StageNb];      // sint16 [12][64][StageNb]


	// end of pst.h


	// random.h

	// "constants"
	final static int RandomNb = 781;   // max size

	private int[] Random64 = new int[RandomNb];    // uint64[RandomNb]  array of const fixed randoms

	// end of random.h


	private boolean setjmp = false;        // c++ has setjmp-longjmp feature


	// variables

	public search_input_t SearchInput = new search_input_t();      // search_input_t[1]
	private search_info_t SearchInfo = new search_info_t();        // search_info_t[1]
	private search_root_t SearchRoot = new search_root_t();        // search_root_t[1]
	private search_current_t SearchCurrent = new search_current_t();  // search_current_t[1]
	private search_best_t SearchBest = new search_best_t();        // search_best_t[1]


	// search.h

	// constants and variables

	// main search

	final static boolean UseDistancePruning = true;   // const bool

	// transposition table

	final static boolean UseMateValues = true; // use mate values from shallower searches?   // const bool

	// null move

	private boolean Usenull = true;     // const bool
	private boolean UsenullEval = true; // const bool
	final static int nullDepth = 2;      // const int
	private int nullReduction = 3;  // const int

	private boolean UseVer = true;         // const bool
	private boolean UseVerEndgame = true;  // const bool
	private int VerReduction = 5;      // const int   was 3

	// move ordering

	final static boolean UseIID = true;      // const bool
	final static int IIDDepth = 3;       // const int
	final static int IIDReduction = 2;   // const int

	// extensions

	final static boolean ExtendSingleReply = true;   // const bool

	// history pruning

	private boolean UseHistory = true;       // const bool
	final static int HistoryDepth = 3;        // const int
	final static int HistoryMoveNb = 3;       // const int
	private int HistoryValue = 9830;     // const int 60%
	final static boolean HistoryReSearch = true;  // const bool

	// futility pruning

	private boolean UseFutility = false;     // const bool
	private int FutilityMargin = 100;    // const int

	// quiescence search

	private boolean UseDelta = false;        // const bool
	private int DeltaMargin = 50;        // const int

	private int CheckNb = 1;             // const int
	private int CheckDepth = 0;          // const int   1 - this.CheckNb

	// misc

	final static int NodeAll = -1;   // const int
	final static int NodePV  =  0;   // const int
	final static int NodeCut = 1;   // const int


	// end of search.h



	// fen.h

	// "constants"

	final static String StartFen = "rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w KQkq - 0 1";   // const char

	// variables

	final static boolean Strictq = false;   // const bool

	// end of fen.h



	// protocol.h

	// constants

	final static String VERSION = "Fruit 2.1 by Fabien Letouzey, port to java by Chessforeva";
	final static int NormalRatio = 100;   // const
	final static int PonderRatio = 100;   // const

	// variables

	private boolean Init = false;       // bool

	// end of protocol.h



	// sort.h


	// constants

	final static int KillerNb = 2;   // const int

	final static int HistorySize = 12 * 64;   // const int
	final static int HistoryMax = 16384;      // const int

	final static int TransScore  = 32766;   // const int
	final static int GoodScore =  4000;   // const int
	final static int KillerScore =     4;   // const int
	final static int HistoryScore = -24000;   // const int
	final static int BadScore = -28000;   // const int

	final static int CODE_SIZE = 256;         // const int


	final static int GEN_ERROR = 0;
	final static int GEN_LEGAL_EVASION = 1;
	final static int GEN_TRANS = 2;
	final static int GEN_GOOD_CAPTURE = 3;
	final static int GEN_BAD_CAPTURE = 4;
	final static int GEN_KILLER = 5;
	final static int GEN_QUIET = 6;
	final static int GEN_EVASION_QS = 7;
	final static int GEN_CAPTURE_QS = 8;
	final static int GEN_CHECK_QS = 9;
	final static int GEN_END = 10;

	final static int TEST_ERROR = 0;
	final static int TEST_NONE = 1;
	final static int TEST_LEGAL = 2;
	final static int TEST_TRANS_KILLER = 3;
	final static int TEST_GOOD_CAPTURE = 4;
	final static int TEST_BAD_CAPTURE = 5;
	final static int TEST_KILLER = 6;
	final static int TEST_QUIET = 7;
	final static int TEST_CAPTURE_QS = 8;
	final static int TEST_CHECK_QS = 9;


	// variables

	private int PosLegalEvasion = 0;   // int
	private int PosSEE = 0;            // int

	private int PosEvasionQS = 0;      // int
	private int PosCheckQS = 0;        // int
	private int PosCaptureQS = 0;      // int

	private int[] Code = new int[CODE_SIZE];             // int[CODE_SIZE]

	private int[][] Killer = new int[HeightMax][KillerNb];           // uint16[HeightMax][KillerNb]

	private int[] History = new int[HistorySize];          // uint16[HistorySize]
	private int[] HistHit = new int[HistorySize];          // uint16[HistorySize]
	private int[] HistTot = new int[HistorySize];          // uint16[HistorySize]



	// end of sort.h



	// value.h

	// variables

	private int[] ValuePiece = new int[PieceNb];   // int[PieceNb]

	// constants

	final static int ValuePawn = 100;   // was 100   const int
	final static int ValueKnight = 325;   // was 300   const int
	final static int ValueBishop = 325;   // was 300   const int
	final static int ValueRook = 500;   // was 500   const int
	final static int ValueQueen = 1000;  // was 900   const int
	final static int ValueKing = 10000; // was 10000 const int

	final static int ValueNone = -32767;          // const int
	final static int ValueDraw = 0;               // const int
	final static int ValueMate = 30000;           // const int
	final static int ValueInf = ValueMate;       // const int
	final static int ValueEvalInf = ValueMate - 256; // const int handle mates upto 255 plies


	// end of value.h


	// eval.h

	// constants and variables

	final static int KnightUnit = 4;   // const int
	final static int BishopUnit = 6;   // const int
	final static int RookUnit = 7;     // const int
	final static int QueenUnit = 13;   // const int

	final static int MobMove = 1;      // const int
	final static int MobAttack = 1;    // const int
	final static int MobDefense = 0;   // const int

	final static int KnightMobOpening  = 4; // const int
	final static int KnightMobEndgame = 4; // const int
	final static int BishopMobOpening = 5; // const int
	final static int BishopMobEndgame = 5; // const int
	final static int RookMobOpening  = 2;   // const int
	final static int RookMobEndgame = 4;   // const int
	final static int QueenMobOpening = 1;  // const int
	final static int QueenMobEndgame  = 2;  // const int
	final static int KingMobOpening = 0;   // const int
	final static int KingMobEndgame = 0;   // const int

	final static boolean UseOpenFile = true;   // const bool
	final static int RookSemiOpenFileOpening = 10;  // const int
	final static int RookSemiOpenFileEndgame = 10;  // const int
	final static int RookOpenFileOpening = 20;      // const int
	final static int RookOpenFileEndgame = 20;      // const int
	final static int RookSemiKingFileOpening = 10;  // const int
	final static int RookKingFileOpening = 20;      // const int

	final static boolean UseKingAttack = true;     // const bool
	final static int KingAttackOpening = 20;   // const int

	final static boolean UseShelter = true;    // const bool
	final static int ShelterOpening = 256; // 100%  const int
	final static boolean UseStorm = true;      // const bool
	final static int StormOpening = 10;    // const int

	final static int Rook7thOpening = 20;   // const int
	final static int Rook7thEndgame = 40;   // const int
	final static int Queen7thOpening = 10;  // const int
	final static int Queen7Endgame = 20;  // const int

	final static int TrappedBishop = 100;   // const int

	final static int BlockedBishop = 50;   // const int
	final static int BlockedRook = 50;     // const int

	final static int PassedOpeningMin = 10;   // const int
	final static int PassedOpeningMax = 70;   // const int
	final static int PassedEndgameMin = 20;   // const int
	final static int PassedEndgameMax = 140;  // const int

	final static int UnstoppablePasser = 800; // const int
	final static int FreePasser = 60;         // const int

	final static int AttackerDistance = 5;    // const int
	final static int DefenderDistance = 20;   // const int

	// "constants"

	final static int[] KingAttackWeight = { 0, 0, 128, 192, 224, 240, 248, 252, 254, 255, 256, 256 ,256, 256, 256, 256 };  // const int[16]

	// variables

	private int[][] MobUnit = new int[ColourNb][PieceNb];        // int[ColourNb][PieceNb]

	private int[] KingAttackUnit = new int[PieceNb];  // int[PieceNb]

	// macros
/*
	private boolean THROUGH(int piece) { 
	return (piece==Empty);
	}
*/
	// end of eval.h



	// hash.h

	// variables

	private int[] Castle64 = new int[16];   // int[16]

	// end of hash.h




	// vector.h

	// variables

	private int[] Distance = new int[DeltaNb];   // int[DeltaNb]

	// macros

	private int DISTANCE(int square_1, int square_2) { 
	return this.Distance[DeltaOffset+(square_2-square_1)];
	}

	// end of vector.h


	// option.h

	// types

	// variables
	
	private opt_t_def[] Option = new opt_t_def[21];

	private void set_opt_t_def( int k, String vary, boolean decl, String init, String type, String extra ) { 
	this.Option[k]= new opt_t_def();
	this.Option[k].vary = vary;        // string
	this.Option[k].decl = decl;        // bool
	this.Option[k].init = init;        // string
	this.Option[k].val  = init;        // string the same as init
	this.Option[k].type = type;        // string
	this.Option[k].extra = extra;      // string
	}
	// end of option.h


	//
	// Programs C
	//


	// attack.cpp

	// attack_init()

	private void attack_init() {  

	int delta = 0;   // int
	int inc = 0;     // int
	int piece = 0;   // int
	int dir = 0;     // int
	int dist = 0;    // int
	int size = 0;    // int
	int king = 0;    // int
	int from = 0;    // int
	int to = 0;      // int
	int pos = 0;     // int

	// pawn attacks

	this.DeltaMask[DeltaOffset-17] |= BlackPawnFlag;
	this.DeltaMask[DeltaOffset-15] |= BlackPawnFlag;

	this.DeltaMask[DeltaOffset+15] |= WhitePawnFlag;
	this.DeltaMask[DeltaOffset+17] |= WhitePawnFlag;

	// knight attacks

	for (dir = 0; dir <8 ; dir++) {

	delta = KnightInc[dir];
	//ASSERT(3, delta_is_ok(delta));

	//ASSERT(4, this.DeltaIncAll[DeltaOffset+delta]==IncNone);
	this.DeltaIncAll[DeltaOffset+delta] = delta;
	this.DeltaMask[DeltaOffset+delta] |= KnightFlag;
	}

	// bishop/queen attacks

	for (dir = 0; dir<=3 ; dir++ ) {

	inc = BishopInc[dir];
	//ASSERT(5, inc!=IncNone);

	this.IncMask[IncOffset+inc] |= BishopFlag;

	for (dist = 1; dist<8; dist++ ) {

	delta = inc*dist;
	//ASSERT(6, delta_is_ok(delta));

	//ASSERT(7, this.DeltaIncLine[DeltaOffset+delta]==IncNone);
	this.DeltaIncLine[DeltaOffset+delta] = inc;
	//ASSERT(8, this.DeltaIncAll[DeltaOffset+delta]==IncNone);
	this.DeltaIncAll[DeltaOffset+delta] = inc;
	this.DeltaMask[DeltaOffset+delta] |=  BishopFlag;
	}
	}

	// rook/queen attacks

	for (dir = 0; dir<4 ; dir++ ) {

	inc = RookInc[dir];
	//ASSERT(9, inc!=IncNone);

	this.IncMask[IncOffset+inc] |= RookFlag;

	for (dist = 1; dist<8 ; dist++ ) {

	delta = inc*dist;
	//ASSERT(10, delta_is_ok(delta));

	//ASSERT(11, this.DeltaIncLine[DeltaOffset+delta]==IncNone);
	this.DeltaIncLine[DeltaOffset+delta] = inc;
	//ASSERT(12, this.DeltaIncAll[DeltaOffset+delta]==IncNone);
	this.DeltaIncAll[DeltaOffset+delta] = inc;
	this.DeltaMask[DeltaOffset+delta] |= RookFlag;
	}
	}

	// king attacks

	for (dir = 0; dir<8 ; dir++ ) {

	delta = KingInc[dir];
	//ASSERT(13, delta_is_ok(delta));

	this.DeltaMask[DeltaOffset+delta] |= KingFlag;
	}

	// this.PieceCode[]

	for (piece = 0; piece< PieceNb; piece++ ) {
	this.PieceCode[piece] = -1;
	}

	this.PieceCode[WN] = 0;
	this.PieceCode[WB] = 1;
	this.PieceCode[WR] = 2;
	this.PieceCode[WQ] = 3;

	this.PieceCode[BN] = 0;
	this.PieceCode[BB] = 1;
	this.PieceCode[BR] = 2;
	this.PieceCode[BQ] = 3;

	// this.PieceDeltaSize[][] & this.PieceDeltaDelta[][][]


	//for (piece = 0; piece <=3; piece++ ) {

	//this.PieceDeltaSize[piece] = [];
	//this.PieceDeltaDelta[piece] = [];

	//for (delta = 0; delta<256; delta++) {

	//this.PieceDeltaDelta[piece][delta] = [];
	//this.PieceDeltaSize[piece][delta] = 0;

	//}
	//}


	for (king = 0; king<SquareNb; king++ ) {

	if (SQUARE_IS_OK(king)) {

	for (from = 0; from<SquareNb; from++ ) {

	if (SQUARE_IS_OK(from)) {

	// knight
	pos = 0;
	while (true) {
	inc=KnightInc[pos];
	if(inc == IncNone) {
	break;
	}
	to = from + inc;
	if (SQUARE_IS_OK(to)  &&  DISTANCE(to,king) == 1) {
	add_attack(0,king-from,to-from);
	}
	pos++;
	}

	// bishop
	pos = 0;
	while (true) {
	inc=BishopInc[pos];
	if(inc == IncNone) {
	break;
	}
	to = from+inc;
	while( SQUARE_IS_OK(to) ) {
	if (DISTANCE(to,king) == 1) {
	add_attack(1,king-from,to-from);
	break;
	}
	to +=  inc;
	}
	pos++;
	}

	// rook
	pos = 0;
	while (true) {
	inc=RookInc[pos];
	if(inc == IncNone) {
	break;
	}
	to = from+inc;
	while( SQUARE_IS_OK(to) ) {
	if (DISTANCE(to,king) == 1) {
	add_attack(2,king-from,to-from);
	break;
	}
	to +=  inc;
	}
	pos++;
	}

	// queen
	pos = 0;
	while (true) {
	inc=QueenInc[pos];
	if(inc == IncNone) {
	break;
	}
	to = from+inc;
	while( SQUARE_IS_OK(to) ) {
	if (DISTANCE(to,king) == 1) {
	add_attack(3,king-from,to-from);
	break;
	}
	to +=  inc;
	}
	pos++;
	}
	}
	}
	}


	for (piece = 0; piece<4; piece++ ) {
	for (delta = 0; delta< 256; delta++ ) {
	size = this.PieceDeltaSize[piece][delta];
	//ASSERT(14, size>=0 && size<3);
	this.PieceDeltaDelta[piece][delta][size] = DeltaNone;
	}
	}
	}

	}

	// add_attack()

	private void add_attack(int piece, int king, int target)   {

	int size = 0;   // int
	int i = 0;      // int

	//ASSERT(15, piece>=0 && piece<4);
	//ASSERT(16, delta_is_ok(king));
	//ASSERT(17, delta_is_ok(target));

	size = this.PieceDeltaSize[piece][DeltaOffset+king];
	//ASSERT(18, size>=0 && size<3);

	for (i = 0; i<size; i++ ) {
	if (this.PieceDeltaDelta[piece][DeltaOffset+king][i] == target) {
	return;    // already in the table
	}
	}

	if (size < 2)  {
	this.PieceDeltaDelta[piece][DeltaOffset+king][size] = target;
	size++;
	this.PieceDeltaSize[piece][DeltaOffset+king] = size;
	}
	}

	// is_attacked()

	private boolean is_attacked( board_t board, int to, int colour) {  // bool

	int inc = 0;    // int
	int pawn = 0;   // int
	int ptr = 0;    // int
	int from = 0;   // int
	int piece  = 0;  // int
	int delta = 0;  // int
	int sq = 0;     // int

	//ASSERT(20, SQUARE_IS_OK(to));
	//ASSERT(21, COLOUR_IS_OK(colour));

	// pawn attack

	inc = PawnMoveInc[colour];
	pawn = PawnMake[colour];

	if (board.square[to-(inc-1)] == pawn) {
	return true;
	}
	if (board.square[to-(inc+1)] == pawn) {
	return true;
	}

	// piece attack

	ptr = 0;
	while(true) {
	from = board.piece[colour][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];
	delta = to - from;

	if (PSEUDO_ATTACK(piece,delta)) {

	inc = DELTA_INC_ALL(delta);
	//ASSERT(22, inc!=IncNone);

	sq = from;
	while(true) {

	sq+=inc;
	if (sq == to) {
	return true;
	}
	if (board.square[sq] != Empty) {
	break;
	}
	}
	}
	ptr++;
	}

	return false;

	}

	// line_is_empty()

	private boolean line_is_empty( board_t board, int from, int to ) {

	int delta = 0;  // int
	int inc = 0;    // int
	int sq = 0;     // int

	//ASSERT(24, SQUARE_IS_OK(from));
	//ASSERT(25, SQUARE_IS_OK(to));

	delta = to - from;
	//ASSERT(26, delta_is_ok(delta));

	inc = DELTA_INC_ALL(delta);
	//ASSERT(27, inc!=IncNone);

	sq = from;
	while(true) {

	sq+=inc;
	if (sq == to) {
	return true;
	}
	if (board.square[sq] != Empty) {
	break;
	}
	}

	return false;  // blocker
	}

	// is_pinned()

	private boolean is_pinned( board_t board, int square, int colour) {

	int from = 0;  // int
	int to = 0;    // int
	int inc = 0;   // int
	int sq = 0;    // int
	int piece = 0; // int

	//ASSERT(29, SQUARE_IS_OK(square));
	//ASSERT(30, COLOUR_IS_OK(colour));

	from = square;
	to = KING_POS(board,colour);

	inc = DELTA_INC_LINE(to-from);
	if (inc == IncNone) {
	return false;  // not a line
	}

	sq = from;
	while(true) {
	sq+=inc;
	if (board.square[sq] != Empty) {
	break;
	}
	}

	if (sq != to) {
	return false; // blocker
	}

	sq = from;
	while(true) {
	sq-=inc;
	piece = board.square[sq];
	if ( piece!= Empty) {
	break;
	}
	}

	return COLOUR_IS(piece,COLOUR_OPP(colour)) && SLIDER_ATTACK(piece,inc);
	}

	// attack_is_ok()

	private boolean attack_is_ok( attack_t attack ) {

	int i = 0;   // int
	int sq = 0;  // int
	int inc = 0; // int

	// checks

	if (attack.dn < 0 || attack.dn > 2) {
	return false;
	}

	for (i = 0; i < attack.dn; i++ ) {
	sq = attack.ds[i];
	if (! SQUARE_IS_OK(sq)) {
	return false;
	}
	inc = attack.di[i];
	if (inc != IncNone  &&  (! inc_is_ok(inc))) {
	return false;
	}
	}

	if (attack.ds[attack.dn] != SquareNone) {
	return false;
	}
	if (attack.di[attack.dn] != IncNone) {
	return false;
	}

	return true;
	}

	// attack_set()

	private void attack_set( attack_t attack, board_t board )   {

	int me  = 0;    // int
	int opp  = 0;   // int
	int ptr  = 0;   // int
	int from  = 0;  // int
	int to  = 0;    // int
	int inc  = 0;   // int
	int pawn  = 0;  // int
	int delta  = 0; // int
	int piece  = 0; // int
	int sq  = 0;    // int
	boolean cont = false;


	// init

	attack.dn = 0;

	me = board.turn;
	opp = COLOUR_OPP(me);

	to = KING_POS(board,me);

	// pawn attacks

	inc = PawnMoveInc[opp];
	pawn = PawnMake[opp];

	from = to - (inc-1);
	if (board.square[from] == pawn) {
	attack.ds[attack.dn] = from;
	attack.di[attack.dn] = IncNone;
	attack.dn++;
	}

	from = to - (inc+1);
	if (board.square[from] == pawn) {
	attack.ds[attack.dn] = from;
	attack.di[attack.dn] = IncNone;
	attack.dn++;
	}

	// piece attacks

	ptr = 1;	// HACK: no king
	while(true) {
	from = board.piece[opp][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	delta = to - from;
	//ASSERT(33, delta_is_ok(delta));

	if (PSEUDO_ATTACK(piece,delta)) {

	inc = IncNone;

	if (PIECE_IS_SLIDER(piece)) {

	// check for (blockers

	inc = DELTA_INC_LINE(delta);
	//ASSERT(34, inc!=IncNone);

	sq = from;
	while(true) {
	sq+=inc;
	if (board.square[sq] != Empty) {
	break;
	}
	}

	if (sq != to) {
	cont = true;     // blocker => next attacker
	}
	}

	if(cont) {
	cont = false;
	} else { 
	attack.ds[attack.dn] = from;
	attack.di[attack.dn] = -inc; // HACK
	attack.dn++;
	}
	}
	ptr++;
	}

	attack.ds[attack.dn] = SquareNone;
	attack.di[attack.dn] = IncNone;

	// debug

	//ASSERT(35, attack_is_ok(attack));
	}

	// piece_attack_king()

	private boolean piece_attack_king( board_t board, int piece, int from, int king ) {

	int code  = 0;      // int
	int delta_ptr  = 0; // int
	int delta  = 0;     // int
	int inc  = 0;       // int
	int to  = 0;        // int
	int sq  = 0;        // int

	//ASSERT(37, piece_is_ok(piece));
	//ASSERT(38, SQUARE_IS_OK(from));
	//ASSERT(39, SQUARE_IS_OK(king));

	code = this.PieceCode[piece];
	//ASSERT(40, code>=0 && code<4);

	if (PIECE_IS_SLIDER(piece)) {

	delta_ptr = 0;
	while(true) {

	delta = this.PieceDeltaDelta[code][DeltaOffset+(king-from)][delta_ptr];
	if(delta==DeltaNone) {
	break;
	}

	//ASSERT(41, delta_is_ok(delta));

	inc = this.DeltaIncLine[DeltaOffset+delta];
	//ASSERT(42, inc!=IncNone);

	to = from + delta;

	sq = from;
	while(true) {
	sq+=inc;

	if (sq == to && SQUARE_IS_OK(to)) {
	//ASSERT(43, DISTANCE(to,king)==1);
	return true;
	}

	if (board.square[sq] != Empty) {
	break;
	}
	}

	delta_ptr++;
	}

	} else {  // non-slider

	delta_ptr = 0;
	while(true) {

	delta = this.PieceDeltaDelta[code][DeltaOffset+(king-from)][delta_ptr];
	if(delta==DeltaNone) {
	break;
	}

	//ASSERT(44, delta_is_ok(delta));

	to = from + delta;

	if (SQUARE_IS_OK(to)) {
	//ASSERT(45, DISTANCE(to,king)==1);
	return true;
	}

	delta_ptr++;
	}
	}

	return false;
	}

	// end of attack.cpp



	// board.cpp



	// board_is_ok()

	private boolean board_is_ok( board_t board ) { 

	int sq  = 0;     // int
	int piece  = 0;  // int
	int colour  = 0; // int
	int size  = 0;   // int
	int pos  = 0;    // int


	// squares

	for (sq = 0; sq<SquareNb; sq++ ) {

	piece = board.square[sq];
	pos = board.pos[sq];

	if (SQUARE_IS_OK(sq)) {

	// inside square

	if (piece == Empty) {

	if (pos != -1) {
	return false;
	}
	} else { 

	if (! piece_is_ok(piece)) {
	return false;
	}

	if (! PIECE_IS_PAWN(piece)) {
	colour = PIECE_COLOUR(piece);
	if (pos < 0  ||  pos >= board.piece_size[colour]) {
	return false;
	}
	if (board.piece[colour][pos] != sq) {
	return false;
	}
	} else {  // pawn
	if (this.SquareIsPromote[sq]) {
	return false;
	}
	colour = PIECE_COLOUR(piece);
	if (pos < 0  ||  pos >= board.pawn_size[colour]) {
	return false;
	}
	if (board.pawn[colour][pos] != sq) {
	return false;
	}
	}
	}

	} else { 

	// edge square

	if (piece != Edge) {
	return false;
	}
	if (pos != -1) {
	return false;
	}
	}
	}

	// piece lists

	for (colour = 0; colour<=1; colour++ ) {

	// piece list

	size = board.piece_size[colour];
	if (size < 1  ||  size > 16) {
	return false;
	}

	for (pos = 0; pos < size; pos++ ) {

	sq = board.piece[colour][pos];
	if (! SQUARE_IS_OK(sq)) {
	return false;
	}
	if (board.pos[sq] != pos) {
	return false;
	}
	piece = board.square[sq];
	if (! COLOUR_IS(piece,colour)) {
	return false;
	}
	if (pos == 0  &&  (! PIECE_IS_KING(piece))) {
	return false;
	}
	if (pos != 0  &&  PIECE_IS_KING(piece)) {
	return false;
	}
	if (pos != 0  &&  this.PieceOrder[piece] > this.PieceOrder[board.square[board.piece[colour][pos-1]]]) {
	return false;
	}
	}

	sq = board.piece[colour][size];
	if (sq != SquareNone) {
	return false;
	}

	// pawn list

	size = board.pawn_size[colour];
	if (size < 0  ||  size > 8) {
	return false;
	}

	for (pos = 0; pos < size; pos++ ) {

	sq = board.pawn[colour][pos];
	if (! SQUARE_IS_OK(sq)) {
	return false;
	}
	if (this.SquareIsPromote[sq]) {
	return false;
	}
	if (board.pos[sq] != pos) {
	return false;
	}
	piece = board.square[sq];
	if (! COLOUR_IS(piece,colour)) {
	return false;
	}
	if (! PIECE_IS_PAWN(piece)) {
	return false;
	}
	}

	sq = board.pawn[colour][size];
	if (sq != SquareNone) {
	return false;
	}

	// piece total

	if (board.piece_size[colour] + board.pawn_size[colour] > 16) {
	return false;
	}
	}

	// material

	if (board.piece_nb != board.piece_size[White] + board.pawn_size[White]
	+ board.piece_size[Black] + board.pawn_size[Black]) {
	return false;
	}

	if (board.number[WhitePawn12] != board.pawn_size[White]) {
	return false;
	}
	if (board.number[BlackPawn12] != board.pawn_size[Black]) {
	return false;
	}
	if (board.number[WhiteKing12] != 1) {
	return false;
	}
	if (board.number[BlackKing12] != 1) {
	return false;
	}

	// misc

	if (! COLOUR_IS_OK(board.turn)) {
	return false;
	}

	if (board.ply_nb < 0) {
	return false;
	}

	if (board.sp < board.ply_nb) {
	return false;
	}

	if (board.cap_sq != SquareNone  &&  (! SQUARE_IS_OK(board.cap_sq))) {
	return false;
	}

	if (board.opening != board_opening(board)) {
	return false;
	}
	if (board.endgame != board_endgame(board)) {
	return false;
	}

	// we can not guarantee that the key is the same, it is just a random number
	//
	//if (board.key != hash_key(board)) {
	//  return false;
	//}
	//if (board.pawn_key != hash_pawn_key(board)) {
	//  return false;
	//}
	//if (board.material_key != hash_material_key(board)) {
	//  return false;
	//}

	return true;
	}

	// board_clear()

	private void board_clear( board_t board )  {

	int sq  = 0;     // int
	int sq_64  = 0;  // int

	// edge squares

	for (sq = 0; sq<SquareNb; sq++ ) {
	board.square[sq] = Edge;
	}

	// empty squares

	for (sq_64 = 0; sq_64<=63; sq_64++ ) {
	sq = SquareFrom64[sq_64];
	board.square[sq] = Empty;
	}

	// misc

	board.turn = ColourNone;
	board.flags = FlagsNone;
	board.ep_square = SquareNone;
	board.ply_nb = 0;

	}

	// board_copy()

	private void board_copy( board_t dst, board_t src )  {

	int i  = 0;  // int

	//ASSERT(48, board_is_ok(src));

	dst.square = src.square;
	dst.pos = src.pos;
	//dst.piece = [];
	//dst.piece[0] = [];
	//dst.piece[1] = [];

	for (i = 0; i<src.piece[0].length; i++ ) {
	dst.piece[0][i] = src.piece[0][i];
	}
	for (i = 0; i<src.piece[1].length; i++ ) {
	dst.piece[1][i] = src.piece[1][i];
	}

	dst.piece_size = src.piece_size;

	for (i = 0; i<src.pawn[0].length; i++ ) {
	dst.pawn[0][i] = src.pawn[0][i];
	}
	for (i = 0; i<src.pawn[1].length; i++ ) {
	dst.pawn[1][i] = src.pawn[1][i];
	}

	for (i = 0; i<src.pawn_size.length; i++ ) {
	dst.pawn_size[i] = src.pawn_size[i];
	}

	dst.piece_nb = src.piece_nb;
	//dst.number = [];
	for (i = 0; i<src.number.length; i++ ) {
	dst.number[i] = src.number[i];
	}

	for (i = 0; i<src.pawn_file[0].length; i++ ) {
	dst.pawn_file[0][i] = src.pawn_file[0][i];
	}

	for (i = 0; i<src.pawn_file[1].length; i++ ) {
	dst.pawn_file[1][i] = src.pawn_file[1][i];
	}


	dst.turn = src.turn;
	dst.flags = src.flags;
	dst.ep_square = src.ep_square;
	dst.ply_nb = src.ply_nb;
	dst.sp = src.sp;

	dst.cap_sq = src.cap_sq;

	dst.opening = src.opening;
	dst.endgame = src.endgame;

	dst.key = src.key;
	dst.pawn_key = src.pawn_key;
	dst.material_key = src.material_key;

	//dst.stack = [];
	for (i = 0; i<src.stack.length; i++ ) {
	dst.stack[i] = src.stack[i];
	}

	}


	// board_init_list()

	private void board_init_list( board_t board )  {

	int sq_64  = 0;   // int
	int sq  = 0;      // int
	int piece  = 0;   // int
	int colour  = 0;  // int
	int pos  = 0;     // int
	int i  = 0;       // int
	int size  = 0;    // int
	int square  = 0;  // int
	int order  = 0;   // int
	int file  = 0;    // int

	// init

	for (sq = 0; sq<SquareNb; sq++ ) {
	board.pos[sq] = -1;
	}

	board.piece_nb = 0;
	for (piece = 0; piece<=11; piece++ ) {
	board.number[piece] = 0;
	}

	// piece lists

	for (colour = 0; colour<=1; colour++ ) {

	// piece list

	pos = 0;

	for (sq_64 = 0; sq_64<=63; sq_64++ ) {

	sq = SquareFrom64[sq_64];
	piece = board.square[sq];
	if (piece != Empty  &&  (! piece_is_ok(piece))) {
	my_fatal("board_init_list(): illegal position\n");
	}

	if (COLOUR_IS(piece,colour)  &&  (! PIECE_IS_PAWN(piece))) {

	if (pos >= 16) {
	my_fatal("board_init_list(): illegal position\n");
	}
	//ASSERT(50, pos>=0 && pos<16);

	board.pos[sq] = pos;
	board.piece[colour][pos] = sq;
	pos++;

	board.piece_nb++;
	board.number[this.PieceTo12[piece]]++;
	}
	}

	if ( board.number[ ( COLOUR_IS_WHITE(colour) ? WhiteKing12 : BlackKing12 ) ] != 1) {
	my_fatal("board_init_list(): illegal position\n");
	}

	//ASSERT(51, pos>=1 && pos<=16);
	board.piece[colour][pos] = SquareNone;
	board.piece_size[colour] = pos;

	// MV sort

	size = board.piece_size[colour];

	for (i = 1; i<size; i++ ) {

	square = board.piece[colour][i];
	piece = board.square[square];
	order = this.PieceOrder[piece];
	pos = i;
	while( pos > 0 ) {
	sq=board.piece[colour][pos-1];
	if( order <= this.PieceOrder[board.square[sq]] ) {
	break;
	}
	//ASSERT(52, pos>0 && pos<size);
	board.piece[colour][pos] = sq;
	//ASSERT(53, board.pos[sq]==pos-1);
	board.pos[sq] = pos;
	pos--;
	}

	//ASSERT(54, pos>=0 && pos<size);
	board.piece[colour][pos] = square;
	//ASSERT(55, board.pos[square]==i);
	board.pos[square] = pos;
	}

// --for debugging
	
//	if (true) {
//
//	for (i = 0; i<board.piece_size[colour]; i++ ) {
//
//	sq = board.piece[colour][i];
//	//ASSERT(56, board.pos[sq]==i);
//
//	if (i == 0) {  // king
//	//ASSERT(57, PIECE_IS_KING(board.square[sq]));
//	} else { 
//	//ASSERT(58, ! PIECE_IS_KING(board.square[sq]));
//	//ASSERT(59, this.PieceOrder[board.square[board.piece[colour][i]]] <= this.PieceOrder[board.square[board.piece[colour][i-1]]]);
//	}
//	}
//	}

	// pawn list

	for (file = 0; file< FileNb; file++ ) {
	board.pawn_file[colour][file] = 0;
	}

	pos = 0;

	for (sq_64 = 0; sq_64<=63; sq_64++ ) {

	sq = SquareFrom64[sq_64];
	piece = board.square[sq];

	if (COLOUR_IS(piece,colour)  &&  PIECE_IS_PAWN(piece)) {

	if (pos >= 8  ||  this.SquareIsPromote[sq]) {
	my_fatal("board_init_list(): illegal position\n");
	}
	//ASSERT(60, pos>=0 && pos<8);

	board.pos[sq] = pos;
	board.pawn[colour][pos] = sq;
	pos++;

	board.piece_nb++;
	board.number[this.PieceTo12[piece]]++;
	board.pawn_file[colour][SQUARE_FILE(sq)] =
	( board.pawn_file[colour][SQUARE_FILE(sq)] | this.BitEQ[PAWN_RANK(sq,colour)]);
	}
	}

	//ASSERT(61, pos>=0 && pos<=8);
	board.pawn[colour][pos] = SquareNone;
	board.pawn_size[colour] = pos;

	if (board.piece_size[colour] + board.pawn_size[colour] > 16) {
	my_fatal("board_init_list(): illegal position\n");
	}
	}

	// last square

	board.cap_sq = SquareNone;

	// PST

	board.opening = board_opening(board);
	board.endgame = board_endgame(board);

	// hash key

	for (i = 0; i<board.ply_nb; i++ ) {
	board.stack[i] = 0; // HACK
	}
	board.sp = board.ply_nb;

	board.key = hash_key(board);
	board.pawn_key = hash_pawn_key(board);
	board.material_key = hash_material_key(board);

	// legality

	if (! board_is_legal(board)) {
	my_fatal("board_init_list(): illegal position\n");
	}

	// debug

	//ASSERT(62, board_is_ok(board));
	}

	// board_is_legal()

	private boolean board_is_legal( board_t board ) { 

	return (! IS_IN_CHECK(board,COLOUR_OPP(board.turn)));
	}

	// board_is_check()

	public boolean board_is_check( board_t board ) { 

	return IS_IN_CHECK(board,board.turn);
	}

	// board_is_mate()

	public boolean board_is_mate( board_t board ) { 

	attack_t attack = new attack_t();   // attack_t[1]

	attack_set(attack,board);

	if (! ATTACK_IN_CHECK(attack)) {
	return false; // not in check => not mate
	}

	if (legal_evasion_exist(board,attack)) {
	return false; // legal move => not mate
	}

	return true; // in check && no legal move => mate
	}

	// board_is_stalemate()

	public boolean board_is_stalemate( board_t board ) {

	list_t list = new list_t();   // list_t[1];
	int i  = 0;      // int
	int move  = 0;   // int

	// init

	if (IS_IN_CHECK(board,board.turn)) {
	return false; // in check => not stalemate
	}

	// move loop

	gen_moves(list,board);

	for (i = 0; i<list.size; i++ ) {
	move = list.move[i];
	if (pseudo_is_legal(move,board)) {
	return false; // legal move => not stalemate
	}
	}

	return true; // in check && no legal move => mate
	}

	// board_is_repetition()

	private boolean board_is_repetition( board_t board ) {

	int i  = 0;   // int

	// 50-move rule

	if (board.ply_nb >= 100) { // potential draw

	if (board.ply_nb > 100) {
	return true;
	}

	//ASSERT(68, board.ply_nb==100);
	return (! board_is_mate(board));
	}

	// position repetition

	//ASSERT(69, board.sp>=board.ply_nb);

	for (i = 4; i< board.ply_nb-1; i+=2 ) {
	if (board.stack[board.sp-i] == board.key) {
	return true;
	}
	}

	return false;
	}

	// board_opening()

	private int board_opening( board_t board )  {

	int opening  = 0;   // int
	int colour  = 0;    // int
	int ptr  = 0;       // int
	int sq  = 0;        // int
	int piece  = 0;     // int

	opening = 0;
	for (colour = 0;  colour<=1; colour++ ) {

	ptr = 0;
	while(true) {
	sq = board.piece[colour][ptr];
	if(sq==SquareNone) {
	break;
	}
	piece = board.square[sq];
	opening += Pget( this.PieceTo12[piece], this.SquareTo64[sq], Opening );
	ptr++;
	}

	ptr = 0;
	while(true) {
	sq = board.pawn[colour][ptr];
	if(sq==SquareNone) {
	break;
	}
	piece = board.square[sq];
	opening += Pget( this.PieceTo12[piece], this.SquareTo64[sq], Opening );
	ptr++;
	}

	}

	return opening;
	}

	// board_endgame()

	private int board_endgame( board_t board )  {

	int endgame  = 0;   // int
	int colour  = 0;    // int
	int ptr  = 0;       // int
	int sq  = 0;        // int
	int piece  = 0;     // int

	endgame = 0;
	for (colour = 0;  colour<=1; colour++ ) {

	ptr = 0;
	while(true) {
	sq = board.piece[colour][ptr];
	if(sq==SquareNone) {
	break;
	}
	piece = board.square[sq];
	endgame += Pget( this.PieceTo12[piece], this.SquareTo64[sq], Endgame );
	ptr++;
	}

	ptr = 0;
	while(true) {
	sq = board.pawn[colour][ptr];
	if(sq==SquareNone) {
	break;
	}
	piece = board.square[sq];
	endgame += Pget( this.PieceTo12[piece], this.SquareTo64[sq], Endgame );
	ptr++;
	}

	}

	return endgame;
	}

	// end of board.cpp




	// eval.cpp



	// eval_init()

	private void eval_init() { 

	int colour  = 0;   // int
	int piece  = 0;    // int

	// UCI options

	this.PieceActivityWeight = (option_get_int("Piece Activity") * 256 + 50) / 100;
	this.KingSafetyWeight    = (option_get_int("King Safety")    * 256 + 50) / 100;
	this.PassedPawnWeight    = (option_get_int("Passed Pawns")   * 256 + 50) / 100;

	// mobility table

	for (colour = 0;  colour<=1; colour++ ) {
	//this.MobUnit[colour] = [];
	for (piece = 0; piece<PieceNb; piece++ ) {
	this.MobUnit[colour][piece] = 0;
	}
	}

	this.MobUnit[White][Empty] = MobMove;

	this.MobUnit[White][BP] = MobAttack;
	this.MobUnit[White][BN] = MobAttack;
	this.MobUnit[White][BB] = MobAttack;
	this.MobUnit[White][BR] = MobAttack;
	this.MobUnit[White][BQ] = MobAttack;
	this.MobUnit[White][BK] = MobAttack;

	this.MobUnit[White][WP] = MobDefense;
	this.MobUnit[White][WN] = MobDefense;
	this.MobUnit[White][WB] = MobDefense;
	this.MobUnit[White][WR] = MobDefense;
	this.MobUnit[White][WQ] = MobDefense;
	this.MobUnit[White][WK] = MobDefense;

	this.MobUnit[Black][Empty] = MobMove;

	this.MobUnit[Black][WP] = MobAttack;
	this.MobUnit[Black][WN] = MobAttack;
	this.MobUnit[Black][WB] = MobAttack;
	this.MobUnit[Black][WR] = MobAttack;
	this.MobUnit[Black][WQ] = MobAttack;
	this.MobUnit[Black][WK] = MobAttack;

	this.MobUnit[Black][BP] = MobDefense;
	this.MobUnit[Black][BN] = MobDefense;
	this.MobUnit[Black][BB] = MobDefense;
	this.MobUnit[Black][BR] = MobDefense;
	this.MobUnit[Black][BQ] = MobDefense;
	this.MobUnit[Black][BK] = MobDefense;

	// KingAttackUnit[]

	for (piece = 0; piece<PieceNb; piece++ ) {
	this.KingAttackUnit[piece] = 0;
	}

	this.KingAttackUnit[WN] = 1;
	this.KingAttackUnit[WB] = 1;
	this.KingAttackUnit[WR] = 2;
	this.KingAttackUnit[WQ] = 4;

	this.KingAttackUnit[BN] = 1;
	this.KingAttackUnit[BB] = 1;
	this.KingAttackUnit[BR] = 2;
	this.KingAttackUnit[BQ] = 4;
	}

	// evalpos()

	private int evalpos( board_t board )  {

	opening_t opening = new opening_t();   // int
	endgame_t endgame = new endgame_t();   // int
	material_info_t mat_info = new material_info_t();  // material_info_t[1]
	pawn_info_t pawn_info = new pawn_info_t();     // pawn_info_t[1]
	int[] mul = { 0, 0 };   // int[ColourNb]
	int phase  = 0;  // int
	int eval1  = 0;   // int
	int wb  = 0;     // int
	int bb  = 0;     // int

	//ASSERT(85, board_is_legal(board));
	//ASSERT(86, ! board_is_check(board)); // exceptions are extremely rare

	// material

	material_get_info(mat_info,board);

	opening.v +=  mat_info.opening;
	endgame.v +=  mat_info.endgame;

	mul[White] = mat_info.mul[White];
	mul[Black] = mat_info.mul[Black];

	// PST

	opening.v +=  board.opening;
	endgame.v +=  board.endgame;

	// pawns

	pawn_get_info(pawn_info,board);

	opening.v +=  pawn_info.opening;
	endgame.v +=  pawn_info.endgame;

	// draw

	eval_draw(board,mat_info,pawn_info,mul);

	if (mat_info.mul[White] < mul[White]) {
	mul[White] = mat_info.mul[White];
	}
	if (mat_info.mul[Black] < mul[Black]) {
	mul[Black] = mat_info.mul[Black];
	}

	if (mul[White] == 0  &&  mul[Black] == 0) {
	return ValueDraw;
	}

	// eval

	eval_piece(board,mat_info,pawn_info,opening,endgame);
	eval_king(board,mat_info,opening,endgame);
	eval_passer(board,pawn_info,opening,endgame);
	eval_pattern(board,opening,endgame);

	// phase mix

	phase = mat_info.phase;
	eval1 = ((opening.v * (256 - phase)) + (endgame.v * phase)) / 256;

	// drawish bishop endgames

	if ( ( mat_info.flags & DrawBishopFlag ) != 0) {

	wb = board.piece[White][1];
	//ASSERT(87, PIECE_IS_BISHOP(board.square[wb]));

	bb = board.piece[Black][1];
	//ASSERT(88, PIECE_IS_BISHOP(board.square[bb]));

	if (SQUARE_COLOUR(wb) != SQUARE_COLOUR(bb)) {
	if (mul[White] == 16) {
	mul[White] = 8; // 1/2
	}
	if (mul[Black] == 16) {
	mul[Black] = 8; // 1/2
	}
	}
	}

	// draw bound

	if (eval1 > ValueDraw) {
	eval1 = (eval1 * mul[White]) / 16;
	} else { 
	if (eval1 < ValueDraw) {
	eval1 = (eval1 * mul[Black]) / 16;
	}
	}

	// value range

	if (eval1 < -ValueEvalInf) {
	eval1 = -ValueEvalInf;
	}
	if (eval1 > ValueEvalInf) {
	eval1 = ValueEvalInf;
	}

	//ASSERT(89, eval1>=-ValueEvalInf && eval1<=ValueEvalInf);

	// turn

	if (COLOUR_IS_BLACK(board.turn)) {
	eval1 = -eval1;
	}

	//ASSERT(90, ! value_is_mate(eval1));

	return eval1;
	}

	// eval_draw()

	private void eval_draw( board_t board, material_info_t mat_info, pawn_info_t pawn_info, int[] mul )  { 

	int colour  = 0;    // int
	int me  = 0;        // int
	int opp  = 0;       // int
	int pawn  = 0;      // int
	int king  = 0;      // int
	int pawn_file  = 0; // int
	int prom  = 0;      // int
	int[] list = new int[8];     // int list[7+1]
	boolean ifelse = false;

	// draw patterns

	for (colour = 0;  colour<=1; colour++ ) {

	me = colour;
	opp = COLOUR_OPP(me);

	// KB*P+K* draw

	if ( ( mat_info.cflags[me] & MatRookPawnFlag ) != 0 ) {

	pawn = pawn_info.single_file[me];

	if (pawn != SquareNone) {   // all pawns on one file

	pawn_file = SQUARE_FILE(pawn);

	if (pawn_file == FileA  ||  pawn_file == FileH) {

	king = KING_POS(board,opp);
	prom = PAWN_PROMOTE(pawn,me);

	if (DISTANCE(king,prom) <= 1  && ( ! bishop_can_attack(board,prom,me))) {
	mul[me] = 0;
	}
	}
	}
	}

	// K(B)P+K+ draw

	if ( ( mat_info.cflags[me] & MatBishopFlag ) != 0) {

	pawn = pawn_info.single_file[me];

	if (pawn != SquareNone) {   // all pawns on one file

	king = KING_POS(board,opp);

	if (SQUARE_FILE(king)  == SQUARE_FILE(pawn)
	&& PAWN_RANK(king,me) >  PAWN_RANK(pawn,me)
	&& (! bishop_can_attack(board,king,me))) {
	mul[me] = 1;  // 1/16
	}
	}
	}

	// KNPK* draw

	if ( ( mat_info.cflags[me] & MatKnightFlag ) != 0 ) {

	pawn = board.pawn[me][0];
	king = KING_POS(board,opp);

	if (SQUARE_FILE(king)  == SQUARE_FILE(pawn)
	&& PAWN_RANK(king,me) >  PAWN_RANK(pawn,me)
	&& PAWN_RANK(pawn,me) <= Rank6) {
	mul[me] = 1;  // 1/16
	}
	}
	}

	// recognisers, only heuristic draws herenot

	ifelse = true;

	if (ifelse && mat_info.recog == MAT_KPKQ) {

	// KPKQ (white)

	draw_init_list(list,board,White);

	if (draw_kpkq(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KQKP) {

	// KPKQ (black)

	draw_init_list(list,board,Black);

	if (draw_kpkq(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KPKR) {

	// KPKR (white)

	draw_init_list(list,board,White);

	if (draw_kpkr(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KRKP) {

	// KPKR (black)

	draw_init_list(list,board,Black);

	if (draw_kpkr(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KPKB) {

	// KPKB (white)

	draw_init_list(list,board,White);

	if (draw_kpkb(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KBKP) {

	// KPKB (black)

	draw_init_list(list,board,Black);

	if (draw_kpkb(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KPKN) {

	// KPKN (white)

	draw_init_list(list,board,White);

	if (draw_kpkn(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KNKP) {

	// KPKN (black)

	draw_init_list(list,board,Black);

	if (draw_kpkn(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KNPK) {

	// KNPK (white)

	draw_init_list(list,board,White);

	if (draw_knpk(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KKNP) {

	// KNPK (black)

	draw_init_list(list,board,Black);

	if (draw_knpk(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KRPKR) {

	// KRPKR (white)

	draw_init_list(list,board,White);

	if (draw_krpkr(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KRKRP) {

	// KRPKR (black)

	draw_init_list(list,board,Black);

	if (draw_krpkr(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KBPKB) {

	// KBPKB (white)

	draw_init_list(list,board,White);

	if (draw_kbpkb(list,board.turn)) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KBKBP) {

	// KBPKB (black)

	draw_init_list(list,board,Black);

	if (draw_kbpkb(list,COLOUR_OPP(board.turn))) {
	mul[White] = 1; // 1/16;
	mul[Black] = 1; // 1/16;
	}

	ifelse = false;
	}
	}

	//
	private int add_line( board_t board, int me, int from, int dx )  { 

	int to = from + dx;
	int capture = 0;
	int mob = 0;

	while(true) {
	capture=board.square[to];
	if(capture!=Empty) {
	break;
	}
	mob +=  MobMove;
	to +=  dx;
	}

	mob +=  this.MobUnit[me][capture];

	return mob;
	}


	// eval_piece()

	private void eval_piece( board_t board, material_info_t mat_info, pawn_info_t pawn_info, opening_t opening, endgame_t endgame )  {

	int colour  = 0;   // int
	int[] op = { 0, 0 };      // int[ColourNb]
	int[] eg = { 0, 0 };      // int[ColourNb]
	int me  = 0;       // int
	int opp  = 0;      // int
	//int opp_flag  = 0; // int
	int ptr  = 0;      // int
	int from  = 0;     // int
	//int to  = 0;       // int
	int piece  = 0;    // int
	int mob  = 0;      // int
	//int capture  = 0;  // int
	int[] unit = null;          // int *
	int rook_file  = 0;  // int
	int king_file  = 0;  // int
	int king  = 0;     // int
	int delta  = 0;    // int
	int ptype = 0;

	// eval

	for (colour = 0;  colour<=1; colour++ ) {

	me = colour;
	opp = COLOUR_OPP(me);

	//opp_flag = COLOUR_FLAG(opp);

	unit = this.MobUnit[me];

	// piece loop

	ptr = 1;            // HACK: no king
	while(true) {
	from = board.piece[me][ptr];
	if(from==SquareNone) {
	break;
	}

	piece = board.square[from];

	ptype = PIECE_TYPE(piece);

	if(ptype == Knight64) {

	// mobility

	mob = -KnightUnit;

	mob +=  unit[board.square[from-33]];
	mob +=  unit[board.square[from-31]];
	mob +=  unit[board.square[from-18]];
	mob +=  unit[board.square[from-14]];
	mob +=  unit[board.square[from+14]];
	mob +=  unit[board.square[from+18]];
	mob +=  unit[board.square[from+31]];
	mob +=  unit[board.square[from+33]];

	op[me] +=  (mob * KnightMobOpening);
	eg[me] +=  (mob * KnightMobEndgame);

	}

	if(ptype == Bishop64) {

	// mobility

	mob = -BishopUnit;

	mob +=  add_line( board, me, from, -17 );
	mob +=  add_line( board, me, from, -15 );
	mob +=  add_line( board, me, from, 15 );
	mob +=  add_line( board, me, from, 17 );

	op[me] +=  (mob * BishopMobOpening);
	eg[me] +=  (mob * BishopMobEndgame);

	}

	if(ptype == Rook64) {

	// mobility

	mob = -RookUnit;

	mob +=  add_line( board, me, from, -16 );
	mob +=  add_line( board, me, from, -1 );
	mob +=  add_line( board, me, from, 1 );
	mob +=  add_line( board, me, from, 16 );

	op[me] +=  (mob * RookMobOpening);
	eg[me] +=  (mob * RookMobEndgame);

	// open file

	if (UseOpenFile) {

	op[me] -=  (RookOpenFileOpening / 2);
	eg[me] -=  (RookOpenFileEndgame / 2);


	rook_file = SQUARE_FILE(from);

	if (board.pawn_file[me][rook_file] == 0) {   // no friendly pawn

	op[me] +=  RookSemiOpenFileOpening;
	eg[me] +=  RookSemiOpenFileEndgame;

	if (board.pawn_file[opp][rook_file] == 0) {  // no enemy pawn

	op[me] +=  RookOpenFileOpening - RookSemiOpenFileOpening;
	eg[me] +=  RookOpenFileEndgame - RookSemiOpenFileEndgame;

	}

	if ( ( mat_info.cflags[opp] & MatKingFlag ) != 0) {

	king = KING_POS(board,opp);
	king_file = SQUARE_FILE(king);

	delta = Math.abs(rook_file-king_file); // file distance

	if (delta <= 1) {
	op[me] +=  RookSemiKingFileOpening;
	if (delta == 0) {
	op[me] +=  RookKingFileOpening - RookSemiKingFileOpening;
	}
	}
	}
	}
	}

	// 7th rank

	if (PAWN_RANK(from,me) == Rank7) {
	// if opponent pawn on 7th rank+.
	if ( ( pawn_info.flags[opp] & BackRankFlag ) != 0 ||
	PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
	op[me] +=  Rook7thOpening;
	eg[me] +=  Rook7thEndgame;
	}
	}

	}

	if(ptype == Queen64) {

	// mobility

	mob = -QueenUnit;

	mob +=  add_line( board, me, from, -17 );
	mob +=  add_line( board, me, from, -16 );
	mob +=  add_line( board, me, from, -15 );
	mob +=  add_line( board, me, from, -1 );
	mob +=  add_line( board, me, from, 1 );
	mob +=  add_line( board, me, from, 15 );
	mob +=  add_line( board, me, from, 16 );
	mob +=  add_line( board, me, from, 17 );


	op[me] +=  (mob * QueenMobOpening);
	eg[me] +=  (mob * QueenMobEndgame);

	// 7th rank

	if (PAWN_RANK(from,me) == Rank7) {
	// if opponent pawn on 7th rank+.
	if ( ( pawn_info.flags[opp] & BackRankFlag ) != 0 ||
	PAWN_RANK(KING_POS(board,opp),me) == Rank8) {
	op[me] +=  Queen7thOpening;
	eg[me] +=  Queen7Endgame;
	}
	}

	}

	ptr++;
	}
	}

	// update

	opening.v +=  ((op[White] - op[Black]) * this.PieceActivityWeight) / 256;
	endgame.v +=  ((eg[White] - eg[Black]) * this.PieceActivityWeight) / 256;
	}

	// eval_king()

	private void eval_king( board_t board, material_info_t mat_info, opening_t opening, endgame_t endgame )  {

	int colour  = 0;   // int
	int[] op = { 0, 0 }; // int[ColourNb]
	int[] eg = { 0, 0 }; // int[ColourNb]
	int me  = 0;       // int
	int opp  = 0;      // int
	int from  = 0;     // int

	int penalty_1  = 0;   // int
	int penalty_2  = 0;   // int

	int tmp  = 0;       // int
	int penalty  = 0;   // int

	int king  = 0;       // int
	int ptr  = 0;        // int
	int piece  = 0;      // int
	int attack_tot  = 0; // int
	int piece_nb  = 0;   // int

	// king attacks

	if (UseKingAttack) {

	for (colour = 0;  colour<=1; colour++ ) {

	if ( ( mat_info.cflags[colour] & MatKingFlag ) != 0) {

	me = colour;
	opp = COLOUR_OPP(me);

	king = KING_POS(board,me);

	// piece attacks

	attack_tot = 0;
	piece_nb = 0;

	ptr = 1;        // HACK: no king
	while(true) {
	from=board.piece[opp][ptr];
	if(from==SquareNone) {
	break;
	}

	piece = board.square[from];

	if (piece_attack_king(board,piece,from,king)) {
	piece_nb++;
	attack_tot = attack_tot + this.KingAttackUnit[piece];
	}
	ptr++;
	}

	// scoring

	//ASSERT(104, piece_nb>=0 && piece_nb<16);
	op[colour] -= (attack_tot * KingAttackOpening * KingAttackWeight[piece_nb]) / 256;
	}
	}
	}

	// white pawn shelter

	if (UseShelter  &&  ( mat_info.cflags[White] & MatKingFlag ) != 0) {

	me = White;

	// king

	penalty_1 = shelter_square(board,KING_POS(board,me),me);

	// castling

	penalty_2 = penalty_1;

	if ( ( board.flags & FlagsWhiteKingCastle ) != 0) {
	tmp = shelter_square(board,G1,me);
	if (tmp < penalty_2) {
	penalty_2 = tmp;
	}
	}

	if ( ( board.flags & FlagsWhiteQueenCastle ) != 0) {
	tmp = shelter_square(board,B1,me);
	if (tmp < penalty_2) {
	penalty_2 = tmp;
	}
	}

	//ASSERT(105, penalty_2>=0 && penalty_2<=penalty_1);

	// penalty

	penalty = (penalty_1 + penalty_2) / 2;
	//ASSERT(106, penalty>=0);

	op[me] -=  (penalty * ShelterOpening) / 256;
	}

	// black pawn shelter

	if (UseShelter  &&  ( mat_info.cflags[Black] & MatKingFlag ) != 0) {

	me = Black;

	// king

	penalty_1 = shelter_square(board,KING_POS(board,me),me);

	// castling

	penalty_2 = penalty_1;

	if ( ( board.flags & FlagsBlackKingCastle ) != 0) {
	tmp = shelter_square(board,G8,me);
	if (tmp < penalty_2) {
	penalty_2 = tmp;
	}
	}

	if ( ( board.flags & FlagsBlackQueenCastle ) != 0) {
	tmp = shelter_square(board,B8,me);
	if (tmp < penalty_2) {
	penalty_2 = tmp;
	}
	}

	//ASSERT(107, penalty_2>=0 && penalty_2<=penalty_1);

	// penalty

	penalty = (penalty_1 + penalty_2) / 2;
	//ASSERT(108, penalty>=0);

	op[me] -=  (penalty * ShelterOpening) / 256;
	}

	// update

	opening.v +=  ((op[White] - op[Black]) * this.KingSafetyWeight) / 256;
	endgame.v +=  ((eg[White] - eg[Black]) * this.KingSafetyWeight) / 256;
	}

	// eval_passer()

	private void eval_passer( board_t board, pawn_info_t pawn_info, opening_t opening, endgame_t endgame )  {


	int colour  = 0;   // int
	int[] op  = { 0, 0 }; // int[ColourNb]
	int[] eg  = { 0, 0 }; // int[ColourNb]
	int att  = 0;      // int
	int def  = 0;      // int
	int bits  = 0;     // int
	int file  = 0;     // int
	int rank  = 0;     // int
	int sq  = 0;       // int
	int min  = 0;      // int
	int max  = 0;      // int
	int delta  = 0;    // int

	// passed pawns

	for (colour = 0;  colour<=1; colour++ ) {

	att = colour;
	def = COLOUR_OPP(att);
	bits = pawn_info.passed_bits[att];
	while(true) {
	if(bits == 0) {
	break;
	}

	file = this.BitFirst[bits];
	//ASSERT(113, file>=FileA && file<=FileH);

	rank = this.BitLast[board.pawn_file[att][file] ];
	//ASSERT(114, rank>=Rank2 && rank<=Rank7);

	sq = SQUARE_MAKE(file,rank);
	if (COLOUR_IS_BLACK(att)) {
	sq = SQUARE_RANK_MIRROR(sq);
	}

	//ASSERT(115, PIECE_IS_PAWN(board.square[sq]));
	//ASSERT(116, COLOUR_IS(board.square[sq],att));

	// opening scoring

	op[att] += quad(PassedOpeningMin,PassedOpeningMax,rank);

	// endgame scoring init

	min = PassedEndgameMin;
	max = PassedEndgameMax;

	delta = max - min;
	//ASSERT(117, delta>0);

	// "dangerous" bonus

	// defender has no piece
	if (board.piece_size[def] <= 1
	&&  (unstoppable_passer(board,sq,att)  ||  king_passer(board,sq,att))) {
	delta +=  UnstoppablePasser;
	} else { 
	if (free_passer(board,sq,att)) {
	delta +=  FreePasser;
	}
	}

	// king-distance bonus

	delta -=  (pawn_att_dist(sq,KING_POS(board,att),att) * AttackerDistance);
	delta +=  (pawn_def_dist(sq,KING_POS(board,def),att) * DefenderDistance);

	// endgame scoring

	eg[att] +=  min;
	if (delta > 0) {
	eg[att] +=  quad(0,delta,rank);
	}

	bits = (bits & bits-1);
	}
	}

	// update

	opening.v +=  ((op[White] - op[Black]) * this.PassedPawnWeight) / 256;
	endgame.v +=  ((eg[White] - eg[Black]) * this.PassedPawnWeight) / 256;
	}

	// eval_pattern()

	private void eval_pattern( board_t board, opening_t opening, endgame_t endgame )  {

	// trapped bishop (7th rank)

	if ((board.square[A7] == WB  &&  board.square[B6] == BP)
	||  (board.square[B8] == WB  &&  board.square[C7] == BP)) {
	opening.v -=  TrappedBishop;
	endgame.v -=  TrappedBishop;
	}

	if ((board.square[H7] == WB  &&  board.square[G6] == BP)
	||  (board.square[G8] == WB  &&  board.square[F7] == BP)) {
	opening.v -=  TrappedBishop;
	endgame.v -=  TrappedBishop;
	}

	if ((board.square[A2] == BB  &&  board.square[B3] == WP)
	||  (board.square[B1] == BB  &&  board.square[C2] == WP)) {
	opening.v +=  TrappedBishop;
	endgame.v +=  TrappedBishop;
	}

	if ((board.square[H2] == BB  &&  board.square[G3] == WP)
	||  (board.square[G1] == BB  &&  board.square[F2] == WP)) {
	opening.v +=  TrappedBishop;
	endgame.v +=  TrappedBishop;
	}

	// trapped bishop (6th rank)

	if (board.square[A6] == WB  &&  board.square[B5] == BP) {
	opening.v -=  (TrappedBishop / 2);
	endgame.v -=  (TrappedBishop / 2);
	}

	if (board.square[H6] == WB  &&  board.square[G5] == BP) {
	opening.v -=  (TrappedBishop / 2);
	endgame.v -=  (TrappedBishop / 2);
	}

	if (board.square[A3] == BB  &&  board.square[B4] == WP) {
	opening.v +=  (TrappedBishop / 2);
	endgame.v +=  (TrappedBishop / 2);
	}

	if (board.square[H3] == BB  &&  board.square[G4] == WP) {
	opening.v +=  (TrappedBishop / 2);
	endgame.v +=  (TrappedBishop / 2);
	}

	// blocked bishop

	if (board.square[D2] == WP  &&  board.square[D3] != Empty  &&  board.square[C1] == WB) {
	opening.v -=  BlockedBishop;
	}

	if (board.square[E2] == WP  &&  board.square[E3] != Empty  &&  board.square[F1] == WB) {
	opening.v -=  BlockedBishop;
	}

	if (board.square[D7] == BP  &&  board.square[D6] != Empty  &&  board.square[C8] == BB) {
	opening.v +=  BlockedBishop;
	}

	if (board.square[E7] == BP  &&  board.square[E6] != Empty  &&  board.square[F8] == BB) {
	opening.v +=  BlockedBishop;
	}

	// blocked rook

	if ((board.square[C1] == WK  ||  board.square[B1] == WK)
	&&  (board.square[A1] == WR  ||  board.square[A2] == WR  ||  board.square[B1] == WR)) {
	opening.v -=  BlockedRook;
	}

	if ((board.square[F1] == WK  ||  board.square[G1] == WK)
	&&  (board.square[H1] == WR  ||  board.square[H2] == WR  ||  board.square[G1] == WR)) {
	opening.v -=  BlockedRook;
	}

	if ((board.square[C8] == BK  ||  board.square[B8] == BK)
	&&  (board.square[A8] == BR  ||  board.square[A7] == BR  ||  board.square[B8] == BR)) {
	opening.v +=  BlockedRook;
	}

	if ((board.square[F8] == BK  ||  board.square[G8] == BK)
	&&  (board.square[H8] == BR  ||  board.square[H7] == BR  ||  board.square[G8] == BR)) {
	opening.v +=  BlockedRook;
	}
	}

	// unstoppable_passer()

	private boolean unstoppable_passer( board_t board, int pawn, int colour ) {

	int me  = 0;     // int
	int opp  = 0;    // int
	int file  = 0;   // int
	int rank  = 0;   // int
	int king  = 0;   // int
	int prom  = 0;   // int
	int ptr  = 0;    // int
	int sq  = 0;     // int
	int dist  = 0;   // int

	//ASSERT(122, SQUARE_IS_OK(pawn));
	//ASSERT(123, COLOUR_IS_OK(colour));

	me = colour;
	opp = COLOUR_OPP(me);

	file = SQUARE_FILE(pawn);
	rank = PAWN_RANK(pawn,me);

	king = KING_POS(board,opp);

	// clear promotion path?


	ptr = 0;
	while(true) {
	sq=board.piece[me][ptr];
	if(sq==SquareNone) {
	break;
	}

	if (SQUARE_FILE(sq) == file  &&  PAWN_RANK(sq,me) > rank) {
	return false; // "friendly" blocker
	}
	ptr++;
	}


	// init

	if (rank == Rank2) {
	pawn = pawn + PawnMoveInc[me];
	rank++;
	//ASSERT(124, rank==PAWN_RANK(pawn,me));
	}

	//ASSERT(125, rank>=Rank3 && rank<=Rank7);

	prom = PAWN_PROMOTE(pawn,me);

	dist = DISTANCE(pawn,prom);
	//ASSERT(126, dist==Rank8-rank);
	if (board.turn == opp) {
	dist++;
	}

	if (DISTANCE(king,prom) > dist) {
	return true; // not in the square
	}

	return false;
	}

	// king_passer()

	private boolean king_passer( board_t board, int pawn, int colour ) {

	int me  = 0;     // int
	int king  = 0;   // int
	int file  = 0;   // int
	int prom  = 0;   // int

	//ASSERT(128, SQUARE_IS_OK(pawn));
	//ASSERT(129, COLOUR_IS_OK(colour));

	me = colour;

	king = KING_POS(board,me);
	file = SQUARE_FILE(pawn);
	prom = PAWN_PROMOTE(pawn,me);

	if (DISTANCE(king,prom) <= 1
	&&  DISTANCE(king,pawn) <= 1
	&&  (SQUARE_FILE(king) != file
	||  (file != FileA  &&  file != FileH))) {
	return true;
	}

	return false;
	}

	// free_passer()

	private boolean  free_passer( board_t board, int pawn, int colour ) {

	int me  = 0;    // int
//	int opp  = 0;   // int
	int inc  = 0;   // int
	int sq  = 0;    // int
	int move  = 0;  // int

	//ASSERT(131, SQUARE_IS_OK(pawn));
	//ASSERT(132, COLOUR_IS_OK(colour));

	me = colour;
//	opp = COLOUR_OPP(me);

	inc = PawnMoveInc[me];
	sq = pawn + inc;
	//ASSERT(133, SQUARE_IS_OK(sq));

	if (board.square[sq] != Empty) {
	return false;
	}

	move = MOVE_MAKE(pawn,sq);
	if (see_move(move,board) < 0) {
	return false;
	}

	return true;
	}

	// pawn_att_dist()

	private int pawn_att_dist( int pawn, int king, int colour ) {

	int me = 0;      // int
	int inc  = 0;     // int
	int target  = 0;  // int

	//ASSERT(134, SQUARE_IS_OK(pawn));
	//ASSERT(135, SQUARE_IS_OK(king));
	//ASSERT(136, COLOUR_IS_OK(colour));

	me = colour;
	inc = PawnMoveInc[me];

	target = pawn + inc;

	return DISTANCE(king,target);
	}

	// pawn_def_dist()

	private int pawn_def_dist( int pawn, int king, int colour ) {

	int me  = 0;      // int
	int inc  = 0;     // int
	int target  = 0;  // int

	//ASSERT(137, SQUARE_IS_OK(pawn));
	//ASSERT(138, SQUARE_IS_OK(king));
	//ASSERT(139, COLOUR_IS_OK(colour));

	me = colour;
	inc = PawnMoveInc[me];

	target = pawn + inc;

	return DISTANCE(king,target);
	}

	// draw_init_list()

	private void draw_init_list( int[] list, board_t board, int pawn_colour )  {

	int pos  = 0;   // int
	int att  = 0;   // int
	int def  = 0;   // int
	int ptr  = 0;   // int
	int sq  = 0;    // int
	int pawn  = 0;  // int
	int i  = 0;     // int

	//ASSERT(142, COLOUR_IS_OK(pawn_colour));

	// init

	pos = 0;

	att = pawn_colour;
	def = COLOUR_OPP(att);

	//ASSERT(143, board.pawn_size[att]==1);
	//ASSERT(144, board.pawn_size[def]==0);

	// att

	ptr = 0;
	while(true) {
	sq=board.piece[att][ptr];
	if(sq==SquareNone) {
	break;
	}
	list[pos] = sq;
	pos++;
	ptr++;
	}

	ptr = 0;
	while(true) {
	sq=board.pawn[att][ptr];
	if(sq==SquareNone) {
	break;
	}
	list[pos] = sq;
	pos++;
	ptr++;
	}

	// def

	ptr = 0;
	while(true) {
	sq=board.piece[def][ptr];
	if(sq==SquareNone) {
	break;
	}
	list[pos] = sq;
	pos++;
	ptr++;
	}

	ptr = 0;
	while(true) {
	sq=board.pawn[def][ptr];
	if(sq==SquareNone) {
	break;
	}
	list[pos] = sq;
	pos++;
	ptr++;
	}


	// } marker

	//ASSERT(145, pos==board.piece_nb);

	list[pos] = SquareNone;

	// file flip?

	pawn = board.pawn[att][0];

	if (SQUARE_FILE(pawn) >= FileE) {
	for (i = 0; i< pos; i++ ) {
	list[i] = SQUARE_FILE_MIRROR(list[i]);
	}
	}

	// rank flip?

	if (COLOUR_IS_BLACK(pawn_colour)) {
	for (i = 0; i< pos; i++ ) {
	list[i] = SQUARE_RANK_MIRROR(list[i]);
	}
	}
	}

	// draw_kpkq()

	private boolean draw_kpkq( int[] list, int turn ) {

	int wk  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int
	int bq  = 0;       // int
	int prom  = 0;     // int
	int dist  = 0;     // int
//	int wp_file  = 0;  // int
//	int wp_rank  = 0;  // int
	boolean ifelse = false;

	//ASSERT(147, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(148, SQUARE_IS_OK(wk));

	wp = list[1];
	//ASSERT(149, SQUARE_IS_OK(wp));
	//ASSERT(150, SQUARE_FILE(wp)<=FileD);

	bk =  list[2];
	//ASSERT(151, SQUARE_IS_OK(bk));

	bq =  list[3];
	//ASSERT(152, SQUARE_IS_OK(bq));

	//ASSERT(153, list[4]==SquareNone);

	// test

	if (wp == A7) {

	prom = A8;
	dist = 4;

	if (wk == B7  ||  wk == B8) {  // best case
	if (COLOUR_IS_WHITE(turn)) {
	dist--;
	}
	} else { 
	if (wk == A8  || ((wk == C7  ||  wk == C8)  &&  bq != A8)) {    // white loses a tempo
	if (COLOUR_IS_BLACK(turn)  &&  SQUARE_FILE(bq) != FileB) {
	return false;
	}
	} else { 
	return false;
	}
	}

	//ASSERT(154, bq!=prom);
	if (DISTANCE(bk,prom) > dist) {
	return true;
	}
	} else { 
	if (wp == C7) {

	prom = C8;
	dist = 4;

	ifelse = true;
	if (ifelse && wk == C8) {     // dist = 0

	dist++; // self-blocking penalty
	if (COLOUR_IS_WHITE(turn)) {
	dist--; // right-to-move bonus
	}

	ifelse = false;
	}
	if (ifelse && (wk == B7  ||  wk == B8)) { // dist = 1, right side

	dist--; // right-side bonus
	if (DELTA_INC_LINE(wp-bq) == wk-wp) {
	dist++; // pinned-pawn penalty
	}
	if (COLOUR_IS_WHITE(turn)) {
	dist--; // right-to-move bonus
	}

	ifelse = false;
	}

	if (ifelse && (wk == D7  ||  wk == D8)) { // dist = 1, wrong side

	if (DELTA_INC_LINE(wp-bq) == wk-wp) {
	dist++; // pinned-pawn penalty
	}
	if (COLOUR_IS_WHITE(turn)) {
	dist--; // right-to-move bonus
	}

	ifelse = false;
	}

	if (ifelse && ((wk == A7  ||  wk == A8)  &&  bq != C8)) {  // dist = 2, right side

	if (COLOUR_IS_BLACK(turn)  &&  SQUARE_FILE(bq) != FileB) {
	return false;
	}

	dist--; // right-side bonus

	ifelse = false;
	}

	if (ifelse && ((wk == E7  ||  wk == E8)  &&  bq != C8)) { // dist = 2, wrong side

	if (COLOUR_IS_BLACK(turn)  &&  SQUARE_FILE(bq) != FileD) {
	return false;
	}

	ifelse = false;
	}
	if (ifelse) {
	return false;
	}

	//ASSERT(155, bq!=prom);
	if (DISTANCE(bk,prom) > dist) {
	return true;
	}
	}
	}

	return false;
	}

	// draw_kpkr()

	private boolean draw_kpkr( int[] list, int turn ) {

	int wk  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int
	int br  = 0;       // int
	int inc  = 0;      // int
	int prom  = 0;     // int
	int dist  = 0;     // int
	int wk_file  = 0;  // int
	int wk_rank  = 0;  // int
	int wp_file  = 0;  // int
	int wp_rank  = 0;  // int
	int br_file  = 0;  // int
	int br_rank  = 0;  // int


	//ASSERT(157, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(158, SQUARE_IS_OK(wk));

	wp = list[1];
	//ASSERT(159, SQUARE_IS_OK(wp));
	//ASSERT(160, SQUARE_FILE(wp)<=FileD);

	bk = list[2];
	//ASSERT(161, SQUARE_IS_OK(bk));

	br = list[3];
	//ASSERT(162, SQUARE_IS_OK(br));

	//ASSERT(163, list[4]==SquareNone);

	// init

	wk_file = SQUARE_FILE(wk);
	wk_rank = SQUARE_RANK(wk);

	wp_file = SQUARE_FILE(wp);
	wp_rank = SQUARE_RANK(wp);

	br_file = SQUARE_FILE(br);
	br_rank = SQUARE_RANK(br);

	inc = PawnMoveInc[White];
	prom = PAWN_PROMOTE(wp,White);

	// conditions

	if (DISTANCE(wk,wp) == 1) {

	//ASSERT(164, Math.abs(wk_file-wp_file)<=1);
	//ASSERT(165, Math.abs(wk_rank-wp_rank)<=1);

	// no-op

	} else { 
	if (DISTANCE(wk,wp) == 2  &&  Math.abs(wk_rank-wp_rank) <= 1) {

	//ASSERT(166, Math.abs(wk_file-wp_file)==2);
	//ASSERT(167, Math.abs(wk_rank-wp_rank)<=1);

	if (COLOUR_IS_BLACK(turn)  &&  br_file != (wk_file + wp_file) / 2) {
	return false;
	}
	} else { 
	return false;
	}
	}

	// white features

	dist = DISTANCE(wk,prom) + DISTANCE(wp,prom);
	if (wk == prom) {
	dist++;
	}

	if (wk == wp+inc) {  // king on pawn's "front square"
	if (wp_file == FileA) {
	return false;
	}
	dist++; // self-blocking penalty
	}

	// black features

	if (br_file != wp_file  &&  br_rank != Rank8) {
	dist--; // misplaced-rook bonus
	}

	// test

	if (COLOUR_IS_WHITE(turn)) {
	dist--; // right-to-move bonus
	}

	if (DISTANCE(bk,prom) > dist) {
	return true;
	}

	return false;
	}

	// draw_kpkb()

	private boolean draw_kpkb( int[] list, int turn ) {

	int wk  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int
	int bb  = 0;       // int
	int inc  = 0;      // int
	int en2  = 0;      // int
	int to  = 0;       // int
	int delta  = 0;    // int
	int inc_2  = 0;    // int
	int sq  = 0;       // int


	//ASSERT(169, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(170, SQUARE_IS_OK(wk));

	wp = list[1];
	//ASSERT(171, SQUARE_IS_OK(wp));
	//ASSERT(172, SQUARE_FILE(wp)<=FileD);

	bk = list[2];
	//ASSERT(173, SQUARE_IS_OK(bk));

	bb = list[3];
	//ASSERT(174, SQUARE_IS_OK(bb));

	//ASSERT(175, list[4]==SquareNone);

	// blocked pawn?

	inc = PawnMoveInc[White];
	en2 = PAWN_PROMOTE(wp,White) + inc;

	to = wp+inc;
	while(to != en2) {

	//ASSERT(176, SQUARE_IS_OK(to));

	if (to == bb) {
	return true; // direct blockade
	}

	delta = to - bb;
	//ASSERT(177, delta_is_ok(delta));

	if (PSEUDO_ATTACK(BB,delta)) {

	inc_2 = DELTA_INC_ALL(delta);
	//ASSERT(178, inc_2!=IncNone);

	sq = bb;
	while(true) {

	sq+=inc_2;
	//ASSERT(179, SQUARE_IS_OK(sq));
	//ASSERT(180, sq!=wk);
	//ASSERT(181, sq!=wp);
	//ASSERT(182, sq!=bb);
	if (sq == to) {
	return true; // indirect blockade
	}
	if(sq == bk) {
	break;
	}

	}
	}
	to +=  inc;
	}

	return false;
	}

	// draw_kpkn()

	private boolean draw_kpkn( int[] list, int turn ) {

	int wk  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int
	int bn  = 0;       // int
	int inc  = 0;      // int
	int en2  = 0;      // int
	int file  = 0;     // int
	int sq  = 0;       // int


	//ASSERT(184, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(185, SQUARE_IS_OK(wk));

	wp = list[1];
	//ASSERT(186, SQUARE_IS_OK(wp));
	//ASSERT(187, SQUARE_FILE(wp)<=FileD);

	bk = list[2];
	//ASSERT(188, SQUARE_IS_OK(bk));

	bn = list[3];
	//ASSERT(189, SQUARE_IS_OK(bn));

	//ASSERT(190, list[4]==SquareNone);

	// blocked pawn?

	inc = PawnMoveInc[White];
	en2 = PAWN_PROMOTE(wp,White) + inc;

	file = SQUARE_FILE(wp);
	if (file == FileA  ||  file == FileH) {
	en2-=inc;
	}

	sq = wp+inc;
	while(sq != en2) {

	//ASSERT(191, SQUARE_IS_OK(sq));

	if (sq == bn  ||  PSEUDO_ATTACK(BN,sq-bn)) {
	return true; // blockade
	}

	sq+=inc;
	}

	return false;
	}

	// draw_knpk()

	private boolean draw_knpk( int[] list, int turn ) {

	int wk  = 0;       // int
	int wn  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int


	//ASSERT(193, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(194, SQUARE_IS_OK(wk));

	wn = list[1];
	//ASSERT(195, SQUARE_IS_OK(wn));

	wp = list[2];
	//ASSERT(196, SQUARE_IS_OK(wp));
	//ASSERT(197, SQUARE_FILE(wp)<=FileD);

	bk = list[3];
	//ASSERT(198, SQUARE_IS_OK(bk));

	//ASSERT(199, list[4]==SquareNone);

	// test

	if (wp == A7  &&  DISTANCE(bk,A8) <= 1) {
	return true;
	}

	return false;
	}

	// draw_krpkr()

	private boolean draw_krpkr( int[] list, int turn ) {

	int wk  = 0;       // int
	int wr  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int
	int br  = 0;       // int

	int wp_file  = 0;  // int
	int wp_rank  = 0;  // int
	int bk_file  = 0;  // int
	int bk_rank  = 0;  // int
	int br_file  = 0;  // int
	int br_rank  = 0;  // int

	int prom  = 0;     // int

	//ASSERT(201, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(202, SQUARE_IS_OK(wk));

	wr = list[1];
	//ASSERT(203, SQUARE_IS_OK(wr));

	wp = list[2];
	//ASSERT(204, SQUARE_IS_OK(wp));
	//ASSERT(205, SQUARE_FILE(wp)<=FileD);

	bk = list[3];
	//ASSERT(206, SQUARE_IS_OK(bk));

	br = list[4];
	//ASSERT(207, SQUARE_IS_OK(br));

	//ASSERT(208, list[5]==SquareNone);

	// test

	wp_file = SQUARE_FILE(wp);
	wp_rank = SQUARE_RANK(wp);

	bk_file = SQUARE_FILE(bk);
	bk_rank = SQUARE_RANK(bk);

	br_file = SQUARE_FILE(br);
	br_rank = SQUARE_RANK(br);

	prom = PAWN_PROMOTE(wp,White);

	if (bk == prom) {

	// to.do: rook near Rank1 if wp_rank == Rank6?

	if (br_file > wp_file) {
	return true;
	}

	} else { 
	if (bk_file == wp_file  &&  bk_rank > wp_rank) {

	return true;

	} else { 
	if (wr == prom  &&  wp_rank == Rank7  &&  (bk == G7  ||  bk == H7)  &&  br_file == wp_file) {

	if (br_rank <= Rank3) {
	if (DISTANCE(wk,wp) > 1) {
	return true;
	}
	} else {  // br_rank >= Rank4
	if (DISTANCE(wk,wp) > 2) {
	return true;
	}
	}
	}
	}
	}

	return false;
	}

	// draw_kbpkb()

	private boolean draw_kbpkb( int[] list, int turn ) {

	int wk  = 0;       // int
	int wb  = 0;       // int
	int wp  = 0;       // int
	int bk  = 0;       // int
	int bb  = 0;       // int

	int inc  = 0;      // int
	int en2  = 0;      // int
	int to  = 0;       // int
	int delta  = 0;    // int
	int inc_2  = 0;    // int
	int sq  = 0;       // int


	//ASSERT(210, COLOUR_IS_OK(turn));

	// load

	wk = list[0];
	//ASSERT(211, SQUARE_IS_OK(wk));

	wb = list[1];
	//ASSERT(212, SQUARE_IS_OK(wb));

	wp = list[2];
	//ASSERT(213, SQUARE_IS_OK(wp));
	//ASSERT(214, SQUARE_FILE(wp)<=FileD);

	bk = list[3];
	//ASSERT(215, SQUARE_IS_OK(bk));

	bb = list[4];
	//ASSERT(216, SQUARE_IS_OK(bb));

	//ASSERT(217, list[5]==SquareNone);

	// opposit colour?

	if (SQUARE_COLOUR(wb) == SQUARE_COLOUR(bb)) {
	return false; // 
	}

	// blocked pawn?

	inc = PawnMoveInc[White];
	en2 = PAWN_PROMOTE(wp,White) + inc;

	to = wp+inc;
	while( to != en2 ) {

	//ASSERT(218, SQUARE_IS_OK(to));

	if (to == bb) {
	return true; // direct blockade
	}

	delta = to - bb;
	//ASSERT(219, delta_is_ok(delta));

	if (PSEUDO_ATTACK(BB,delta)) {

	inc_2 = DELTA_INC_ALL(delta);
	//ASSERT(220, inc_2!=IncNone);

	sq = bb;
	while(true) {
	sq+=inc_2;
	//ASSERT(221, SQUARE_IS_OK(sq));
	//ASSERT(222, sq!=wk);
	//ASSERT(223, sq!=wb);
	//ASSERT(224, sq!=wp);
	//ASSERT(225, sq!=bb);
	if (sq == to) {
	return true; // indirect blockade
	}
	if (sq == bk) {
	break;
	}
	}
	}
	to +=  inc;
	}

	return false;
	}

	// shelter_square()

	private int shelter_square( board_t board, int square, int colour )  {

	int penalty  = 0;   // int
	int file  = 0;      // int
	int rank  = 0;      // int

	//ASSERT(227, SQUARE_IS_OK(square));
	//ASSERT(228, COLOUR_IS_OK(colour));

	penalty = 0;

	file = SQUARE_FILE(square);
	rank = PAWN_RANK(square,colour);

	penalty += ( shelter_file(board,file,rank,colour) * 2 );
	if (file != FileA) {
	penalty += shelter_file(board,file-1,rank,colour);
	}
	if (file != FileH) {
	penalty += shelter_file(board,file+1,rank,colour);
	}

	if (penalty == 0) {
	penalty = 11; // weak back rank
	}

	if (UseStorm) {
	penalty += storm_file(board,file,colour);
	if (file != FileA) {
	penalty += storm_file(board,file-1,colour);
	}
	if (file != FileH) {
	penalty += storm_file(board,file+1,colour);
	}
	}

	return penalty;
	}

	// shelter_file()

	private int shelter_file( board_t board, int file, int rank, int colour ) {

	int dist  = 0;      // int
	int penalty  = 0;   // int

	//ASSERT(230, file>=FileA && file<=FileH);
	//ASSERT(231, rank>=Rank1 && rank<=Rank8);
	//ASSERT(232, COLOUR_IS_OK(colour));

	dist = this.BitFirst[ ( board.pawn_file[colour][file] & this.BitGE[rank]) ];
	//ASSERT(233, dist>=Rank2 && dist<=Rank8);

	dist = Rank8 - dist;
	//ASSERT(234, dist>=0 && dist<=6);

	penalty = 36 - (dist * dist);
	//ASSERT(235, penalty>=0 && penalty<=36);

	return penalty;
	}

	// storm_file()

	private int storm_file( board_t board, int file, int colour ) {

	int dist  = 0;      // int
	int penalty  = 0;   // int

	//ASSERT(237, file>=FileA && file<=FileH);
	//ASSERT(238, COLOUR_IS_OK(colour));

	dist = this.BitLast[board.pawn_file[COLOUR_OPP(colour)][file] ];
	//ASSERT(239, dist>=Rank1 && dist<=Rank7);

	penalty = 0;

	if(dist == Rank4) {
	penalty = StormOpening * 1;
	} else { 
	if(dist == Rank5) {
	penalty = StormOpening * 3;
	} else { 
	if(dist == Rank6) {
	penalty = StormOpening * 6;
	}
	}
	}

	return penalty;
	}

	// bishop_can_attack()

	private boolean bishop_can_attack( board_t board, int to, int colour ) {

	int ptr  = 0;    // int
	int from  = 0;   // int
	int piece  = 0;  // int

	//ASSERT(241, SQUARE_IS_OK(to));
	//ASSERT(242, COLOUR_IS_OK(colour));

	ptr = 1;                // HACK: no king
	while(true) {
	from = board.piece[colour][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	if (PIECE_IS_BISHOP(piece)  &&  SQUARE_COLOUR(from) == SQUARE_COLOUR(to)) {
	return true;
	}
	ptr++;
	}

	return false;
	}

	// end of eval.cpp



	// fen.cpp



	private void if_fen_err( boolean logic, String fenstr, int pos ) { 
	if(logic) {
	my_fatal("board_from_fen(): bad FEN " + fenstr + " at pos=" + string_from_int(pos) + " \n");
	}
	}

	// board_from_fen()

	private void board_from_fen( board_t board, String fenstr )  {

	int pos  = 0;   // int
	int file  = 0;  // int
	int rank  = 0;  // int
	int sq  = 0;    // int
	char c;   // char
	String nb = "";   // string
	int i  = 0;     // int
	int len  = 0;   // int
	int piece  = 0; // int
	int pawn  = 0;  // int
	int q = fenstr.length();
	char[] fen = new char[q];  // char[]
	boolean gotoupdate = false;

	board_clear(board);

	for (i = 0; i<q; i++ ) {
	fen[i] = fenstr.charAt(i);
	}

	pos = 0;
	c = fen[pos];

	// piece placement

	for (rank = Rank8; rank>=Rank1; rank-- ) {

	file = FileA;
	while ( file <= FileH ) {

	if (c >= '1'  &&  c <= '8') {            // empty square(s)

	len = (c - '0');

	for (i = 0; i<len; i++ ) {
	if_fen_err( file > FileH, fenstr, pos );

	board.square[SQUARE_MAKE(file,rank)] = Empty;
	file++;
	}

	} else {   // piece

	piece = piece_from_char(c);
	if_fen_err( piece == PieceNone256, fenstr, pos );

	board.square[SQUARE_MAKE(file,rank)] = piece;
	file++;
	}

	pos++;
	c = fen[pos];
	}

	if (rank > Rank1) {
	if_fen_err( c != '/', fenstr, pos );
	pos++;
	c = fen[pos];

	}
	}

	// active colour

	if_fen_err( c != ' ', fenstr, pos );

	pos++;
	c = fen[pos];

	if(c=='w') {
	board.turn = White;
	} else { 
	if(c=='b') {
	board.turn = Black;
	} else { 
	if_fen_err( true, fenstr, pos );
	}
	}

	pos++;
	c = fen[pos];

	// castling

	if_fen_err( c != ' ', fenstr, pos );

	pos++;
	c = fen[pos];

	board.flags = FlagsNone;

	if (c == '-') {    // no castling rights

	pos++;
	c = fen[pos];

	} else { 


	if (c == 'K') {
	if (board.square[E1] == WK  &&  board.square[H1] == WR) {
	board.flags |= FlagsWhiteKingCastle;
	}
	pos++;
	c = fen[pos];
	}

	if (c == 'Q') {
	if (board.square[E1] == WK  &&  board.square[A1] == WR) {
	board.flags |= FlagsWhiteQueenCastle;
	}
	pos++;
	c = fen[pos];
	}

	if (c == 'k') {
	if (board.square[E8] == BK  &&  board.square[H8] == BR) {
	board.flags |= FlagsBlackKingCastle;
	}
	pos++;
	c = fen[pos];
	}

	if (c == 'q') {
	if (board.square[E8] == BK  &&  board.square[A8] == BR) {
	board.flags |= FlagsBlackQueenCastle;
	}
	pos++;
	c = fen[pos];
	}
	}

	// en-passant

	if_fen_err( c != ' ', fenstr, pos );

	pos++;
	c = fen[pos];

	if (c == '-') {   // no en-passant

	sq = SquareNone;
	pos++;
	c = fen[pos];

	} else { 

	if_fen_err( c < 'a'  ||  c > 'h', fenstr, pos );
	file = file_from_char(c);
	pos++;
	c = fen[pos];

	if_fen_err( c != (COLOUR_IS_WHITE(board.turn) ? '6' : '3'), fenstr, pos );

	rank = rank_from_char(c);
	pos++;
	c = fen[pos];

	sq = SQUARE_MAKE(file,rank);
	pawn = SQUARE_EP_DUAL(sq);

	if (board.square[sq] != Empty
	||  board.square[pawn] != PawnMake[COLOUR_OPP(board.turn)]
	||  (board.square[pawn-1] != PawnMake[board.turn]
	&&  board.square[pawn+1] != PawnMake[board.turn])) {
	sq = SquareNone;
	}
	}

	board.ep_square = sq;

	// halfmove clock

	board.ply_nb = 0;
	board.movenumb = 0;

	if (c != ' ') {
	if (! Strictq) {
	gotoupdate = true;
	} else { 
	if_fen_err( true, fenstr, pos );
	}
	}

	if( ! gotoupdate ) {
	pos++;
	c = fen[pos];

	if (c<'0' || c>'9') {
	if (! Strictq) {
	gotoupdate = true;
	} else { 
	if_fen_err( true, fenstr, pos );
	}
	}
	}

	if( ! gotoupdate ) {
	nb = str_after_ok( fenstr.substring( pos ), " ");  // ignore halfmove clock
	board.ply_nb = Integer.parseInt( nb );
	board.movenumb = board.ply_nb;  // just save it
	}

	// board update

	// update:
	board_init_list(board);
	}

	// board_to_fen()

	public boolean board_to_fen( board_t board, string_t strfen ) {

	int file  = 0;   // int
	int rank  = 0;   // int
	int sq  = 0;     // int
	int piece  = 0;  // int
	char c = ' ';    // string
	int len  = 0;    // int
	String fen = "";   // string
	string_t str1 = new string_t();

	// piece placement

	for (rank = Rank8; rank>=Rank1; rank-- ) {

	file = FileA;
	while( file <= FileH ) {

	sq = SQUARE_MAKE(file,rank);
	piece = board.square[sq];
	//ASSERT(248, piece==Empty || piece_is_ok(piece));

	if (piece == Empty) {

	len = 0;
	while( file <= FileH  &&  board.square[SQUARE_MAKE(file,rank)] == Empty ) {

	file++;
	len++;
	}

	//ASSERT(249, len>=1 && len<=8);
	c = (char)( '0' + len );

	} else { 

	c = piece_to_char(piece);
	file++;
	}

	fen +=  c;

	}

	if( rank != Rank1 ) {
	fen +=  "/";
	}
	}

	// active colour

	fen +=  " " + (COLOUR_IS_WHITE(board.turn) ? "w" : "b" ) + " ";

	// castling

	if (board.flags == FlagsNone) {
	fen +=  "-";
	} else { 
	if ( ( board.flags & FlagsWhiteKingCastle) != 0) {
	fen +=  "K";
	}
	if ( ( board.flags & FlagsWhiteQueenCastle) != 0) {
	fen +=  "Q";
	}
	if ( ( board.flags & FlagsBlackKingCastle) != 0) {
	fen +=  "k";
	}
	if ( ( board.flags & FlagsBlackQueenCastle) != 0) {
	fen +=  "q";
	}
	}

	fen +=  " ";

	// en-passant

	if (board.ep_square == SquareNone) {
	fen +=  "-";
	} else { 
	square_to_string(board.ep_square, str1 );
	fen +=  str1.v;
	}

	fen +=  " ";

	// ignoring halfmove clock

	fen +=  "0 " + string_from_int(board.movenumb);

	strfen.v = fen;

	return true;
	}

	// to see on screen

	private void printboard()  {

	int file  = 0;   // int
	int rank  = 0;   // int
	int sq  = 0;     // int
	int piece  = 0;  // int
	string_t str1 = new string_t();
	String s = "";     //  string
	board_t board = this.SearchInput.board;

	// piece placement

	for (rank = Rank8; rank>=Rank1; rank-- ) {

	file = FileA;
	while( file <= FileH ) {

	sq = SQUARE_MAKE(file,rank);
	piece = board.square[sq];
	//ASSERT(248, piece==Empty || piece_is_ok(piece));

	if(piece == Empty) {
	s += ".";
	} else { 
	s += piece_to_char(piece);
	}

	file++;
	}

	s += "\n";
	}

	board_to_fen( board, str1 );

	s += str1.v + "\n";

	print2out(s);

	}

	// end of fen.cpp


	// hash.cpp

	// we accept some level collisions

	// hash_init()

	private void hash_init() { 

	int i  = 0;   // int

	for (i = 0; i<=15; i++ ) {
	this.Castle64[i] = hash_castle_key(i);
	}
	}

	// hash_key()

	private int hash_key( board_t board ) {    // uint64 
	int key  = 0;     // uint64;
	int colour  = 0; // int
	int ptr  = 0;    // int
	int sq  = 0;     // int
	int piece  = 0;  // int

	// init

	key = 0;

	// pieces

	for (colour = 0;  colour<=1; colour++ ) {

	ptr = 0;
	while(true) {
	sq=board.piece[colour][ptr];
	if(sq== SquareNone) {
	break;
	}

	piece = board.square[sq];
	key ^= hash_piece_key(piece,sq);

	ptr++;
	}


	ptr = 0;
	while(true) {
	sq=board.pawn[colour][ptr];
	if(sq== SquareNone) {
	break;
	}

	piece = board.square[sq];
	key ^= hash_piece_key(piece,sq);

	ptr++;
	}

	}

	// castle flags

	key ^= hash_castle_key(board.flags);

	// en-passant square

	sq = board.ep_square;
	if (sq != SquareNone) {
	key ^= hash_ep_key(sq);
	}

	// turn

	key ^= hash_turn_key(board.turn);

	return key;
	}

	// hash_pawn_key()

	private int hash_pawn_key( board_t board ) { // uint64 

	int key  = 0;     // uint64;
	int colour  = 0; // int
	int ptr  = 0;    // int
	int sq  = 0;     // int
	int piece  = 0;  // int


	// init

	key = 0;

	// pawns

	for (colour = 0;  colour<=1; colour++ ) {

	ptr = 0;
	while(true) {
	sq=board.pawn[colour][ptr];
	if(sq== SquareNone) {
	break;
	}

	piece = board.square[sq];
	key ^= hash_piece_key(piece,sq);

	ptr++;
	}

	}

	return key;
	}

	// hash_material_key()

	private int hash_material_key( board_t board ) {  // uint64 

	int key  = 0;     // uint64;
	int piece1 = 0;  // int
	int count  = 0;  // int


	// init

	key = 0;

	// counters

	for (piece1 = 0; piece1<=11; piece1++ ) {
	count = board.number[piece1];
	key ^= hash_counter_key(piece1,count);
	}

	return key;

	}

	// hash_piece_key()

	private int hash_piece_key( int piece, int square )  {  // uint64 

	//ASSERT(253, piece_is_ok(piece));
	//ASSERT(254, SQUARE_IS_OK(square));

	return this.Random64[RandomPiece+(this.PieceTo12[piece] ^ 1)*64 + this.SquareTo64[square] ];
	// HACK: xor 1 for PolyGlot book (not AS3)
	}

	// hash_castle_key()

	private int hash_castle_key( int flags )  {  // uint64 

	int key  = 0;     // uint64;
	int i  = 0;      // int

	//ASSERT(255, (flags & bnotxF)==0);

	key = 0;

	for (i = 0; i<=3; i++ ) {
	if ( ( flags & (1 << i) ) != 0) {
	key ^= this.Random64[RandomCastle+i];
	}
	}

	return key;
	}

	// hash_ep_key()

	private int hash_ep_key( int square )  {  // uint64 

	//ASSERT(256, SQUARE_IS_OK(square));

	return this.Random64[RandomEnPassant+SQUARE_FILE(square)-FileA ];
	}

	// hash_turn_key()

	private int hash_turn_key( int colour ) { // uint64 

	//ASSERT(257, COLOUR_IS_OK(colour));

	return (COLOUR_IS_WHITE(colour) ? this.Random64[RandomTurn] : 0 );
	}

	// hash_counter_key()

	private int hash_counter_key( int piece_12, int count )  { // uint64 

	int key  = 0;     // uint64;
	int i  = 0;      // int
	int index  = 0;  // int

	//ASSERT(258, piece_12>=0 && piece_12<12);
	//ASSERT(259, count>=0 && count<=10);

	// init

	key = 0;

	// counter

	index = piece_12 * 16;
	for (i = 0; i<count; i++ ) {
	key ^= this.Random64[index+i];
	}

	return key;

	}

	// end of hash.cpp





	// list.cpp



	// list_is_ok()

	private boolean list_is_ok( list_t list ) {


	if (list.size < 0  ||  list.size >= ListSize) {
	return false;
	}

	return true;
	}

	// list_remove()

/*
	private void list_remove( list_t list, int pos )  {

	int i  = 0;   // int

	//ASSERT(260, list_is_ok(list));
	//ASSERT(261, pos>=0 && pos<list.size);

	for (i = pos; i<= list.size-2; i++ ) {
	list.move[i] = list.move[i+1];
	list.value[i] = list.value[i+1];
	}

	list.size--;
	}
*/

	// list_copy()

	private void list_copy( list_t dst, list_t src )   {

	int i  = 0;   // int

	//ASSERT(263, list_is_ok(src));

	dst.size = src.size;

	for (i = 0; i< src.size; i++ ) {
	dst.move[i] = src.move[i];
	dst.value[i] = src.value[i];
	}
	}

	// list_sort()

	private void list_sort( list_t list )   {

	int size  = 0;   // int
	int i  = 0;      // int
	int j  = 0;      // int
	int move  = 0;   // int
	int value  = 0;  // int

	//ASSERT(264, list_is_ok(list));

	// init

	size = list.size;
	list.value[size] = -32768; // HACK: sentinel

	// insert sort (stable)

	for (i = size-2; i>=0; i-- ) {

	move = list.move[i];
	value = list.value[i];

	j = i;
	while( value < list.value[j+1] ) {
	list.move[j] = list.move[j+1];
	list.value[j] = list.value[j+1];
	j++;
	}

	//ASSERT(265, j<size);

	list.move[j] = move;
	list.value[j] = value;
	}

// --for debugging
//	for (i = 0; i<=size-2; i++ ) {
//	//ASSERT(266, list.value[i]>=list.value[i+1]);
//	}
	
	}

	
	// list_contain()

	private boolean list_contain( list_t list, int move ) {

	int i  = 0;   // int

	//ASSERT(267, list_is_ok(list));
	//ASSERT(268, move_is_ok(move));

	for (i = 0; i<list.size; i++ ) {
	if (list.move[i] == move) {
	return true;
	}
	}

	return false;
	}

	// list_note()

	private void list_note( list_t list )   {

	int i  = 0;      // int
	int move  = 0;   // int

	//ASSERT(269, list_is_ok(list));

	for (i = 0; i<list.size; i++ ) {
	move = list.move[i];
	//ASSERT(270, move_is_ok(move));
	list.value[i] = -move_order(move);
	}
	}

	// list_filter()

	private void list_filter( list_t list, board_t board, boolean keep )  {

	int pos  = 0;   // int
	int i  = 0;     // int
	int move  = 0;  // int
	int value  = 0; // int

	pos = 0;

	for (i = 0; i<list.size; i++ ) {

	//ASSERT(275, pos>=0 && pos<=i);

	move = list.move[i];
	value = list.value[i];

	if (pseudo_is_legal(move,board) == keep) {
	list.move[pos] = move;
	list.value[pos] = value;
	pos++;
	}
	}

	//ASSERT(276, pos>=0 && pos<=list.size);
	list.size = pos;

	// debug

	//ASSERT(277, list_is_ok(list));
	}

	// end of list.cpp




	// material.cpp



	// material_init()

	private void material_init() { 

	// UCI options

	this.MaterialWeight = (option_get_int("Material") * 256 + 50) / 100;
	}

	// material_alloc()

	private void material_alloc() {
		
	if (UseTable) {
	material_clear();
	//Material.table = (entry_t *) my_malloc(Material.size*sizeof(entry_t));
	}

	}

	// material_clear()

	private void material_clear() {

	Material.size = MaterialTableSize;
	Material.mask = Material.size - 1;   // 2^x -1
	
	}

	// material_get_info()

	private void material_get_info( material_info_t info, board_t board )   {

	int key  = 0;             // uint64
	material_info_t entry = new material_info_t();         // *
	int index = 0;

	// probe

	if (UseTable) {

	this.Material.read_nb++;

	key = board.material_key;
	index = ( KEY_INDEX(key) & this.Material.mask );

	entry = this.Material.table[index];

	if(entry==null) {
	this.Material.table[index] = new material_info_t();
	entry = this.Material.table[index];
	}

	if (entry.lock == KEY_LOCK(key)) {

	// found

	this.Material.read_hit++;

	material_info_copy( info, entry );

	return;
	}
	}

	// calculation

	material_comp_info(info,board);

	// store

	if (UseTable) {

	this.Material.write_nb++;

	if (entry.lock == 0) {     // HACK: assume free entry
	this.Material.used++;
	} else { 
	this.Material.write_collision++;
	}

	material_info_copy( entry, info );

	entry.lock = KEY_LOCK(key);
	}

	}

	// material_comp_info()

	private void material_comp_info( material_info_t info,  board_t board)   {

	int wp  = 0;   // int
	int wn  = 0;   // int
	int wb  = 0;   // int
	int wr  = 0;   // int
	int wq  = 0;   // int
	int bp  = 0;   // int
	int bn  = 0;   // int
	int bb  = 0;   // int
	int br  = 0;   // int
	int bq  = 0;   // int

	int wt  = 0;   // int
	int bt  = 0;   // int
	int wm  = 0;   // int
	int bm  = 0;   // int

	int colour  = 0;  // int
	int recog  = 0;   // int
	int flags  = 0;   // int
	int[] cflags = { 0, 0 }; // int[ColourNb]
	int[] mul = { 16, 16 };    // int[ColourNb]
	int phase  = 0;   // int
	int opening  = 0; // int
	int endgame  = 0; // int
	boolean ifelse = false;


	// init

	wp = board.number[WhitePawn12];
	wn = board.number[WhiteKnight12];
	wb = board.number[WhiteBishop12];
	wr = board.number[WhiteRook12];
	wq = board.number[WhiteQueen12];

	bp = board.number[BlackPawn12];
	bn = board.number[BlackKnight12];
	bb = board.number[BlackBishop12];
	br = board.number[BlackRook12];
	bq = board.number[BlackQueen12];

	wt = wq + wr + wb + wn + wp; // no king
	bt = bq + br + bb + bn + bp; // no king

	wm = wb + wn;
	bm = bb + bn;

	int w_maj  = wq * 2 + wr;         // int
	int w_min  = wb + wn;             // int
	int w_tot  = w_maj * 2 + w_min;   // int

	int b_maj  = bq * 2 + br;         // int
	int b_min  = bb + bn;             // int
	int b_tot  = b_maj * 2 + b_min;   // int

	// recogniser

	recog = MAT_NONE;

	ifelse = true;

	if (ifelse && (wt == 0  &&  bt == 0)) {

	recog = MAT_KK;

	ifelse = false;
	}

	if (ifelse && (wt == 1  &&  bt == 0)) {

	if (wb == 1) {
	recog = MAT_KBK;
	}
	if (wn == 1) {
	recog = MAT_KNK;
	}
	if (wp == 1) {
	recog = MAT_KPK;
	}

	ifelse = false;
	}

	if (ifelse && (wt == 0  &&  bt == 1)) {

	if (bb == 1) {
	recog = MAT_KKB;
	}
	if (bn == 1) {
	recog = MAT_KKN;
	}
	if (bp == 1) {
	recog = MAT_KKP;
	}

	ifelse = false;
	}

	if (ifelse && (wt == 1  &&  bt == 1)) {

	if (wq == 1  &&  bq == 1) {
	recog = MAT_KQKQ;
	}
	if (wq == 1  &&  bp == 1) {
	recog = MAT_KQKP;
	}
	if (wp == 1  &&  bq == 1) {
	recog = MAT_KPKQ;
	}
	if (wr == 1  &&  br == 1) {
	recog = MAT_KRKR;
	}
	if (wr == 1  &&  bp == 1) {
	recog = MAT_KRKP;
	}
	if (wp == 1  &&  br == 1) {
	recog = MAT_KPKR;
	}
	if (wb == 1  &&  bb == 1) {
	recog = MAT_KBKB;
	}
	if (wb == 1  &&  bp == 1) {
	recog = MAT_KBKP;
	}
	if (wp == 1  &&  bb == 1) {
	recog = MAT_KPKB;
	}
	if (wn == 1  &&  bn == 1) {
	recog = MAT_KNKN;
	}
	if (wn == 1  &&  bp == 1) {
	recog = MAT_KNKP;
	}
	if (wp == 1  &&  bn == 1) {
	recog = MAT_KPKN;
	}

	ifelse = false;
	}

	if (ifelse && (wt == 2  &&  bt == 0)) {

	if (wb == 1  &&  wp == 1) {
	recog = MAT_KBPK;
	}
	if (wn == 1  &&  wp == 1) {
	recog = MAT_KNPK;
	}

	ifelse = false;
	}

	if (ifelse && (wt == 0  &&  bt == 2)) {

	if (bb == 1  &&  bp == 1) {
	recog = MAT_KKBP;
	}
	if (bn == 1  &&  bp == 1) {
	recog = MAT_KKNP;
	}

	ifelse = false;
	}

	if (ifelse && (wt == 2  &&  bt == 1)) {

	if (wr == 1  &&  wp == 1  &&  br == 1) {
	recog = MAT_KRPKR;
	}
	if (wb == 1  &&  wp == 1  &&  bb == 1) {
	recog = MAT_KBPKB;
	}

	ifelse = false;
	}

	if (ifelse && (wt == 1  &&  bt == 2)) {

	if (wr == 1  &&  br == 1  &&  bp == 1) {
	recog = MAT_KRKRP;
	}
	if (wb == 1  &&  bb == 1  &&  bp == 1) {
	recog = MAT_KBKBP;
	}

	ifelse = false;
	}

	// draw node (exact-draw recogniser)

	flags = 0; // to.do: MOVE ME

	// if no major piece || pawn
	if (wq+wr+wp == 0  &&  bq+br+bp == 0) {
	// at most one minor => KK, KBK || KNK
	if (wm + bm <= 1 ||  recog == MAT_KBKB) {
	flags = ( flags | DrawNodeFlag );
	}

	} else { 
	if (recog == MAT_KPK   ||  recog == MAT_KKP ||  recog == MAT_KBPK  ||  recog == MAT_KKBP) {
	flags = ( flags | DrawNodeFlag );
	}
	}

	// bishop endgame
	// if only bishops
	if (wq+wr+wn == 0  &&  bq+br+bn == 0) {
	if (wb == 1  &&  bb == 1) {
	if (wp-bp >= -2  &&  wp-bp <= 2) {    // pawn diff <= 2
	flags = ( flags | DrawBishopFlag );
	}
	}
	}

	// white multiplier

	if (wp == 0) {  // white has no pawns

	ifelse = true;
	if (ifelse && (w_tot == 1)) {

	//ASSERT(283, w_maj==0);
	//ASSERT(284, w_min==1);

	// KBK* || KNK*, always insufficient

	mul[White] = 0;


	ifelse = false;
	}

	if (ifelse && (w_tot == 2  &&  wn == 2)) {

	//ASSERT(285, w_maj==0);
	//ASSERT(286, w_min==2);

	// KNNK*, usually insufficient

	if (b_tot != 0  ||  bp == 0) {
	mul[White] = 0;
	} else {    // KNNKP+, might not be draw
	mul[White] = 1; // 1/16
	}

	ifelse = false;
	}

	if (ifelse && (w_tot == 2  &&  wb == 2  &&  b_tot == 1  &&  bn == 1)) {

	//ASSERT(287, w_maj==0);
	//ASSERT(288, w_min==2);
	//ASSERT(289, b_maj==0);
	//ASSERT(290, b_min==1);

	// KBBKN*, barely drawish (not at all?)

	mul[White] = 8; // 1/2

	ifelse = false;
	}

	if (ifelse && (w_tot-b_tot <= 1  &&  w_maj <= 2)) {

	// no more than 1 minor up, drawish

	mul[White] = 2; // 1/8
	ifelse = false;
	}

	} else { 

	if (wp == 1) { // white has one pawn

	if (b_min != 0) {

	// assume black sacrifices a minor against the lone pawn

	b_min--;
	b_tot++;

	ifelse = true;
	if (ifelse && (w_tot == 1)) {

	//ASSERT(291, w_maj==0);
	//ASSERT(292, w_min==1);

	// KBK* || KNK*, always insufficient

	mul[White] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (w_tot == 2  &&  wn == 2)) {

	//ASSERT(293, w_maj==0);
	//ASSERT(294, w_min==2);

	// KNNK*, usually insufficient

	mul[White] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (w_tot-b_tot <= 1  &&  w_maj <= 2)) {

	// no more than 1 minor up, drawish

	mul[White] = 8; // 1/2

	ifelse = false;
	}

	} else { 
	if (br != 0) {

	// assume black sacrifices a rook against the lone pawn

	b_maj--;
	b_tot-=2;

	ifelse = true;
	if (ifelse && (w_tot == 1)) {

	//ASSERT(295, w_maj==0);
	//ASSERT(296, w_min==1);

	// KBK* || KNK*, always insufficient

	mul[White] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (w_tot == 2  &&  wn == 2)) {

	//ASSERT(297, w_maj==0);
	//ASSERT(298, w_min==2);

	// KNNK*, usually insufficient

	mul[White] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (w_tot-b_tot <= 1  &&  w_maj <= 2)) {

	// no more than 1 minor up, drawish

	mul[White] = 8; // 1/2

	ifelse = false;
	}

	}
	}

	}
	}

	// black multiplier

	if (bp == 0) {    // black has no pawns


	ifelse = true;
	if (ifelse && (b_tot == 1)) {

	//ASSERT(299, b_maj==0);
	//ASSERT(300, b_min==1);

	// KBK* || KNK*, always insufficient

	mul[Black] = 0;

	ifelse = false;
	}

	if (ifelse && (b_tot == 2  &&  bn == 2)) {

	//ASSERT(301, b_maj==0);
	//ASSERT(302, b_min==2);

	// KNNK*, usually insufficient

	if (w_tot != 0  ||  wp == 0) {
	mul[Black] = 0;
	} else {   // KNNKP+, might not be draw
	mul[Black] = 1; // 1/16
	}

	ifelse = false;
	}

	if (ifelse && (b_tot == 2  &&  bb == 2  &&  w_tot == 1  &&  wn == 1)) {

	//ASSERT(303, b_maj==0);
	//ASSERT(304, b_min==2);
	//ASSERT(305, w_maj==0);
	//ASSERT(306, w_min==1);

	// KBBKN*, barely drawish (not at all?)

	mul[Black] = 8; // 1/2

	ifelse = false;
	}

	if (ifelse && (b_tot-w_tot <= 1  &&  b_maj <= 2)) {

	// no more than 1 minor up, drawish

	mul[Black] = 2; // 1/8

	ifelse = false;
	}

	} else { 
	if (bp == 1) {  // black has one pawn

	if (w_min != 0) {

	// assume white sacrifices a minor against the lone pawn

	w_min--;
	w_tot--;

	ifelse = true;
	if (ifelse && (b_tot == 1)) {

	//ASSERT(307, b_maj==0);
	//ASSERT(308, b_min==1);

	// KBK* || KNK*, always insufficient

	mul[Black] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (b_tot == 2  &&  bn == 2)) {

	//ASSERT(309, b_maj==0);
	//ASSERT(310, b_min==2);

	// KNNK*, usually insufficient

	mul[Black] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (b_tot-w_tot <= 1  &&  b_maj <= 2)) {

	// no more than 1 minor up, drawish

	mul[Black] = 8; // 1/2

	ifelse = false;
	}

	} else { 
	if (wr != 0) {

	// assume white sacrifices a rook against the lone pawn

	w_maj--;
	w_tot-=2;

	ifelse = true;
	if (ifelse && (b_tot == 1)) {

	//ASSERT(311, b_maj==0);
	//ASSERT(312, b_min==1);

	// KBK* || KNK*, always insufficient

	mul[Black] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (b_tot == 2  &&  bn == 2)) {

	//ASSERT(313, b_maj==0);
	//ASSERT(314, b_min==2);

	// KNNK*, usually insufficient

	mul[Black] = 4; // 1/4

	ifelse = false;
	}

	if (ifelse && (b_tot-w_tot <= 1  &&  b_maj <= 2)) {

	// no more than 1 minor up, drawish

	mul[Black] = 8; // 1/2

	ifelse = false;
	}

	}
	}
	}
	}

	// potential draw for white

	if (wt == wb+wp  &&  wp >= 1) {
	cflags[White] = ( cflags[White] | MatRookPawnFlag );
	}
	if (wt == wb+wp  &&  wb <= 1  &&  wp >= 1  &&  bt > bp) {
	cflags[White] = ( cflags[White] | MatBishopFlag );
	}

	if (wt == 2  &&  wn == 1  &&  wp == 1  &&  bt > bp) {
	cflags[White] = ( cflags[White] | MatKnightFlag );
	}

	// potential draw for black

	if (bt == bb+bp  &&  bp >= 1) {
	cflags[Black] = ( cflags[Black] | MatRookPawnFlag );
	}
	if (bt == bb+bp  &&  bb <= 1  &&  bp >= 1  &&  wt > wp) {
	cflags[Black] = ( cflags[Black] | MatBishopFlag );
	}

	if (bt == 2  &&  bn == 1  &&  bp == 1  &&  wt > wp) {
	cflags[Black] = ( cflags[Black] | MatKnightFlag );
	}

	// draw leaf (likely draw)

	if (recog == MAT_KQKQ  ||  recog == MAT_KRKR) {
	mul[White] = 0;
	mul[Black] = 0;
	}

	// king safety

	if (bq >= 1  &&  bq+br+bb+bn >= 2) {
	cflags[White] = ( cflags[White] | MatKingFlag );
	}
	if (wq >= 1  &&  wq+wr+wb+wn >= 2) {
	cflags[Black] = ( cflags[Black] | MatKingFlag );
	}

	// phase (0: opening . 256: endgame)

	phase = TotalPhase;

	phase -=  (wp * PawnPhase);
	phase -=  (wn * KnightPhase);
	phase -=  (wb * BishopPhase);
	phase -=  (wr * RookPhase);
	phase -=  (wq * QueenPhase);

	phase -=  (bp * PawnPhase);
	phase -=  (bn * KnightPhase);
	phase -=  (bb * BishopPhase);
	phase -=  (br * RookPhase);
	phase -=  (bq * QueenPhase);

	if (phase < 0) {
	phase = 0;
	}

	//ASSERT(315, phase>=0 && phase<=TotalPhase);
	phase = Math.min( ((phase * 256) + (TotalPhase / 2)) / TotalPhase, 256 );

	//ASSERT(316, phase>=0 && phase<=256);

	// material

	opening = 0;
	endgame = 0;

	opening += (wp * PawnOpening);
	opening += (wn * KnightOpening);
	opening += (wb * BishopOpening);
	opening += (wr * RookOpening);
	opening += (wq * QueenOpening);

	opening -= (bp * PawnOpening);
	opening -= (bn * KnightOpening);
	opening -= (bb * BishopOpening);
	opening -= (br * RookOpening);
	opening -= (bq * QueenOpening);

	endgame +=  (wp * PawnEndgame);
	endgame +=  (wn * KnightEndgame);
	endgame +=  (wb * BishopEndgame);
	endgame +=  (wr * RookEndgame);
	endgame +=  (wq * QueenEndgame);

	endgame -=  (bp * PawnEndgame);
	endgame -=  (bn * KnightEndgame);
	endgame -=  (bb * BishopEndgame);
	endgame -=  (br * RookEndgame);
	endgame -=  (bq * QueenEndgame);

	// bishop pair

	if (wb >= 2) {     // HACK: assumes different colours
	opening += BishopPairOpening;
	endgame +=  BishopPairEndgame;
	}

	if (bb >= 2) {     // HACK: assumes different colours
	opening -= BishopPairOpening;
	endgame -=  BishopPairEndgame;
	}

	// store info

	info.recog = recog;
	info.flags = flags;

	for (colour = 0;  colour<=1; colour++ ) {
	info.cflags[colour] = cflags[colour];
	info.mul[colour] = mul[colour];
	}

	info.phase = phase;
	info.opening = (opening * this.MaterialWeight) / 256;
	info.endgame = (endgame * this.MaterialWeight) / 256;
	}

	// end of material.cpp



	// move.cpp



	// move_is_ok()

	private boolean move_is_ok( int move ) {

	if (move < 0  ||  move >= 65536 || move == MoveNone || move == Movenull) {
	return false;
	}
	return true;
	}

	// move_promote()

	private int move_promote( int move ) {

	int code  = 0;   // int
	int piece  = 0;  // int

	//ASSERT(317, move_is_ok(move));

	//ASSERT(318, MOVE_IS_PROMOTE(move));

	code = ( (move >> 12) & 3 );
	piece = this.PromotePiece[code];

	if (SQUARE_RANK(MOVE_TO(move)) == Rank8) {
	piece = ( piece | WhiteFlag );
	} else { 
	//ASSERT(319, SQUARE_RANK(MOVE_TO(move))==Rank1);
	piece = ( piece | BlackFlag );
	}

	//ASSERT(320, piece_is_ok(piece));

	return piece;
	}

	// move_order()

	private int move_order( int move ) {

	//ASSERT(321, move_is_ok(move));

	return ( ( (move & V07777) << 2 ) | ( (move >> 12) & 3 ) );
	}

	// move_is_capture()

	private boolean move_is_capture( int move, board_t board ) {

	//ASSERT(322, move_is_ok(move));

	return MOVE_IS_EN_PASSANT(move) || (board.square[MOVE_TO(move)] != Empty);
	}

	// move_is_under_promote()

	private boolean move_is_under_promote( int move ) {

	//ASSERT(324, move_is_ok(move));

	return MOVE_IS_PROMOTE(move) && ( ( move & MoveAllFlags ) != MovePromoteQueen );
	}

	// move_is_tactical()

	private boolean move_is_tactical( int move, board_t board ) {

	//ASSERT(325, move_is_ok(move));

	return ( (move & (1 << 15))!= 0 )  ||  (board.square[MOVE_TO(move)] != Empty); // HACK
	}

	// move_capture()


	private int move_capture( int move, board_t board ) {

	//ASSERT(327, move_is_ok(move));

	if (MOVE_IS_EN_PASSANT(move)) {
	return PAWN_OPP(board.square[MOVE_FROM(move)]);
	}

	return board.square[MOVE_TO(move)];
	}

	// move_to_string()

	private boolean move_to_string( int move, string_t str1 ) {

	string_t str2 = new string_t();

	//ASSERT(329, move==Movenull || move_is_ok(move));

	// null move

	if (move == Movenull) {
	return true;
	}

	// normal moves

	str1.v = "";
	square_to_string( MOVE_FROM(move), str2 );
	str1.v += str2.v;
	square_to_string( MOVE_TO(move), str2 );
	str1.v += str2.v;
	//ASSERT(332, (str1.v.length()==4));

	// promotes

	if (MOVE_IS_PROMOTE(move)) {
	str1.v += ("" + ( piece_to_char(move_promote(move)) ) ).toLowerCase();
	}

	return true;
	}

	// move_from_string()

	private int move_from_string( string_t str1, board_t board ) {

	string_t str2 = new string_t();
	char c = ' ';         // char;

	int from  = 0;        // int
	int to  = 0;          // int
	int move  = 0;        // int
	int piece  = 0;       // int
	int delta  = 0;       // int

	// from

	str2.v = str1.v.substring( 0, 2 );

	from = square_from_string(str2);
	if (from == SquareNone) {
	return MoveNone;
	}

	// to

	str2.v = str1.v.substring( 2, 4 );

	to = square_from_string(str2);
	if (to == SquareNone) {
	return MoveNone;
	}

	move = MOVE_MAKE(from,to);

	// promote

	if( str1.v.length()>4 ) {
	c = str1.v.charAt( 4 );
	if(c=='n') {
	move = ( move | MovePromoteKnight );
	}
	if(c=='b') {
	move = ( move | MovePromoteBishop );
	}
	if(c=='r') {
	move = ( move | MovePromoteRook );
	}
	if(c=='q') {
	move = ( move | MovePromoteQueen );
	}
	}

	// flags

	piece = board.square[from];

	if (PIECE_IS_PAWN(piece)) {
	if (to == board.ep_square) {
	move = ( move | MoveEnPassant );
	}
	} else { 
	if (PIECE_IS_KING(piece)) {
	delta = to - from;
	if (delta == 2  ||  delta == -2) {
	move = ( move | MoveCastle );
	}
	}
	}

	return move;
	}

	// end of move.cpp




	// move_check.cpp



	// gen_quiet_checks()

	private void gen_quiet_checks( list_t list,  board_t board )  {


	//ASSERT(337, ! board_is_check(board));

	list.size=0;

	add_quiet_checks(list,board);
	add_castle_checks(list,board);

	// debug

	//ASSERT(338, list_is_ok(list));
	}

	// add_quiet_checks()

	private void add_quiet_checks( list_t list,  board_t board )  {

	int me  = 0;    // int
	int opp  = 0;   // int
	int king  = 0;  // int

	int ptr  = 0;   // int
	int ptr_2  = 0; // int

	int from  = 0;  // int
	int to  = 0;    // int
	int sq  = 0;    // int

	int piece  = 0;    // int
	int inc_ptr  = 0;  // int
	int inc  = 0;      // int
	int pawn  = 0;   // int
	int rank  = 0;   // int
	int[] pin = new int[9];   // int[8+1]
	boolean gotonextpiece = false;

	// init

	me = board.turn;
	opp = COLOUR_OPP(me);

	king = KING_POS(board,opp);

	find_pins(pin,board);

	// indirect checks

	ptr = 0;
	while(true) {
	from = pin[ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	//ASSERT(341, is_pinned(board,from,opp));

	if (PIECE_IS_PAWN(piece)) {

	inc = PawnMoveInc[me];
	rank = PAWN_RANK(from,me);

	if (rank != Rank7) {    // promotes are generated with captures
	to = from + inc;
	if (board.square[to] == Empty) {
	if (DELTA_INC_LINE(to-king) != DELTA_INC_LINE(from-king)) {
	//ASSERT(342, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	if (rank == Rank2) {
	to = from + (2*inc);
	if (board.square[to] == Empty) {
	//ASSERT(343, DELTA_INC_LINE(to-king)!=DELTA_INC_LINE(from-king));
	//ASSERT(344, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}
	}
	}

	} else { 
	if (PIECE_IS_SLIDER(piece)) {

	inc_ptr = 0;
	while(true) {
	inc = this.PieceInc[piece][inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from+inc;
	while(true) {

	if( board.square[to] != Empty ) {
	break;
	}

	//ASSERT(345, DELTA_INC_LINE(to-king)!=DELTA_INC_LINE(from-king));
	LIST_ADD(list,MOVE_MAKE(from,to));

	to +=  inc;
	}
	inc_ptr++;
	}

	} else { 

	inc_ptr = 0;
	while(true) {
	inc = this.PieceInc[piece][inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from + inc;
	if (board.square[to] == Empty) {
	if (DELTA_INC_LINE(to-king) != DELTA_INC_LINE(from-king)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}

	inc_ptr++;
	}

	}
	}
	ptr++;
	}

	// piece direct checks

	ptr = 1;       // HACK: no king
	while(true) {
	from = board.piece[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	ptr_2 = 0;
	while(true) {
	sq = pin[ptr_2];
	if( sq == SquareNone ) {
	break;
	}

	if (sq == from) {
	gotonextpiece = true;
	break;
	}

	ptr_2++;
	}

	if(gotonextpiece) {

	gotonextpiece = false;

	} else { 

	//ASSERT(346, ! is_pinned(board,from,opp));

	piece = board.square[from];

	if (PIECE_IS_SLIDER(piece)) {

	inc_ptr = 0;
	while(true) {
	inc = this.PieceInc[piece][inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from+inc;
	while(true) {

	if( board.square[to] != Empty ) {
	break;
	}

	if (PIECE_ATTACK(board,piece,to,king)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}

	to +=  inc;
	}
	inc_ptr++;
	}


	} else { 

	inc_ptr = 0;
	while(true) {
	inc = this.PieceInc[piece][inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from + inc;
	if (board.square[to] == Empty) {
	if (PSEUDO_ATTACK(piece,king-to)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}

	inc_ptr++;
	}

	}

	}

	// next_piece:

	ptr++;
	}

	// pawn direct checks

	inc = PawnMoveInc[me];
	pawn = PawnMake[me];

	to = king - (inc-1);
	//ASSERT(347, PSEUDO_ATTACK(pawn,king-to));

	from = to - inc;
	if (board.square[from] == pawn) {
	if (board.square[to] == Empty) {
	//ASSERT(348, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	} else { 
	from = to - (2*inc);
	if (board.square[from] == pawn) {
	if (PAWN_RANK(from,me) == Rank2
	&&  board.square[to] == Empty
	&&  board.square[from+inc] == Empty) {
	//ASSERT(349, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}

	to = king - (inc+1);
	//ASSERT(350, PSEUDO_ATTACK(pawn,king-to));

	from = to - inc;
	if (board.square[from] == pawn) {
	if (board.square[to] == Empty) {
	//ASSERT(351, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	} else { 
	from = to - (2*inc);
	if (board.square[from] == pawn) {
	if (PAWN_RANK(from,me) == Rank2
	&&  board.square[to] == Empty
	&&  board.square[from+inc] == Empty) {
	//ASSERT(352, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}

	}

	// add_castle_checks()

	private void add_castle_checks( list_t list,  board_t board )  {


	//ASSERT(355, ! board_is_check(board));

	if (COLOUR_IS_WHITE(board.turn)) {

	if ( ( board.flags & FlagsWhiteKingCastle) != 0
	&&  board.square[F1] == Empty
	&&  board.square[G1] == Empty
	&&  (! is_attacked(board,F1,Black))) {
	add_check(list,MOVE_MAKE_FLAGS(E1,G1,MoveCastle),board);
	}

	if ( ( board.flags & FlagsWhiteQueenCastle) != 0
	&&  board.square[D1] == Empty
	&&  board.square[C1] == Empty
	&&  board.square[B1] == Empty
	&&  (! is_attacked(board,D1,Black))) {
	add_check(list,MOVE_MAKE_FLAGS(E1,C1,MoveCastle),board);
	}

	} else {  // black

	if ( ( board.flags & FlagsBlackKingCastle) != 0
	&&  board.square[F8] == Empty
	&&  board.square[G8] == Empty
	&&  (! is_attacked(board,F8,White))) {
	add_check(list,MOVE_MAKE_FLAGS(E8,G8,MoveCastle),board);
	}

	if ( ( board.flags & FlagsBlackQueenCastle) != 0
	&&  board.square[D8] == Empty
	&&  board.square[C8] == Empty
	&&  board.square[B8] == Empty
	&&  (! is_attacked(board,D8,White))) {
	add_check(list,MOVE_MAKE_FLAGS(E8,C8,MoveCastle),board);
	}
	}
	}

	// add_check()

	private void add_check( list_t list, int move, board_t board )  {

	undo_t undo = new undo_t();    // undo_t[1];

	//ASSERT(357, move_is_ok(move));

	move_do(board,move,undo);
	if (IS_IN_CHECK(board,board.turn)) {
	LIST_ADD(list,move);
	}
	move_undo(board,move,undo);
	}

	// move_is_check()

	private boolean move_is_check( int move, board_t board ) {

	undo_t undo = new undo_t();    // undo_t[1];

	boolean check = false;   // bool
	int me  = 0;          // int
	int opp  = 0;         // int
	int king  = 0;        // int
	int from  = 0;        // int
	int to  = 0;          // int
	int piece  = 0;       // int

	//ASSERT(359, move_is_ok(move));

	// slow test for complex moves

	if (MOVE_IS_SPECIAL(move)) {

	move_do(board,move,undo);
	check = IS_IN_CHECK(board,board.turn);
	move_undo(board,move,undo);

	return check;
	}

	// init

	me = board.turn;
	opp = COLOUR_OPP(me);
	king = KING_POS(board,opp);

	from = MOVE_FROM(move);
	to = MOVE_TO(move);
	piece = board.square[from];
	//ASSERT(361, COLOUR_IS(piece,me));

	// direct check

	if (PIECE_ATTACK(board,piece,to,king)) {
	return true;
	}

	// indirect check

	if (is_pinned(board,from,opp)
	&&  DELTA_INC_LINE(king-to) != DELTA_INC_LINE(king-from)) {
	return true;
	}

	return false;
	}

	// find_pins()

	private void find_pins( int[] list, board_t board )  {

	int me  = 0;    // int
	int opp  = 0;   // int
	int king  = 0;  // int
	int ptr  = 0;   // int
	int from  = 0;  // int
	int piece  = 0; // int
	int delta  = 0; // int
	int inc  = 0;   // int
	int sq  = 0;    // int
	int pin  = 0;   // int
	int capture  = 0;   // int
	int q  = 0;         // int

	// init

	me = board.turn;
	opp = COLOUR_OPP(me);

	king = KING_POS(board,opp);

	ptr = 1;            // HACK: no king
	while(true) {
	from = board.piece[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	delta = king - from;
	//ASSERT(364, delta_is_ok(delta));

	if (PSEUDO_ATTACK(piece,delta)) {

	//ASSERT(365, PIECE_IS_SLIDER(piece));

	inc = DELTA_INC_LINE(delta);
	//ASSERT(366, inc!=IncNone);

	//ASSERT(367, SLIDER_ATTACK(piece,inc));

	sq = from;

	while(true) {
	sq+=inc;
	capture = board.square[sq];
	if( capture != Empty ) {
	break;
	}
	}

	//ASSERT(368, sq!=king);

	if (COLOUR_IS(capture,me)) {
	pin = sq;

	while(true) {
	sq+=inc;

	if( board.square[sq] != Empty ) {
	break;
	}
	}

	if (sq == king) {

	list[q] = pin;
	q++;

	}
	}
	}

	ptr++;
	}

	list[q] = SquareNone;
	}

	// end of move_check.cpp



	// move_do.cpp


	private void initCmsk( int sq, int flagMask ) { 
	this.CastleMask[sq] = ( this.CastleMask[sq] & (~ flagMask ) );
	}

	// move_do_init()

	private void move_do_init() { 

	int sq  = 0;   // int

	for (sq = 0; sq<SquareNb; sq++ ) {
	this.CastleMask[sq] = 0xF;
	}

	initCmsk( E1, FlagsWhiteKingCastle );
	initCmsk( H1, FlagsWhiteKingCastle );

	initCmsk( E1, FlagsWhiteQueenCastle );
	initCmsk( A1, FlagsWhiteQueenCastle );

	initCmsk( E8, FlagsBlackKingCastle );
	initCmsk( H8, FlagsBlackKingCastle );

	initCmsk( E8, FlagsBlackQueenCastle );
	initCmsk( A8, FlagsBlackQueenCastle );

	}

	// move_do()

	private void move_do( board_t board, int move, undo_t undo )  {

	int me  = 0;        // int
	int opp  = 0;       // int
	int from  = 0;      // int
	int to  = 0;        // int
	int piece  = 0;     // int
	int pos  = 0;       // int
	int capture  = 0;   // int
	int old_flags  = 0; // int
	int new_flags  = 0; // int

	int delta  = 0;  // int
	int sq  = 0;     // int
	int pawn  = 0;   // int
	int rook  = 0;   // int

	//ASSERT(370, move_is_ok(move));

	//ASSERT(372, board_is_legal(board));

	// initialise undo

	undo.capture = false;

	undo.turn = board.turn;
	undo.flags = board.flags;
	undo.ep_square = board.ep_square;
	undo.ply_nb = board.ply_nb;

	undo.cap_sq = board.cap_sq;

	undo.opening = board.opening;
	undo.endgame = board.endgame;

	undo.key = board.key;
	undo.pawn_key = board.pawn_key;
	undo.material_key = board.material_key;

	// init

	me = board.turn;
	opp = COLOUR_OPP(me);

	from = MOVE_FROM(move);
	to = MOVE_TO(move);

	piece = board.square[from];

	//ASSERT(373, COLOUR_IS(piece,me));

	// update key stack

	//ASSERT(374, board.sp<StackSize);
	board.stack[board.sp] = board.key;
	board.sp++;

	// update turn

	board.turn = opp;


	// update castling rights

	old_flags = board.flags;
	new_flags = ( ( old_flags & this.CastleMask[from] ) & this.CastleMask[to] );

	board.flags = new_flags;


	// update en-passant square

	sq = board.ep_square;
	if (sq != SquareNone) {

	board.ep_square = SquareNone;
	}

	if (PIECE_IS_PAWN(piece)) {

	delta = to - from;

	if (delta == 32  ||  delta == -32) {
	pawn = PawnMake[opp];
	if (board.square[to-1] == pawn  ||  board.square[to+1] == pawn) {
	board.ep_square = (from + to) / 2;
	}
	}
	}

	// update move number (captures are handled later)

	board.ply_nb++;
	if (PIECE_IS_PAWN(piece)) {
	board.ply_nb = 0; // conversion
	}

	// update last square

	board.cap_sq = SquareNone;

	// remove the captured piece

	sq = to;
	if (MOVE_IS_EN_PASSANT(move)) {
	sq = SQUARE_EP_DUAL(sq);
	}

	capture=board.square[sq];
	if (capture!= Empty) {

	//ASSERT(375, COLOUR_IS(capture,opp));
	//ASSERT(376, ! PIECE_IS_KING(capture));

	undo.capture = true;
	undo.capture_square = sq;
	undo.capture_piece = capture;
	undo.capture_pos = board.pos[sq];

	square_clear(board,sq,capture,true);

	board.ply_nb = 0; // conversion
	board.cap_sq = to;
	}

	// move the piece

	if (MOVE_IS_PROMOTE(move)) {

	// promote

	undo.pawn_pos = board.pos[from];

	square_clear(board,from,piece,true);

	piece = move_promote(move);

	// insert the promote piece in MV order

	pos = board.piece_size[me];
	while( pos > 0  &&  piece > board.square[board.piece[me][pos-1]] ) {
	pos--;   // HACK
	}

	square_set(board,to,piece,pos,true);

	board.cap_sq = to;

	} else { 

	// normal move

	square_move(board,from,to,piece,true);
	}

	// move the rook in case of castling

	if (MOVE_IS_CASTLE(move)) {

	rook =  ( Rook64 | COLOUR_FLAG(me) ); // HACK

	if (to == G1) {
	square_move(board,H1,F1,rook,true);
	} else { 
	if (to == C1) {
	square_move(board,A1,D1,rook,true);
	} else { 
	if (to == G8) {
	square_move(board,H8,F8,rook,true);
	} else { 
	if (to == C8) {
	square_move(board,A8,D8,rook,true);
	} else { 
	//ASSERT(377, false);
	}
	}
	}
	}
	}

	// debug

	//ASSERT(378, board_is_ok(board));

	}

	// move_undo()

	private void move_undo( board_t board, int move, undo_t undo )  {

	int me  = 0;    // int
	int from  = 0;  // int
	int to  = 0;    // int
	int piece  = 0; // int
	int pos  = 0;   // int
	int rook  = 0;  // int

	//ASSERT(380, move_is_ok(move));

	// init

	me = undo.turn;

	from = MOVE_FROM(move);
	to = MOVE_TO(move);

	piece = board.square[to];
	//ASSERT(382, COLOUR_IS(piece,me));

	// castle

	if (MOVE_IS_CASTLE(move)) {

	rook =  ( Rook64 | COLOUR_FLAG(me) ); // HACK

	if (to == G1) {
	square_move(board,F1,H1,rook,false);
	} else { 
	if (to == C1) {
	square_move(board,D1,A1,rook,false);
	} else { 
	if (to == G8) {
	square_move(board,F8,H8,rook,false);
	} else { 
	if (to == C8) {
	square_move(board,D8,A8,rook,false);
	} else { 
	//ASSERT(383, false);
	}
	}
	}
	}
	}



	// move the piece backward

	if (MOVE_IS_PROMOTE(move)) {

	// promote

	//ASSERT(384, piece==move_promote(move));
	square_clear(board,to,piece,false);

	piece = PawnMake[me];
	pos = undo.pawn_pos;

	square_set(board,from,piece,pos,false);

	} else { 

	// normal move

	square_move(board,to,from,piece,false);
	}

	// put the captured piece back

	if (undo.capture) {
	square_set(board,undo.capture_square,undo.capture_piece,undo.capture_pos,false);
	}

	// update board info

	board.turn = undo.turn;
	board.flags = undo.flags;
	board.ep_square = undo.ep_square;
	board.ply_nb = undo.ply_nb;

	board.cap_sq = undo.cap_sq;

	board.opening = undo.opening;
	board.endgame = undo.endgame;

	board.key = undo.key;
	board.pawn_key = undo.pawn_key;
	board.material_key = undo.material_key;

	// update key stack

	//ASSERT(385, board.sp>0);
	board.sp--;

	// debug

	//ASSERT(386, board_is_ok(board));
	//ASSERT(387, board_is_legal(board));
	}

	// move_do_null()

	private void move_do_null( board_t board, undo_t undo )   {

	int sq  = 0;   // int

	//ASSERT(390, board_is_legal(board));
	//ASSERT(391, ! board_is_check(board));

	// initialise undo

	undo.turn = board.turn;
	undo.ep_square = board.ep_square;
	undo.ply_nb = board.ply_nb;
	undo.cap_sq = board.cap_sq;
	undo.key = board.key;

	// update key stack

	//ASSERT(392, board.sp<StackSize);
	board.stack[board.sp] = board.key;
	board.sp++;

	// update turn

	board.turn = COLOUR_OPP(board.turn);

	// update en-passant square

	sq = board.ep_square;
	if (sq != SquareNone) {

	board.ep_square = SquareNone;
	}

	// update move number

	board.ply_nb = 0; // HACK: null move is considered as a conversion

	// update last square

	board.cap_sq = SquareNone;

	// debug

	//ASSERT(393, board_is_ok(board));
	}

	// move_undo_null()

	private void move_undo_null( board_t board, undo_t undo )   {

	//ASSERT(396, board_is_legal(board));
	//ASSERT(397, ! board_is_check(board));

	// update board info

	board.turn = undo.turn;
	board.ep_square = undo.ep_square;
	board.ply_nb = undo.ply_nb;
	board.cap_sq = undo.cap_sq;
	board.key = undo.key;

	// update key stack

	//ASSERT(398, board.sp>0);
	board.sp--;

	// debug

	//ASSERT(399, board_is_ok(board));
	}

	// square_clear()

	private void square_clear( board_t board, int square, int piece, boolean update )  {

	int pos  = 0;       // int
	int piece_12  = 0;  // int
	int colour  = 0;    // int
	int sq  = 0;        // int
	int i  = 0;         // int
	int size  = 0;      // int
	int sq_64  = 0;     // int
	int t  = 0;         // int
	int hash_xor  = 0;  // uint64

	//ASSERT(401, SQUARE_IS_OK(square));
	//ASSERT(402, piece_is_ok(piece));

	// init

	pos = board.pos[square];
	//ASSERT(404, pos>=0);

	piece_12 = this.PieceTo12[piece];
	colour = PIECE_COLOUR(piece);

	// square

	//ASSERT(405, board.square[square]==piece);
	board.square[square] = Empty;

	// piece list

	if (! PIECE_IS_PAWN(piece)) {

	// init

	size = board.piece_size[colour];
	//ASSERT(406, size>=1);

	// stable swap

	//ASSERT(407, pos>=0 && pos<size);

	//ASSERT(408, board.pos[square]==pos);
	board.pos[square] = -1;

	for (i = pos; i<= size-2; i++ ) {

	sq = board.piece[colour][i+1];

	board.piece[colour][i] = sq;

	//ASSERT(409, board.pos[sq]==i+1);
	board.pos[sq] = i;
	}

	// size

	size--;

	board.piece[colour][size] = SquareNone;
	board.piece_size[colour] = size;

	} else { 

	// init

	size = board.pawn_size[colour];
	//ASSERT(410, size>=1);

	// stable swap

	//ASSERT(411, pos>=0 && pos<size);

	//ASSERT(412, board.pos[square]==pos);
	board.pos[square] = -1;

	for (i = pos; i<= size-2; i++ ) {

	sq = board.pawn[colour][i+1];

	board.pawn[colour][i] = sq;

	//ASSERT(413, board.pos[sq]==i+1);
	board.pos[sq] = i;
	}

	// size

	size--;

	board.pawn[colour][size] = SquareNone;
	board.pawn_size[colour] = size;

	// pawn "bitboard"

	t = SQUARE_FILE(square);
	board.pawn_file[colour][t] = ( board.pawn_file[colour][t] ^ 
	this.BitEQ[PAWN_RANK(square,colour)] );
	}

	// material

	//ASSERT(414, board.piece_nb>0);
	board.piece_nb--;

	//ASSERT(415, board.number[piece_12]>0);
	board.number[piece_12]--;

	// update

	if (update) {

	// init

	sq_64 = this.SquareTo64[square];

	// PST

	board.opening -= Pget( piece_12, sq_64, Opening );
	board.endgame -= Pget( piece_12, sq_64, Endgame );

	// hash key

	hash_xor = this.Random64[RandomPiece+((piece_12 ^ 1)*64)+sq_64];
	// HACK: xor 1 for PolyGlot book (not AS3)

	board.key ^= hash_xor;
	if (PIECE_IS_PAWN(piece)) {
	board.pawn_key ^= hash_xor;
	}

	// material key

	board.material_key ^= this.Random64[(piece_12*16)+board.number[piece_12]];


	}
	}

	// square_set()

	private void square_set( board_t board, int square, int piece, int pos, boolean update )  {

	int piece_12  = 0;  // int
	int colour  = 0;    // int
	int sq  = 0;        // int
	int i  = 0;         // int
	int size  = 0;      // int
	int sq_64  = 0;     // int
	int t  = 0;         // int
	int hash_xor  = 0;  // uint64


	//ASSERT(417, SQUARE_IS_OK(square));
	//ASSERT(418, piece_is_ok(piece));
	//ASSERT(419, pos>=0);


	// init

	piece_12 = this.PieceTo12[piece];
	colour = PIECE_COLOUR(piece);

	// square

	//ASSERT(421, board.square[square]==Empty);
	board.square[square] = piece;

	// piece list

	if (! PIECE_IS_PAWN(piece)) {

	// init

	size = board.piece_size[colour];
	//ASSERT(422, size>=0);

	// size

	size++;

	board.piece[colour][size] = SquareNone;
	board.piece_size[colour] = size;

	// stable swap

	//ASSERT(423, pos>=0 && pos<size);

	for (i = size-1; i>= pos+1; i-- ) {

	sq = board.piece[colour][i-1];

	board.piece[colour][i] = sq;

	//ASSERT(424, board.pos[sq]==i-1);
	board.pos[sq] = i;
	}

	board.piece[colour][pos] = square;

	//ASSERT(425, board.pos[square]==-1);
	board.pos[square] = pos;

	} else { 

	// init

	size = board.pawn_size[colour];
	//ASSERT(426, size>=0);

	// size

	size++;

	board.pawn[colour][size] = SquareNone;
	board.pawn_size[colour] = size;

	// stable swap

	//ASSERT(427, pos>=0 && pos<size);

	for (i = size-1; i>= pos+1; i-- ) {

	sq = board.pawn[colour][i-1];

	board.pawn[colour][i] = sq;

	//ASSERT(428, board.pos[sq]==i-1);
	board.pos[sq] = i;
	}

	board.pawn[colour][pos] = square;

	//ASSERT(429, board.pos[square]==-1);
	board.pos[square] = pos;

	// pawn "bitboard"

	t = SQUARE_FILE(square);
	board.pawn_file[colour][t] = ( board.pawn_file[colour][t] ^ 
	this.BitEQ[PAWN_RANK(square,colour)] );


	}

	// material

	//ASSERT(430, board.piece_nb<32);
	board.piece_nb++;

	//ASSERT(431, board.number[piece_12]<9);
	board.number[piece_12]++;

	// update

	if (update) {

	// init

	sq_64 = this.SquareTo64[square];

	// PST

	board.opening += Pget( piece_12, sq_64, Opening );
	board.endgame += Pget( piece_12, sq_64, Endgame );
	// hash key

	hash_xor = this.Random64[RandomPiece+((piece_12 ^ 1)*64)+sq_64];
	// HACK: xor 1 for PolyGlot book (not AS3)

	board.key ^= hash_xor;
	if (PIECE_IS_PAWN(piece)) {
	board.pawn_key ^= hash_xor;
	}

	// material key

	board.material_key ^= this.Random64[(piece_12*16)+board.number[piece_12]];

	}
	}

	// square_move()

	private void square_move( board_t board, int from, int to, int piece, boolean update )  {

	int piece_12  = 0;    // int
	int colour  = 0;      // int
	int pos  = 0;         // int
	int from_64  = 0;     // int
	int to_64  = 0;       // int
	int piece_index  = 0; // int
	int t  = 0;           // int
	int hash_xor  = 0;    // uint64



	//ASSERT(433, SQUARE_IS_OK(from));
	//ASSERT(434, SQUARE_IS_OK(to));
	//ASSERT(435, piece_is_ok(piece));


	// init

	colour = PIECE_COLOUR(piece);

	pos = board.pos[from];
	//ASSERT(437, pos>=0);

	// from

	//ASSERT(438, board.square[from]==piece);
	board.square[from] = Empty;

	//ASSERT(439, board.pos[from]==pos);
	board.pos[from] = -1; // not needed

	// to

	//ASSERT(440, board.square[to]==Empty);
	board.square[to] = piece;

	//ASSERT(441, board.pos[to]==-1);
	board.pos[to] = pos;

	// piece list

	if (! PIECE_IS_PAWN(piece)) {

	//ASSERT(442, board.piece[colour][pos]==from);
	board.piece[colour][pos] = to;

	} else { 

	//ASSERT(443, board.pawn[colour][pos]==from);
	board.pawn[colour][pos] = to;

	// pawn "bitboard"

	t = SQUARE_FILE(from);
	board.pawn_file[colour][t] = ( board.pawn_file[colour][t] ^ 
	this.BitEQ[PAWN_RANK(from,colour)] );
	t = SQUARE_FILE(to);
	board.pawn_file[colour][t] = ( board.pawn_file[colour][t] ^ 
	this.BitEQ[PAWN_RANK(to,colour)] );

	}

	// update

	if (update) {

	// init

	from_64 = this.SquareTo64[from];
	to_64 = this.SquareTo64[to];
	piece_12 = this.PieceTo12[piece];

	// PST

	board.opening += Pget(piece_12,to_64,Opening) - Pget(piece_12,from_64,Opening);
	board.endgame += Pget(piece_12,to_64,Endgame) - Pget(piece_12,from_64,Endgame);

	// hash key

	piece_index = RandomPiece + ((piece_12 ^ 1) * 64);
	// HACK: xor 1 for PolyGlot book (not AS3)

	hash_xor =  ( this.Random64[piece_index+to_64] ^ this.Random64[piece_index+from_64] );

	board.key ^= hash_xor;
	if (PIECE_IS_PAWN(piece)) {
	board.pawn_key ^= hash_xor;
	}

	}

	}

	// end of move_do.cpp



	// move_evasion.cpp



	// gen_legal_evasions()

	private void gen_legal_evasions( list_t list, board_t board, attack_t attack )  {

	gen_evasions(list,board,attack,true,false);

	// debug

	//ASSERT(447, list_is_ok(list));
	}

	// gen_pseudo_evasions()

	private void gen_pseudo_evasions( list_t list, board_t board, attack_t attack )  {

	gen_evasions(list,board,attack,false,false);

	// debug

	//ASSERT(451, list_is_ok(list));
	}

	// legal_evasion_exist()

	private boolean legal_evasion_exist( board_t board, attack_t attack ) {

	list_t list = new list_t();  // list[1] dummy

	return gen_evasions(list,board,attack,true,true);
	}

	// gen_evasions()

	private boolean gen_evasions( list_t list, board_t board, attack_t attack, boolean legal, boolean stop ) {
	int me  = 0;         // int
	int opp  = 0;        // int
	int opp_flag  = 0;   // int
	int king  = 0;       // int
	int inc_ptr  = 0;    // int
	int inc  = 0;        // int
	int to  = 0;         // int
	int piece  = 0;      // int


	//ASSERT(459, board_is_check(board));
	//ASSERT(460, ATTACK_IN_CHECK(attack));

	// init

	list.size=0;

	me = board.turn;
	opp = COLOUR_OPP(me);

	opp_flag = COLOUR_FLAG(opp);

	king = KING_POS(board,me);

	inc_ptr = 0;
	while(true) {
	inc = KingInc[inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	// avoid escaping along a check line
	if (inc != -attack.di[0]  &&  inc != -attack.di[1]) {
	to = king + inc;
	piece = board.square[to];
	if (piece == Empty  ||  FLAG_IS(piece,opp_flag)) {
	if ((! legal ) || (! is_attacked(board,to,opp))) {
	if (stop) {
	return true;
	}
	LIST_ADD(list,MOVE_MAKE(king,to));
	}
	}
	}

	inc_ptr++;
	}


	if (attack.dn >= 2) {
	return false; // double check, we are {ne
	}

	// single check

	//ASSERT(461, attack.dn==1);

	// capture the checking piece

	if (add_pawn_captures(list,board,attack.ds[0],legal,stop)  &&  stop) {
	return true;
	}
	if (add_piece_moves(list,board,attack.ds[0],legal,stop)  &&  stop) {
	return true;
	}

	// interpose a piece

	inc = attack.di[0];

	if (inc != IncNone) { // line
	to = king+inc;
	while( to != attack.ds[0] ) {

	//ASSERT(462, SQUARE_IS_OK(to));
	//ASSERT(463, board.square[to]==Empty);
	if (add_pawn_moves(list,board,to,legal,stop)  &&  stop) {
	return true;
	}
	if (add_piece_moves(list,board,to,legal,stop)  &&  stop) {
	return true;
	}
	to +=  inc;
	}
	}

	return false;

	}

	// add_pawn_moves()

	private boolean add_pawn_moves( list_t list, board_t board, int to, boolean legal, boolean stop ) {
	int me  = 0;      // int
	int inc  = 0;     // int
	int pawn  = 0;    // int
	int from  = 0;    // int
	int piece  = 0;   // int

	//ASSERT(466, SQUARE_IS_OK(to));

	//ASSERT(469, board.square[to]==Empty);

	me = board.turn;

	inc = PawnMoveInc[me];
	pawn = PawnMake[me];

	from = to - inc;
	piece = board.square[from];

	if (piece == pawn) {  // single push

	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	add_pawn_move(list,from,to);
	}

	} else { 
	if (piece == Empty  &&  PAWN_RANK(to,me) == Rank4)  {   // double push

	from = to - (2*inc);
	if (board.square[from] == pawn) {
	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	//ASSERT(470, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}
	}

	return false;
	}

	// add_pawn_captures()

	private boolean add_pawn_captures( list_t list, board_t board, int to, boolean legal, boolean stop ) { 
	int me  = 0;     // int
	int inc  = 0;    // int
	int pawn  = 0;   // int
	int from  = 0;   // int

	//ASSERT(473, SQUARE_IS_OK(to));

	//ASSERT(476, COLOUR_IS(board.square[to],COLOUR_OPP(board.turn)));

	me = board.turn;

	inc = PawnMoveInc[me];
	pawn = PawnMake[me];

	from = to - (inc-1);
	if (board.square[from] == pawn) {
	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	add_pawn_move(list,from,to);
	}
	}

	from = to - (inc+1);
	if (board.square[from] == pawn) {
	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	add_pawn_move(list,from,to);
	}
	}

	if (board.ep_square != SquareNone &&  to == SQUARE_EP_DUAL(board.ep_square)) {

	//ASSERT(477, PAWN_RANK(to,me)==Rank5);
	//ASSERT(478, PIECE_IS_PAWN(board.square[to]));

	to = board.ep_square;
	//ASSERT(479, PAWN_RANK(to,me)==Rank6);
	//ASSERT(480, board.square[to]==Empty);

	from = to - (inc-1);
	if (board.square[from] == pawn) {
	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	//ASSERT(481, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE_FLAGS(from,to,MoveEnPassant));
	}
	}

	from = to - (inc+1);
	if (board.square[from] == pawn) {
	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	//ASSERT(482, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE_FLAGS(from,to,MoveEnPassant));
	}
	}
	}

	return false;
	}

	// add_piece_moves()

	private boolean add_piece_moves( list_t list, board_t board, int to, boolean legal, boolean stop ) {
	int me  = 0;      // int
	int ptr  = 0;     // int
	int from  = 0;    // int
	int piece  = 0;   // int

	//ASSERT(485, SQUARE_IS_OK(to));

	me = board.turn;

	ptr = 1;            // HACK: no king
	while(true) {
	from = board.piece[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	if (PIECE_ATTACK(board,piece,from,to)) {
	if ((! legal)  ||  (! is_pinned(board,from,me))) {
	if (stop) {
	return true;
	}
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}

	ptr++;
	}

	return false;

	}

	// end of move_evasion.cpp



	// move_gen.cpp



	// gen_legal_moves()

	private void gen_legal_moves( list_t list, board_t board )   {

	attack_t attack = new attack_t();  // attack_t[1]

	attack_set(attack,board);

	if (ATTACK_IN_CHECK(attack)) {
	gen_legal_evasions(list,board,attack);
	} else { 
	gen_moves(list,board);
	list_filter(list,board, true);
	}

	// debug

	//ASSERT(490, list_is_ok(list));
	}

	// gen_moves()

	private void gen_moves( list_t list, board_t board )  {

	//ASSERT(493, ! board_is_check(board));

	list.size=0;

	add_moves(list,board);

	add_en_passant_captures(list,board);
	add_castle_moves(list,board);

	// debug

	//ASSERT(494, list_is_ok(list));
	}

	// gen_captures()

	private void gen_captures( list_t list, board_t board )  {

	list.size=0;

	add_captures(list,board);
	add_en_passant_captures(list,board);

	// debug

	//ASSERT(497, list_is_ok(list));
	}

	// gen_quiet_moves()

	private void gen_quiet_moves( list_t list, board_t board )  {

	//ASSERT(500, ! board_is_check(board));

	list.size=0;

	add_quiet_moves(list,board);
	add_castle_moves(list,board);

	// debug

	//ASSERT(501, list_is_ok(list));
	}

	// add_moves()

	private void add_moves( list_t list, board_t board )  {

	int me  = 0;         // int
	int opp  = 0;        // int
	int opp_flag  = 0;   // int
	int ptr  = 0;        // int
	int from  = 0;       // int
	int to  = 0;         // int
	int piece  = 0;      // int
	int capture  = 0;    // int
	int inc_ptr  = 0;    // int
	int inc  = 0;        // int

	me = board.turn;
	opp = COLOUR_OPP(me);

	opp_flag = COLOUR_FLAG(opp);

	// piece moves

	ptr = 0;
	while(true) {
	from = board.piece[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	if (PIECE_IS_SLIDER(piece)) {

	inc_ptr = 0;
	while(true) {
	inc = this.PieceInc[piece][inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from+inc;
	while(true) {
	capture=board.square[to];
	if( capture != Empty ) {
	break;
	}

	LIST_ADD(list,MOVE_MAKE(from,to));

	to +=  inc;
	}

	if (FLAG_IS(capture,opp_flag)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}

	inc_ptr++;
	}

	} else { 

	inc_ptr = 0;
	while(true) {
	inc = this.PieceInc[piece][inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from + inc;
	capture = board.square[to];
	if (capture == Empty  ||  FLAG_IS(capture,opp_flag)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}

	inc_ptr++;
	}

	}

	ptr++;
	}


	// pawn moves

	inc = PawnMoveInc[me];

	ptr = 0;
	while(true) {
	from = board.pawn[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	to = from + (inc-1);
	if (FLAG_IS(board.square[to],opp_flag)) {
	add_pawn_move(list,from,to);
	}

	to = from + (inc+1);
	if (FLAG_IS(board.square[to],opp_flag)) {
	add_pawn_move(list,from,to);
	}

	to = from + inc;
	if (board.square[to] == Empty) {
	add_pawn_move(list,from,to);
	if (PAWN_RANK(from,me) == Rank2) {
	to = from + (2*inc);
	if (board.square[to] == Empty) {
	//ASSERT(504, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}

	ptr++;
	}

	}

	//
	private void add_capt1 ( int from, int dt, list_t list, board_t board, int opp_flag ) { 
	int to = from + dt;
	if (FLAG_IS(board.square[to],opp_flag)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}


	//
	private void add_capt2( int from, int dt, list_t list, board_t board, int opp_flag ) { 
	int to = from + dt;
	int capture = 0;
	while(true) {
	capture=board.square[to];
	if(capture!=Empty) {
	break;
	}
	to +=  dt;
	}
	if (FLAG_IS(capture,opp_flag)) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}

	//
	private void add_capt3( int from, int dt, list_t list, board_t board, int opp_flag )  { 
	int to = from + dt;
	if (FLAG_IS(board.square[to],opp_flag)) {
	add_pawn_move(list,from,to);
	}
	}

	//
	private void add_capt4( int from, int dt, list_t list, board_t board ) { 
	int to = from + dt;
	if (board.square[to] == Empty) {
	add_promote(list,MOVE_MAKE(from,to));
	}
	}

	// add_captures()

	private void add_captures( list_t list, board_t board )  {

	int me  = 0;         // int
	int opp  = 0;        // int
	int opp_flag  = 0;   // int
	int ptr  = 0;        // int
	int from  = 0;       // int
	int piece  = 0;      // int
	int p = 0;

	me = board.turn;
	opp = COLOUR_OPP(me);

	opp_flag = COLOUR_FLAG(opp);

	// piece captures

	ptr = 0;
	while(true) {
	from = board.piece[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	p = PIECE_TYPE(piece);

	if(p == Knight64) {

	add_capt1 ( from, -33, list, board, opp_flag );
	add_capt1 ( from, -31, list, board, opp_flag );
	add_capt1 ( from, -18, list, board, opp_flag );
	add_capt1 ( from, -14, list, board, opp_flag );
	add_capt1 ( from, 14, list, board, opp_flag );
	add_capt1 ( from, 18, list, board, opp_flag );
	add_capt1 ( from, 31, list, board, opp_flag );
	add_capt1 ( from, 33, list, board, opp_flag );
	} else { 

	if(p == Bishop64) {

	add_capt2 ( from, -17, list, board, opp_flag );
	add_capt2 ( from, -15, list, board, opp_flag );
	add_capt2 ( from, 15, list, board, opp_flag );
	add_capt2 ( from, 17, list, board, opp_flag );

	} else { 

	if(p == Rook64) {

	add_capt2 ( from, -16, list, board, opp_flag );
	add_capt2 ( from, -1, list, board, opp_flag );
	add_capt2 ( from, 1, list, board, opp_flag );
	add_capt2 ( from, 16, list, board, opp_flag );

	} else { 

	if(p == Queen64) {

	add_capt2 ( from, -17, list, board, opp_flag );
	add_capt2 ( from, -16, list, board, opp_flag );
	add_capt2 ( from, -15, list, board, opp_flag );
	add_capt2 ( from, -1, list, board, opp_flag );
	add_capt2 ( from, 1, list, board, opp_flag );
	add_capt2 ( from, 15, list, board, opp_flag );
	add_capt2 ( from, 16, list, board, opp_flag );
	add_capt2 ( from, 17, list, board, opp_flag );

	} else { 

	if(p == King64) {

	add_capt1 ( from, -17, list, board, opp_flag );
	add_capt1 ( from, -16, list, board, opp_flag );
	add_capt1 ( from, -15, list, board, opp_flag );
	add_capt1 ( from, -1, list, board, opp_flag );
	add_capt1 ( from, 1, list, board, opp_flag );
	add_capt1 ( from, 15, list, board, opp_flag );
	add_capt1 ( from, 16, list, board, opp_flag );
	add_capt1 ( from, 17, list, board, opp_flag );

	} else { 

	//ASSERT(507, false);

	}
	}
	}
	}
	}

	ptr++;
	}

	// pawn captures

	if (COLOUR_IS_WHITE(me)) {

	ptr = 0;
	while(true) {
	from = board.pawn[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	add_capt3 ( from, 15, list, board, opp_flag );
	add_capt3 ( from, 17, list, board, opp_flag );

	// promote

	if (SQUARE_RANK(from) == Rank7) {
	add_capt4 ( from, 16, list, board );
	}

	ptr++;
	}

	} else {  // black

	ptr = 0;
	while(true) {
	from = board.pawn[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	add_capt3 ( from, -17, list, board, opp_flag );
	add_capt3 ( from, -15, list, board, opp_flag );

	// promote

	if (SQUARE_RANK(from) == Rank2) {
	add_capt4 ( from, -16, list, board );
	}

	ptr++;
	}

	}

	}


	//
	private void add_quietm1( int from, int dt, list_t list, board_t board ) { 
	int to = from + dt;
	if (board.square[to] == Empty) {
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}

	//
	private void add_quietm2( int from, int dt, list_t list, board_t board ) { 
	int to = from + dt;
	while(true) {
	if(board.square[to]!=Empty) {
	break;
	}
	LIST_ADD(list,MOVE_MAKE(from,to));
	to +=  dt;
	}
	}

	// add_quiet_moves()

	private void add_quiet_moves( list_t list, board_t board )  {

	int me  = 0;         // int
	int ptr  = 0;        // int
	int from  = 0;       // int
	int to  = 0;         // int
	int piece  = 0;      // int
	int p = 0;


	me = board.turn;

	// piece moves

	ptr = 0;
	while(true) {
	from = board.piece[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	piece = board.square[from];

	p = PIECE_TYPE(piece);

	if(p == Knight64) {

	add_quietm1 ( from, -33, list, board );
	add_quietm1 ( from, -31, list, board );
	add_quietm1 ( from, -18, list, board );
	add_quietm1 ( from, -14, list, board );
	add_quietm1 ( from, 14, list, board );
	add_quietm1 ( from, 18, list, board );
	add_quietm1 ( from, 31, list, board );
	add_quietm1 ( from, 33, list, board );
	} else { 

	if(p == Bishop64) {

	add_quietm2 ( from, -17, list, board );
	add_quietm2 ( from, -15, list, board );
	add_quietm2 ( from, 15, list, board );
	add_quietm2 ( from, 17, list, board );

	} else { 

	if(p == Rook64) {

	add_quietm2 ( from, -16, list, board );
	add_quietm2 ( from, -1, list, board );
	add_quietm2 ( from, 1, list, board );
	add_quietm2 ( from, 16, list, board );

	} else { 

	if(p == Queen64) {

	add_quietm2 ( from, -17, list, board );
	add_quietm2 ( from, -16, list, board );
	add_quietm2 ( from, -15, list, board );
	add_quietm2 ( from, -1, list, board );
	add_quietm2 ( from, 1, list, board );
	add_quietm2 ( from, 15, list, board );
	add_quietm2 ( from, 16, list, board );
	add_quietm2 ( from, 17, list, board );

	} else { 

	if(p == King64) {

	add_quietm1 ( from, -17, list, board );
	add_quietm1 ( from, -16, list, board );
	add_quietm1 ( from, -15, list, board );
	add_quietm1 ( from, -1, list, board );
	add_quietm1 ( from, 1, list, board );
	add_quietm1 ( from, 15, list, board );
	add_quietm1 ( from, 16, list, board );
	add_quietm1 ( from, 17, list, board );

	} else { 

	//ASSERT(510, false);

	}
	}
	}
	}
	}

	ptr++;
	}

	// pawn moves

	if (COLOUR_IS_WHITE(me)) {

	ptr = 0;
	while(true) {
	from = board.pawn[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	// non promotes

	if (SQUARE_RANK(from) != Rank7) {
	to = from + 16;
	if (board.square[to] == Empty) {
	//ASSERT(511, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	if (SQUARE_RANK(from) == Rank2) {
	to = from + 32;
	if (board.square[to] == Empty) {
	//ASSERT(512, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}
	}

	ptr++;
	}

	} else {  // black

	ptr = 0;
	while(true) {
	from = board.pawn[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	// non promotes

	if (SQUARE_RANK(from) != Rank2) {
	to = from - 16;
	if (board.square[to] == Empty) {
	//ASSERT(513, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	if (SQUARE_RANK(from) == Rank7) {
	to = from - 32;
	if (board.square[to] == Empty) {
	//ASSERT(514, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE(from,to));
	}
	}
	}
	}

	ptr++;
	}

	}

	}

	// add_promotes()
/*
	private void add_promotes( list_t list, board_t board )  {

	int me  = 0;    // int
	int inc  = 0;   // int
	int ptr  = 0;   // int
	int from  = 0;  // int
	int to  = 0;    // int

	me = board.turn;

	inc = PawnMoveInc[me];

	ptr = 0;
	while(true) {
	from = board.pawn[me][ptr];
	if( from == SquareNone ) {
	break;
	}

	if (PAWN_RANK(from,me) == Rank7) {
	add_capt4 ( from, inc, list, board );
	to = from + inc;
	}

	ptr++;
	}
	}
*/
	
	// add_en_passant_captures()

	private void add_en_passant_captures( list_t list, board_t board )  {

	int from  = 0;  // int
	int to  = 0;    // int
	int me  = 0;    // int
	int inc  = 0;   // int
	int pawn  = 0;  // int

	to = board.ep_square;

	if (to != SquareNone) {

	me = board.turn;

	inc = PawnMoveInc[me];
	pawn = PawnMake[me];

	from = to - (inc-1);
	if (board.square[from] == pawn) {
	//ASSERT(519, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE_FLAGS(from,to,MoveEnPassant));
	}

	from = to - (inc+1);
	if (board.square[from] == pawn) {
	//ASSERT(520, ! this.SquareIsPromote[to]);
	LIST_ADD(list,MOVE_MAKE_FLAGS(from,to,MoveEnPassant));
	}

	}

	}

	// add_castle_moves()

	private void add_castle_moves( list_t list, board_t board )  {

	//ASSERT(523, ! board_is_check(board));

	if (COLOUR_IS_WHITE(board.turn)) {

	if ( ( board.flags & FlagsWhiteKingCastle ) != 0
	&&  board.square[F1] == Empty
	&&  board.square[G1] == Empty
	&&  (! is_attacked(board,F1,Black))) {
	LIST_ADD(list,MOVE_MAKE_FLAGS(E1,G1,MoveCastle));
	}

	if ( ( board.flags & FlagsWhiteQueenCastle ) != 0
	&&  board.square[D1] == Empty
	&&  board.square[C1] == Empty
	&&  board.square[B1] == Empty
	&&  (! is_attacked(board,D1,Black))) {
	LIST_ADD(list,MOVE_MAKE_FLAGS(E1,C1,MoveCastle));
	}

	} else {   // black

	if ( ( board.flags & FlagsBlackKingCastle ) != 0
	&&  board.square[F8] == Empty
	&&  board.square[G8] == Empty
	&&  (! is_attacked(board,F8,White))) {
	LIST_ADD(list,MOVE_MAKE_FLAGS(E8,G8,MoveCastle));
	}

	if ( ( board.flags & FlagsBlackQueenCastle ) != 0
	&&  board.square[D8] == Empty
	&&  board.square[C8] == Empty
	&&  board.square[B8] == Empty
	&&  (! is_attacked(board,D8,White))) {
	LIST_ADD(list,MOVE_MAKE_FLAGS(E8,C8,MoveCastle));
	}
	}
	}

	// add_pawn_move()

	private void add_pawn_move( list_t list, int from, int to )   {

	int move  = 0;   // int

	//ASSERT(525, SQUARE_IS_OK(from));
	//ASSERT(526, SQUARE_IS_OK(to));

	move = MOVE_MAKE(from,to);

	if (this.SquareIsPromote[to]) {
	LIST_ADD(list,(move | MovePromoteQueen));
	LIST_ADD(list,(move | MovePromoteKnight));
	LIST_ADD(list,(move | MovePromoteRook));
	LIST_ADD(list,(move | MovePromoteBishop));
	} else { 
	LIST_ADD(list,move);
	}
	}

	// add_promote()

	private void add_promote( list_t list, int move )   {

	//ASSERT(528, move_is_ok(move));

	//ASSERT(529, (move & bnotV07777)==0); // HACK
	//ASSERT(530, this.SquareIsPromote[MOVE_TO(move)]);

	LIST_ADD(list,(move | MovePromoteQueen));
	LIST_ADD(list,(move | MovePromoteKnight));
	LIST_ADD(list,(move | MovePromoteRook));
	LIST_ADD(list,(move | MovePromoteBishop));
	}

	// end of move_gen.cpp




	// move_legal.cpp



	// move_is_pseudo()

	private boolean move_is_pseudo( int move, board_t board ) { 

	int me  = 0;      // int
//	int opp  = 0;     // int
	int from  = 0;    // int
	int to  = 0;      // int
	int piece  = 0;   // int
	int capture  = 0; // int
	int inc  = 0;     // int
	int delta  = 0;   // int

	//ASSERT(531, move_is_ok(move));

	//ASSERT(533, ! board_is_check(board));

	// special cases

	if (MOVE_IS_SPECIAL(move)) {
	return move_is_pseudo_debug(move,board);
	}

	//ASSERT(534, (move & bnotV07777)==0);

	// init

	me = board.turn;
//	opp = COLOUR_OPP(board.turn);

	// from

	from = MOVE_FROM(move);
	//ASSERT(535, SQUARE_IS_OK(from));

	piece = board.square[from];
	if (! COLOUR_IS(piece,me)) {
	return false;
	}

	//ASSERT(536, piece_is_ok(piece));

	// to

	to = MOVE_TO(move);
	//ASSERT(537, SQUARE_IS_OK(to));

	capture = board.square[to];
	if (COLOUR_IS(capture,me)) {
	return false;
	}

	// move

	if (PIECE_IS_PAWN(piece)) {

	if (this.SquareIsPromote[to]) {
	return false;
	}

	inc = PawnMoveInc[me];
	delta = to - from;
	//ASSERT(538, delta_is_ok(delta));

	if (capture == Empty) {

	// pawn push

	if (delta == inc) {
	return true;
	}

	if (delta == (2*inc)
	&&  PAWN_RANK(from,me) == Rank2
	&&  board.square[from+inc] == Empty) {
	return true;
	}

	} else { 

	// pawn capture

	if (delta == (inc-1)  ||  delta == (inc+1)) {
	return true;
	}
	}

	} else { 

	if (PIECE_ATTACK(board,piece,from,to)) {
	return true;
	}
	}

	return false;
	}

	// quiet_is_pseudo()

	private boolean quiet_is_pseudo( int move, board_t board ) {

	int me  = 0;      // int
//	int opp  = 0;     // int
	int from  = 0;    // int
	int to  = 0;      // int
	int piece  = 0;   // int
	int inc  = 0;     // int
	int delta  = 0;   // int

	//ASSERT(539, move_is_ok(move));

	//ASSERT(541, ! board_is_check(board));

	// special cases

	if (MOVE_IS_CASTLE(move)) {
	return move_is_pseudo_debug(move,board);
	} else { 
	if (MOVE_IS_SPECIAL(move)) {
	return false;
	}
	}

	//ASSERT(542, (move & bnotV07777)==0);

	// init

	me = board.turn;
//	opp = COLOUR_OPP(board.turn);

	// from

	from = MOVE_FROM(move);
	//ASSERT(543, SQUARE_IS_OK(from));

	piece = board.square[from];
	if (! COLOUR_IS(piece,me)) {
	return false;
	}

	//ASSERT(544, piece_is_ok(piece));

	// to

	to = MOVE_TO(move);
	//ASSERT(545, SQUARE_IS_OK(to));

	if (board.square[to] != Empty) {
	return false; // capture
	}

	// move

	if (PIECE_IS_PAWN(piece)) {

	if (this.SquareIsPromote[to]) {
	return false;
	}

	inc = PawnMoveInc[me];
	delta = to - from;
	//ASSERT(546, delta_is_ok(delta));

	// pawn push

	if (delta == inc) {
	return true;
	}

	if (delta == (2*inc)
	&&  PAWN_RANK(from,me) == Rank2
	&&  board.square[from+inc] == Empty) {
	return true;
	}

	} else { 

	if (PIECE_ATTACK(board,piece,from,to)) {
	return true;
	}
	}

	return false;
	}

	// pseudo_is_legal()

	private boolean pseudo_is_legal( int move, board_t board ) { 

	int opp  = 0;        // int
	int me  = 0;         // int
	int from  = 0;       // int
	int to  = 0;         // int
	int piece  = 0;      // int
	boolean legal = false;  // bool
	int king  = 0;       // int
	undo_t undo = new undo_t();  //undo_t[1]

	//ASSERT(547, move_is_ok(move));

	// init

	me = board.turn;
	opp = COLOUR_OPP(me);

	from = MOVE_FROM(move);
	to = MOVE_TO(move);

	piece = board.square[from];
	//ASSERT(549, COLOUR_IS(piece,me));

	// slow test for en-passant captures

	if (MOVE_IS_EN_PASSANT(move)) {

	move_do(board,move,undo);
	legal = ! IS_IN_CHECK(board,me);
	move_undo(board,move,undo);

	return legal;
	}

	// king moves (including castle)

	if (PIECE_IS_KING(piece)) {

	legal = ! is_attacked(board,to,opp);

// --for debugging
//	//ASSERT(550, board.square[from]==piece);
//	board.square[from] = Empty;
//	//ASSERT(551, legal==(! is_attacked(board,to,opp)));
//	board.square[from] = piece;
	
	return legal;
	}

	// pins

	if (is_pinned(board,from,me)) {
	king = KING_POS(board,me);
	return (DELTA_INC_LINE(king-to) == DELTA_INC_LINE(king-from)); // does not discover the line
	}

	return true;
	}

	// move_is_pseudo_debug()

	private boolean move_is_pseudo_debug ( int move, board_t board ) { 

	list_t list = new list_t();  //list_t[1]

	//ASSERT(552, move_is_ok(move));

	//ASSERT(554, ! board_is_check(board));

	gen_moves(list,board);

	return list_contain(list,move);
	}

	// end of move_legal.cpp



	// option.cpp



	// option_init()

	private void option_init()  {


	// options are as they are for the execuatable version
	set_opt_t_def( 0, "Hash",  false, "16", "spin", "min 4 max 1024" );
	set_opt_t_def( 1, "Ponder",  false, "false", "check", "" );
	set_opt_t_def( 2, "OwnBook",  false, "false", "check", "" );
	set_opt_t_def( 3, "BookFile",   false, "book_small.bin", "string", "" );
	set_opt_t_def( 4, "nullMove Pruning",  true, "Fail High", "combo", "Always Fail High Never" );
	set_opt_t_def( 5, "nullMove Reduction",  true, "3", "spin", "min 1 max 3" );
	set_opt_t_def( 6, "Verification Search",  true, "endgame", "combo", "Always endgame Never" );
	set_opt_t_def( 7, "Verification Reduction",  true, "5", "spin", "min 1 max 6" );
	set_opt_t_def( 8, "History Pruning", true, "true", "check", "" );
	set_opt_t_def( 9, "History Threshold",  true, "60", "spin", "min 0 max 100" );
	set_opt_t_def( 10, "Futility Pruning",  true, "false", "check", "" );
	set_opt_t_def( 11, "Futility Margin",  true, "100", "spin",  "min 0 max 500" );
	set_opt_t_def( 12, "Delta Pruning",  true, "false", "check", "" );
	set_opt_t_def( 13, "Delta Margin",  true, "50", "spin",  "min 0 max 500" );
	set_opt_t_def( 14, "Quiescence Check Plies", true, "1", "spin", "min 0 max 2" );
	set_opt_t_def( 15, "Material",  true, "100", "spin", "min 0 max 400" );
	set_opt_t_def( 16, "Piece Activity",  true, "100", "spin", "min 0 max 400" );
	set_opt_t_def( 17, "King Safety",  true, "100", "spin", "min 0 max 400" );
	set_opt_t_def( 18, "Pawn Structure",  true, "100", "spin", "min 0 max 400" );
	set_opt_t_def( 19, "Passed Pawns",  true, "100", "spin", "min 0 max 400" );
	set_opt_t_def( 20, "", false, "", "", "" );

	}

	// option_list()

/*
	private void option_list()  {

	opt_t_def opt = null;
	int i = 0;

	while(true) {
	opt = this.Option[i];
	if( opt.vary.length() == 0 ) {
	break;
	}

	if (opt.decl) {
	send("option name "+ opt.vary +" type "+ opt.type +" default "+ opt.val + opt.extra);
	}

	i++;
	}
	}
*/
	// option_set()

	private boolean option_set( String vary, String val ) {

	int i = 0;

	i = option_find(vary);
	if (i<0) { return false; }
	this.Option[i].val = val;

	return true;
	}

	// option_get()

	private String option_get( String vary ) { 

	int i = 0;

	i = option_find(vary);
	if (i<0) {
	my_fatal("option_get(): unknown option : "+ vary + "\n");
	return "";
	}

	return this.Option[i].val;
	}

	// option_get_bool()

	private boolean option_get_bool( String vary ) { 

	String val = option_get(vary);   // string

	if (string_equal(val,"true")  ||  string_equal(val,"yes")  ||  string_equal(val,"1")) {
	return true;
	} else { 
	if (string_equal(val,"false")  ||  string_equal(val,"no")  ||  string_equal(val,"0")) {
	return false;
	}
	}

	//ASSERT(558, false);

	return false;
	}

	// option_get_int()

	private int option_get_int( String vary )  {
	return Integer.parseInt( option_get(vary) );
	}

	// option_get_string()

	private String option_get_string( String vary ) { 
	return option_get(vary);
	}

	// option_find()

	private int option_find( String vary )  { 

	opt_t_def opt = new opt_t_def();
	int i = 0;

	while(true) {
	opt = this.Option[i];
	if( opt.vary.length() == 0 ) {
	break;
	}

	if (string_equal(opt.vary,vary)) {
	return i;
	}

	i++;
	}

	return -1;
	}

	// end of option.cpp



	// pawn.cpp



	// pawn_init_bit()

	private void pawn_init_bit()   {

	int rank  = 0;   // int
	int first  = 0;  // int
	int last  = 0;   // int
	int count  = 0;  // int
	int b  = 0;      // int
	int rev  = 0;    // int


	// rank-indexed Bit*[]

	for (rank = 0; rank<RankNb; rank++ ) {

	this.BitEQ[rank] = 0;
	this.BitLT[rank] = 0;
	this.BitLE[rank] = 0;
	this.BitGT[rank] = 0;
	this.BitGE[rank] = 0;

	this.BitRank1[rank] = 0;
	this.BitRank2[rank] = 0;
	this.BitRank3[rank] = 0;
	}

	for (rank = Rank1; rank<=Rank8; rank++ ) {
	this.BitEQ[rank] = ( 1 << rank - Rank1);
	this.BitLT[rank] = this.BitEQ[rank] - 1;
	this.BitLE[rank] = ( this.BitLT[rank] | this.BitEQ[rank] );
	this.BitGT[rank] = ( this.BitLE[rank] ^ 0xFF );
	this.BitGE[rank] = ( this.BitGT[rank] | this.BitEQ[rank]);
	}

	for (rank = Rank1; rank<=Rank8; rank++ ) {
	this.BitRank1[rank] = this.BitEQ[rank+1];
	this.BitRank2[rank] = ( this.BitEQ[rank+1] | this.BitEQ[rank+2]) ;
	this.BitRank3[rank] = ( ( this.BitEQ[rank+1] | this.BitEQ[rank+2] ) | this.BitEQ[rank+3] );
	}

	// bit-indexed Bit*[]

	for (b = 0; b<= 0x100-1; b++ ) {

	first = Rank8;  // HACK for pawn shelter
	last = Rank1;   // HACK
	count = 0;
	rev = 0;

	for (rank = Rank1; rank<=Rank8; rank++ ) {
	if ( ( b & this.BitEQ[rank] ) != 0) {
	if (rank < first) {
	first = rank;
	}
	if (rank > last) {
	last = rank;
	}
	count++;
	rev = ( rev | this.BitEQ[RANK_OPP(rank)] );
	}
	}

	this.BitFirst[b] = first;
	this.BitLast[b] = last;
	this.BitCount[b] = count;
	this.BitRev[b] = rev;
	}

	}

	// pawn_init()

	private void pawn_init()  {

	int rank  = 0;   // int

	// UCI options

	this.PawnStructureWeight = (option_get_int("Pawn Structure") * 256 + 50) / 100;

	// bonus

	for (rank = 0; rank<RankNb; rank++ ) {

	this.Bonus[rank] = 0;
	}

	this.Bonus[Rank4] = 26;
	this.Bonus[Rank5] = 77;
	this.Bonus[Rank6] = 154;
	this.Bonus[Rank7] = 256;

	}

	// pawn_alloc()

	private void pawn_alloc()   {
		
	if (UseTable)
	{
	pawn_clear();		
	// Pawn.table = (entry_t *) my_malloc(Pawn.size*sizeof(entry_t));
	}
	}

	// pawn_clear()

	private void pawn_clear()   {

	Pawn.size = PawnTableSize;
	Pawn.mask = Pawn.size - 1;     // 2^x -1

	}

	// pawn_get_info()

	private void pawn_get_info( pawn_info_t info, board_t board ) { 

	int key  = 0;           // uint64
	pawn_info_t entry = new pawn_info_t();    // *;
	int index = 0;

	// probe

	if (UseTable) {

	this.Pawn.read_nb++;

	key = board.pawn_key;
	index = ( KEY_INDEX(key) & this.Pawn.mask );

	entry = this.Pawn.table[index];
	if(entry==null) {
	this.Pawn.table[index] = new pawn_info_t();
	entry = this.Pawn.table[index];
	}

	if (entry.lock == KEY_LOCK(key)) {

	// found

	this.Pawn.read_hit++;

	pawn_info_copy( info, entry );

	return;
	}
	}

	// calculation

	pawn_comp_info(info,board);

	// store

	if (UseTable) {

	this.Pawn.write_nb++;

	if (entry.lock == 0) {    // HACK: assume free entry
	this.Pawn.used++;
	} else { 
	this.Pawn.write_collision++;
	}

	pawn_info_copy( entry, info );

	entry.lock = KEY_LOCK(key);
	}

	}

	// pawn_comp_info()

	private void pawn_comp_info( pawn_info_t info, board_t board )  {

	int colour  = 0;   // int
	int file  = 0;     // int
	int rank  = 0;     // int
	int me  = 0;       // int
	int opp  = 0;      // int
	int ptr  = 0;      // int
	int sq  = 0;       // int
	boolean backward = false;    // bool
	boolean candidate = false;   // bool
	boolean doubled = false;     // bool
	boolean isolated = false;    // bool
	boolean open = false;        // bool
//	boolean passed = false;      // bool
	int t1  = 0;        // int
	int t2  = 0;        // int
	int n  = 0;         // int
	int bits  = 0;      // int
	int[] opening = { 0, 0 };  // int[ColourNb]
	int[] endgame = { 0, 0 };  // int[ColourNb]
	int[] flags  = { 0, 0 };    // int[ColourNb]
	int[] file_bits = { 0, 0 };   // int[ColourNb]
	int[] passed_bits = { 0, 0 }; // int[ColourNb]
	int[] single_file = { 0, 0 }; // int[ColourNb]
	int q = 0;
	int om = 0;
	int em = 0;


	// pawn_file[]

//   debug only
//
//	for (colour = 0;  colour<=1; colour++ ) {
//
//	int[] pawn_file = new int[FileNb];   // int[FileNb]
//
//	me = colour;
//
//	for (file = 0; file<FileNb; file++ ) {
//	pawn_file[file] = 0;
//	}
//
//	ptr = 0;
//	while(true) {
//	sq=board.pawn[me][ptr];
//	if(sq==SquareNone) {
//	break;
//	}
//
//	file = SQUARE_FILE(sq);
//	rank = PAWN_RANK(sq,me);
//
	//ASSERT(565, file>=FileA && file<=FileH);
	//ASSERT(566, rank>=Rank2 && rank<=Rank7);
//
//	pawn_file[file] =  ( pawn_file[file] | this.BitEQ[rank] );
//
//	ptr++;
//	}
//
//	for (file = 0; file<FileNb; file++ ) {
//	if (board.pawn_file[colour][file] != pawn_file[file]) {
//	my_fatal("board.pawn_file[][]\n");
//	}
//	}
//	}



	// features && scoring

	for (colour = 0;  colour<=1; colour++ ) {

	me = colour;
	opp = COLOUR_OPP(me);

	ptr = 0;
	while(true) {
	sq=board.pawn[me][ptr];
	if(sq==SquareNone) {
	break;
	}


	// init

	file = SQUARE_FILE(sq);
	rank = PAWN_RANK(sq,me);

	//ASSERT(567, file>=FileA && file<=FileH);
	//ASSERT(568, rank>=Rank2 && rank<=Rank7);

	// flags

	file_bits[me] = ( file_bits[me] | this.BitEQ[file] );
	if (rank == Rank2) {
	flags[me] = ( flags[me] | BackRankFlag );
	}

	// features

	backward = false;
	candidate = false;
	doubled = false;
	isolated = false;
	open = false;
//	passed = false;

	t1 = ( board.pawn_file[me][file-1] | board.pawn_file[me][file+1] );
	t2 = ( board.pawn_file[me][file] | this.BitRev[board.pawn_file[opp][file]] );

	// doubled

	if ( ( board.pawn_file[me][file] & this.BitLT[rank] ) != 0) {
	doubled = true;
	}

	// isolated && backward

	if (t1 == 0) {

	isolated = true;

	} else { 
	if ( ( t1 & this.BitLE[rank] ) == 0) {

	backward = true;

	// really backward?

	if ( ( t1 & this.BitRank1[rank] ) != 0) {

	//ASSERT(569, rank+2<=Rank8);
	q = (t2 & this.BitRank1[rank]);
	q = (q | this.BitRev[board.pawn_file[opp][file-1]]);
	q = (q | this.BitRev[board.pawn_file[opp][file+1]]);

	if ( ( q & this.BitRank2[rank] ) == 0) {

	backward = false;
	}

	} else { 
	if (rank == Rank2  &&  ( ( t1 & this.BitEQ[rank+2] ) != 0)) {

	//ASSERT(570, rank+3<=Rank8);
	q = (t2 & this.BitRank2[rank]);
	q = (q | this.BitRev[board.pawn_file[opp][file-1]]);
	q = (q | this.BitRev[board.pawn_file[opp][file+1]]);

	if ( ( q & this.BitRank3[rank] ) == 0) {

	backward = false;
	}
	}
	}
	}
	}

	// open, candidate && passed

	if ( ( t2 & this.BitGT[rank] ) == 0) {

	open = true;

	q = ( this.BitRev[board.pawn_file[opp][file-1]] | 
	this.BitRev[board.pawn_file[opp][file+1]]);

	if ( ( q & this.BitGT[rank] ) == 0) {

//	passed = true;
	passed_bits[me] = ( passed_bits[me] | this.BitEQ[file] );

	} else { 

	// candidate?

	n = 0;

	n = n + this.BitCount[( board.pawn_file[me][file-1] & this.BitLE[rank] ) ];
	n = n + this.BitCount[( board.pawn_file[me][file+1] & this.BitLE[rank] ) ];

	n = n - this.BitCount[( this.BitRev[board.pawn_file[opp][file-1]] & this.BitGT[rank] ) ];
	n = n - this.BitCount[( this.BitRev[board.pawn_file[opp][file+1]] & this.BitGT[rank] ) ];

	if (n >= 0) {

	// safe?

	n = 0;

	n = n + this.BitCount[( board.pawn_file[me][file-1] & this.BitEQ[rank-1] ) ];
	n = n + this.BitCount[( board.pawn_file[me][file+1] & this.BitEQ[rank-1] ) ];

	n = n - this.BitCount[( this.BitRev[board.pawn_file[opp][file-1]] & this.BitEQ[rank+1] ) ];
	n = n - this.BitCount[( this.BitRev[board.pawn_file[opp][file+1]] & this.BitEQ[rank+1] ) ];

	if (n >= 0) {
	candidate = true;
	}
	}
	}
	}

	// score

	om = opening[me];
	em = endgame[me];

	if (doubled) {
	om = om - doubledOpening;
	em = em - doubledEndgame;
	}

	if (isolated) {
	if (open) {
	om = om - IsolatedOpeningOpen;
	em = em - IsolatedEndgame;
	} else { 
	om = om - IsolatedOpening;
	em = em - IsolatedEndgame;
	}
	}

	if (backward) {
	if (open) {
	om = om - BackwardOpeningOpen;
	em = em - BackwardEndgame;
	} else { 
	om = om - BackwardOpening;
	em = em - BackwardEndgame;
	}
	}

	if (candidate) {
	om = om + quad(CandidateOpeningMin,CandidateOpeningMax,rank);
	em = em + quad(CandidateEndgameMin,CandidateEndgameMax,rank);
	}


	opening[me] = om;
	endgame[me] = em;

	ptr++;

	}
	}

	// store info

	info.opening = ((opening[White] - opening[Black]) * this.PawnStructureWeight) / 256;
	info.endgame = ((endgame[White] - endgame[Black]) * this.PawnStructureWeight) / 256;

	for (colour = 0;  colour<=1; colour++ ) {

	me = colour;
	opp = COLOUR_OPP(me);

	// draw flags

	bits = file_bits[me];

	if (bits != 0  &&  ( ( bits & bits-1 ) == 0) ) {  // one set bit

	file = this.BitFirst[bits];
	rank = this.BitFirst[board.pawn_file[me][file] ];
	//ASSERT(571, rank>=Rank2);

	q = ( this.BitRev[board.pawn_file[opp][file-1]] | 
	this.BitRev[board.pawn_file[opp][file+1]] );

	if ( ( q & this.BitGT[rank] ) == 0) {

	rank = this.BitLast[board.pawn_file[me][file] ];
	single_file[me] = SQUARE_MAKE(file,rank);
	}
	}

	info.flags[colour] = flags[colour];
	info.passed_bits[colour] = passed_bits[colour];
	info.single_file[colour] = single_file[colour];
	}

	}

	// quad()
	
	private int quad( int y_min, int y_max, int x ) {

	int y  = 0;   // int

	//ASSERT(572, y_min>=0 && y_min<=y_max && y_max<=32767);
	//ASSERT(573, x>=Rank2 && x<=Rank7);

	y =  (int)( y_min + ((y_max - y_min) * this.Bonus[x] + 128) / 256 );

	//ASSERT(574, y>=y_min && y<=y_max);

	return y;
	}

	// end of pawn.cpp



	// piece.cpp



	// piece_init()

	private void piece_init()  {

	int piece  = 0;      // int
	int piece_12  = 0;   // int

	// this.PieceTo12[], this.PieceOrder[], this.PieceInc[]

	for (piece = 0; piece<PieceNb; piece++ ) {
	this.PieceTo12[piece] = -1;
	this.PieceOrder[piece] = -1;
	//this.PieceInc[piece] = [];
	}

	for (piece_12 = 0; piece_12<= 11; piece_12++ ) {
	this.PieceTo12[PieceFrom12[piece_12]] = piece_12;
	this.PieceOrder[PieceFrom12[piece_12]] = ( piece_12 >> 1 );
	}

	this.PieceInc[WhiteKnight256] = KnightInc;
	this.PieceInc[WhiteBishop256] = BishopInc;
	this.PieceInc[WhiteRook256]   = RookInc;
	this.PieceInc[WhiteQueen256]  = QueenInc;
	this.PieceInc[WhiteKing256]   = KingInc;

	this.PieceInc[BlackKnight256] = KnightInc;
	this.PieceInc[BlackBishop256] = BishopInc;
	this.PieceInc[BlackRook256]   = RookInc;
	this.PieceInc[BlackQueen256]  = QueenInc;
	this.PieceInc[BlackKing256]   = KingInc;

	}

	// piece_is_ok()

	private boolean piece_is_ok( int piece ) {

	if (piece < 0  ||  piece >= PieceNb) {
	return false;
	}
	if (this.PieceTo12[piece] < 0) {
	return false;
	}
	return true;
	}

	// piece_to_char()

	private char piece_to_char( int piece ) {

	int i = this.PieceTo12[piece];

	//ASSERT(576, piece_is_ok(piece));

	return PieceString.charAt(i);
	}

	// piece_from_char()

	private int piece_from_char( char c ) {

	int ptr  = PieceString.indexOf( c );   // int

	if (ptr<0) {
	return PieceNone256;
	}

	//ASSERT(575, ptr>=0 && ptr<12);

	return PieceFrom12[ptr];
	}

	// end of piece.cpp




	// protocol.cpp



	private void setstartpos() { 

	// init (to help debugging)

	this.Init = false;

	search_clear(true);

	board_from_fen(this.SearchInput.board,StartFen);

	}

	// inits()

	private void inits() { 

	if (! this.Init) {

	// late initialisation

	this.Init = true;

	if (option_get_bool("OwnBook")) {
	//   book_open(option_get_string("BookFile"));
	send("Sorry, no book.");
	}

	trans_alloc();

	pawn_init();
	pawn_alloc();

	material_init();
	material_alloc();

	pst_init();
	eval_init();
	}
	}

	// loop_step()

	public void do_input( String cmd ) { 

	boolean ifelse = true;

	// parse

	if (ifelse && string_start_with(cmd,"go")) {

	inits();
	parse_go( cmd );

	ifelse = false;
	}

	if (ifelse && string_equal(cmd,"isready")) {

	inits();
	send("readyok");

	ifelse = false;
	}


	if (ifelse && string_start_with(cmd,"position ")) {

	inits();
	parse_position(cmd);

	ifelse = false;
	}


	if (ifelse && string_start_with(cmd,"setoption ")) {

	parse_setoption( cmd );

	ifelse = false;
	}


	if (ifelse && string_equal(cmd,"help")) {


	send("supports commands: setposition fen, setposition moves, go depth, go movetime ");

	// can manage also options, but use the default settings

	// option_list();

	ifelse = false;
	}

	}

	// parse_go()

	private void parse_go( String cmd )  {

	String cmd1 = "";          // string
	String cmd2 = "";          // string
	boolean infinite = false;   // bool
	int depth  = -1;         // int
	long movetime = -1;    // int
	boolean ifelse = false;
	string_t save_board = new string_t();

	// parse

	cmd1 = str_after_ok(cmd," ");    // skip "go"
	cmd2 = str_after_ok(cmd1," ");   // value
	cmd1 = str_before_ok(cmd1+" "," ");

	ifelse = true;
	if (ifelse && string_equal(cmd1,"depth")) {

	depth = Integer.parseInt(cmd2);
	//ASSERT(590, depth>=0);

	ifelse = false;
	}

	if (ifelse && string_equal(cmd1,"infinite")) {

	infinite = true;

	ifelse = false;
	}

	if (ifelse && string_equal(cmd1,"movetime")) {

	movetime = Integer.parseInt(cmd2);
	//ASSERT(593, movetime>=0);

	ifelse = false;
	}

	if (ifelse) {

	movetime = 10;   // Otherwise constantly 10 secs

	ifelse = false;
	}


	// init

	ClearAll();

	// depth limit

	if (depth >= 0) {
	this.SearchInput.depth_is_limited = true;
	this.SearchInput.depth_limit = depth;
	}

	// time limit

	if (movetime >= 0) {

	// fixed time

	this.SearchInput.time_is_limited = true;
	this.SearchInput.time_limit_1 = movetime * 1000;
	this.SearchInput.time_limit_2 = movetime * 1000;

	}

	if (infinite) {
	this.SearchInput.infinite = true;
	}

	// search

	if( !this.ShowInfo) {
	send("Thinking (ShowInfo=false)...");
	}

	board_to_fen(this.SearchInput.board, save_board);   // save board for sure

	search();
	search_update_current();

	board_from_fen(this.SearchInput.board, save_board.v); // && restore after search

	send_best_move();

	}

	// parse_position()

	private void parse_position( String cmd )  {

	String cmd1 = "";          // string
	String cmd2 = "";          // string
	int mc = 0;

	string_t move_string = new string_t();   // string

	int move = 0;             // int
	undo_t undo = new undo_t();  // undo_t[1]
	String mnext = "";

	cmd1 = str_after_ok(cmd," ");    // skip "position"
	cmd2 = str_after_ok(cmd1," ");   // value

	// start position

	if ( string_start_with(cmd1,"fen") ) {  // "fen" present

	board_from_fen(this.SearchInput.board,cmd2);

	} else { 

	if ( string_start_with(cmd1,"moves") ) {  // "moves" present

	board_from_fen(this.SearchInput.board,StartFen);
	
	mc = 0;

	mnext = cmd2;
	while(true) {

	if( mnext.length()==0 ) {
	break;
	}

	move_string.v = ( mnext.indexOf(" ")<0 ? mnext : str_before_ok(mnext," ") );

	move = move_from_string(move_string,this.SearchInput.board);

	move_do(this.SearchInput.board,move,undo);

	mnext = str_after_ok(mnext," ");

	mc++;
	}

	this.SearchInput.board.movenumb = 1+(int)(mc/2);

	} else { 

	// HACK: assumes startpos

	board_from_fen(this.SearchInput.board,StartFen);

	}
	}

	}

	// parse_setoption()

	private void parse_setoption( String cmd )  {

	String cmd1 = "";    // string
	String name = "";    // string
	String value = "";   // string

	cmd1 = str_after_ok(cmd," ");    // skip "setoption"

	name = str_after_ok(cmd1,"name ");
	name = str_before_ok(name+" "," ");

	value = str_after_ok(cmd1,"value ");
	value = str_before_ok(value+" "," ");


	if ( name.length()>0 && value.length()>0 )  {

	// update

	option_set(name,value);
	}

	// update transposition-table size if needed

	if (this.Init  &&  string_equal(name,"Hash")) {  // Init => already allocated

	if (option_get_int("Hash") >= 4) {
	trans_alloc();
	}
	}

	}


	// send_best_move()

	private void send_ndtm( int ch ) { 

	String s = "info";
	String s2 = "";

	if(ch>5) {
	s += " depth " + string_from_int(this.SearchCurrent.depth );
	s += " seldepth " + string_from_int(this.SearchCurrent.max_depth ) + " ";
	}

	if(ch>=20 && ch<=22) {
	s2 += " score mate " + string_from_int(this.SearchCurrent.mate ) + " ";
	}
	if(ch==11 || ch==21) {
	s2 +=  "lowerbound ";
	}
	if(ch==12 || ch==22) {
	s2 +=  "upperbound ";
	}

	s += " " + s2 + "time "+ Long.toString(this.SearchCurrent.time/1000) + "s";
	s += " nodes " + Integer.toString(this.SearchCurrent.node_nb);
	s += " nps " +   Long.toString(this.SearchCurrent.speed/1000);

	send( s );

	}

	private void send_best_move() { 

	string_t move_string = new string_t();     // string
//	string_t ponder_string = new string_t();   // string

	int move  = 0;      // int
//	int[] pv = null;    // *int[]

	// info

	send_ndtm(1);

	if(this.DispStats) trans_stats(this.Trans);

	// best move

	move = this.SearchBest.move;
//	pv = this.SearchBest.pv;

	move_to_string(move,move_string);

/*
	if ((false) && pv[0] == move  &&  move_is_ok(pv[1])) {

	move_to_string(pv[1],ponder_string);
	send("bestmove " + move_string.v + " ponder " + ponder_string.v);
	} else { 
	send("bestmove " + move_string.v);
	}
*/

	this.bestmv = move_string.v;

	format_best_mv2( move );

	}

	// move for pgn

	private void format_best_mv2( int move ) { 

	int piece = 0;
	String piecech = "";
	String mvattr = "";
	String promos = "";
	String ckmt = "";
	board_t board = this.SearchInput.board;

	if( MOVE_IS_CASTLE(move) ) {
	this.bestmv2 = ( this.bestmv.charAt(2) == 'g' ? "0-0" : "0-0-0" );
	} else { 
	piece = board.square[MOVE_FROM(move)];
	if( (! piece_is_ok( piece )) || piece == PieceNone64 ) {
	piece = board.square[MOVE_TO(move)];
	}

	piecech = ( "" + piece_to_char(piece) ).toUpperCase();
	if( piecech.equals("P")) {
	piecech = "";
	}

	mvattr = ( move_is_capture(move,board) ? "x" : "-" );

	if( this.bestmv.length()>4 ) {
	promos = (""+this.bestmv.charAt(4));
	}

	if( move_is_check(move,board) ) {
	ckmt = "+";
	}

	this.bestmv2 = piecech + this.bestmv.substring( 0,2 ) + mvattr + this.bestmv.substring( 2, 4 ) + promos + ckmt;

	}
	}


	// send()

	private void send( String str1 )  {

	if( ! this.ShowInfo && string_start_with(str1,"info ")) {
	return;
	}

	print2out( str1 );
	}

	// string_equal()

	private boolean string_equal( String s1, String s2 ) { 
	return s1.equals(s2);
	}

	// string_start_with()

	private boolean string_start_with( String s1, String s2 ) { 
	return s1.startsWith(s2);
	}


	// str_before_ok()

	private String str_before_ok( String str1, String c ) { 
	int i  = str1.indexOf( c );
	return ((i>=0) ? str1.substring( 0, i ) : "" );
	}

	// str_after_ok()

	private String str_after_ok( String str1, String c ) { 
	int i  = str1.indexOf( c );
	return ((i>=0) ? str1.substring( i+ c.length() ) : "" );
	}

	// end of protocol.cpp



	// pst.cpp


	// macros

	private int Pget( int piece_12 , int square_64 , int stage ) { 
	return this.Pst[piece_12][square_64][stage];
	}

	private void Pset( int piece_12 , int square_64, int stage, int value ) { 
	this.Pst[piece_12][square_64][stage] = value;
	}

	private void Padd( int piece_12 , int square_64, int stage, int value ) { 
	this.Pst[piece_12][square_64][stage] += value;
	}

	private void Pmul( int piece_12 , int square_64, int stage, int value ) { 
	this.Pst[piece_12][square_64][stage] *= value;
	}


	// pst_init()

	private void pst_init() { 

	int i  = 0;      // int
	int piece  = 0;  // int
	int sq  = 0;     // int
	int stage  = 0;  // int

	// UCI options

	this.PieceActivityWeight = (option_get_int("Piece Activity") * 256 + 50) / 100;
	this.KingSafetyWeight    = (option_get_int("King Safety")    * 256 + 50) / 100;
	this.PawnStructureWeight = (option_get_int("Pawn Structure") * 256 + 50) / 100;

	// init

	for (piece = 0; piece<=11; piece++ ) {
	//this.Pst[piece] = [];
	for (sq = 0; sq<=63; sq++ ) {
	//this.Pst[piece][sq] = [];
	for (stage = 0; stage<StageNb; stage++ ) {
	Pset(piece,sq,stage, 0);
	}
	}
	}

	// pawns

	piece = WhitePawn12;

	// file

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, PawnFile[square_file(sq)] * PawnFileOpening );
	}

	// centre control

	Padd(piece,pD3,Opening, 10);
	Padd(piece,pE3,Opening, 10);

	Padd(piece,pD4,Opening, 20);
	Padd(piece,pE4,Opening, 20);

	Padd(piece,pD5,Opening, 10);
	Padd(piece,pE5,Opening, 10);

	// weight

	for (sq = 0; sq<=63; sq++ ) {
	Pmul(piece,sq,Opening,  this.PawnStructureWeight / 256);
	Pmul(piece,sq,Endgame,  this.PawnStructureWeight / 256);
	}

	// knights

	piece = WhiteKnight12;

	// centre

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, KnightLine[square_file(sq)] * KnightCentreOpening);
	Padd(piece,sq,Opening, KnightLine[square_rank(sq)] * KnightCentreOpening);
	Padd(piece,sq,Endgame, KnightLine[square_file(sq)] * KnightCentreEndgame);
	Padd(piece,sq,Endgame, KnightLine[square_rank(sq)] * KnightCentreEndgame);
	}

	// rank

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, KnightRank[square_rank(sq)] * KnightRankOpening);
	}

	// back rank

	for (sq = pA1; sq<=pH1; sq++ ) {    // HACK: only first rank
	Padd(piece,sq,Opening, -KnightBackRankOpening);
	}

	// "trapped"

	Padd(piece,pA8,Opening, -KnightTrapped);
	Padd(piece,pH8,Opening, -KnightTrapped);

	// weight

	for (sq = 0; sq<=63; sq++ ) {
	Pmul(piece,sq,Opening,  this.PieceActivityWeight / 256);
	Pmul(piece,sq,Endgame,  this.PieceActivityWeight / 256);
	}

	// bishops

	piece = WhiteBishop12;

	// centre

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening,  BishopLine[square_file(sq)] * BishopCentreOpening);
	Padd(piece,sq,Opening,  BishopLine[square_rank(sq)] * BishopCentreOpening);
	Padd(piece,sq,Endgame,  BishopLine[square_file(sq)] * BishopCentreEndgame);
	Padd(piece,sq,Endgame,  BishopLine[square_rank(sq)] * BishopCentreEndgame);
	}

	// back rank

	for (sq = pA1; sq<=pH1; sq++ ) {    // HACK: only first rank
	Padd(piece,sq,Opening, -BishopBackRankOpening);
	}

	// main diagonals

	for (i = 0; i<=7; i++ ) {
	sq = square_make(i,i);
	Padd(piece,sq,Opening, BishopDiagonalOpening);
	Padd(piece,square_opp(sq),Opening, BishopDiagonalOpening);
	}

	// weight

	for (sq = 0; sq<=63; sq++ ) {
	Pmul(piece,sq,Opening,  this.PieceActivityWeight / 256);
	Pmul(piece,sq,Endgame,  this.PieceActivityWeight / 256);
	}

	// rooks

	piece = WhiteRook12;

	// file

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, RookFile[square_file(sq)] * RookFileOpening);
	}

	// weight

	for (sq = 0; sq<=63; sq++ ) {
	Pmul(piece,sq,Opening,  this.PieceActivityWeight / 256);
	Pmul(piece,sq,Endgame,  this.PieceActivityWeight / 256);
	}

	// queens

	piece = WhiteQueen12;

	// centre

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, QueenLine[square_file(sq)] * QueenCentreOpening);
	Padd(piece,sq,Opening, QueenLine[square_rank(sq)] * QueenCentreOpening);
	Padd(piece,sq,Endgame, QueenLine[square_file(sq)] * QueenCentreEndgame);
	Padd(piece,sq,Endgame, QueenLine[square_rank(sq)] * QueenCentreEndgame);
	}

	// back rank

	for (sq = pA1; sq<=pH1; sq++ ) {    // HACK: only first rank
	Padd(piece,sq,Opening, -QueenBackRankOpening);
	}

	// weight

	for (sq = 0; sq<=63; sq++ ) {
	Pmul(piece,sq,Opening, this.PieceActivityWeight / 256);
	Pmul(piece,sq,Endgame, this.PieceActivityWeight / 256);
	}

	// kings

	piece = WhiteKing12;

	// centre

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Endgame, KingLine[square_file(sq)] * KingCentreEndgame);
	Padd(piece,sq,Endgame, KingLine[square_rank(sq)] * KingCentreEndgame);
	}

	// file

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, KingFile[square_file(sq)] * KingFileOpening);
	}

	// rank

	for (sq = 0; sq<=63; sq++ ) {
	Padd(piece,sq,Opening, KingRank[square_rank(sq)] * KingRankOpening);
	}

	// weight

	for (sq = 0; sq<=63; sq++ ) {
	Pmul(piece,sq,Opening, this.KingSafetyWeight / 256);
	Pmul(piece,sq,Endgame, this.PieceActivityWeight / 256);
	}

	// symmetry copy for black

	for (piece = 0; piece<= 11; piece+=2 ) { // HACK
	for (sq = 0; sq<=63; sq++ ) {
	for (stage = 0; stage<StageNb; stage++ ) {
	Pset(piece+1,sq,stage, -Pget(piece,square_opp(sq),stage) ); // HACK
	}
	}
	}

	}

	// square_make()

	private int square_make( int file, int rank ) {

	//ASSERT(610, file>=0 && file<8);
	//ASSERT(611, rank>=0 && rank<8);

	return ( (rank << 3) | file);
	}

	// square_file()

	private int square_file( int square ) {

	//ASSERT(612, square>=0 && square<64);

	return ( square & 7 );
	}

	// square_rank()

	private int square_rank( int square ) {

	//ASSERT(613, square>=0 && square<64);

	return (square >> 3);
	}

	// square_opp()

	private int square_opp( int square ) {

	//ASSERT(614, square>=0 && square<64);

	return (square ^ 56);
	}

	// end of pst.cpp




	// pv.cpp




	// pv_is_ok()

	private boolean pv_is_ok( int[] pv ) {

	int pos  = 0;    // int
	int move  = 0;   // int

	while(true) {

	if (pos >= 256) {
	return false;
	}
	move = pv[pos];

	if (move == MoveNone) {
	return true;
	}
	if (! move_is_ok(move)) {
	return false;
	}

	pos++;
	}

	//return true;
	}

	// pv_copy()

	private void pv_copy( int[] dst, int[] src )  {

	int i  = 0;  // int
	int m  = 0;  // int

	//ASSERT(615, pv_is_ok(src));

	while(true) {
	m = src[i];
	dst[i] = m;
	if( m == MoveNone) {
	break;
	}
	i++;
	}

	}

	// pv_cat()

	private void pv_cat( int[] dst, int[] src, int move )  {

	int i  = 0;  // int
	int m  = 0;  // int

	//ASSERT(617, pv_is_ok(src));

	dst[0] = move;

	while(true) {
	m = src[i];
	dst[i+1] = m;
	if( m == MoveNone) {
	break;
	}
	i++;
	}

	}

	// pv_to_string()


	private boolean pv_to_string( int[] pv, string_t str1 ) {

	int i  = 0;              // int
	int move  = 0;           // int
	string_t str2 = new string_t();  // string_t[1]

	//ASSERT(619, pv_is_ok(pv));


	// init

	str1.v = "";

	// loop

	while(true) {

	move = pv[i];
	if(move==MoveNone) {
	break;
	}

	if(i>0) {
	str1.v += " ";
	}

	move_to_string(move, str2);
	str1.v += str2.v;

	i++;
	}

	return true;

	}

	// end of pv.cpp




	// random.cpp

	// we simply ignore 32bits of number
	// so, we can't read polyglot book
	// anyway, we can hash now


	// 
	// This was modified to get shorter resulting code
	// 
	// random_init()

	private void random_init() {

	Random rand = new Random();		// simplified
	
	int i = 0;
	for( i=0; i<RandomNb; i++) Random64[i] = rand.nextInt();

	}
	// end of random.cpp


	// recog.cpp

	// recog_draw()

	private boolean recog_draw( board_t board ) {

	material_info_t mat_info = new material_info_t();  // material_info_t[1]
	boolean ifelse = false;

	int me  = 0;    // int
	int opp  = 0;   // int
	int wp  = 0;   // int
	int wk  = 0;   // int
	int bk  = 0;   // int
	int wb  = 0;   // int
	int bb  = 0;   // int


	// material

	if (board.piece_nb > 4) {
	return false;
	}

	material_get_info(mat_info,board);

	if ( ( mat_info.flags & DrawNodeFlag ) == 0) {
	return false;
	}

	// recognisers


	ifelse = true;
	if (mat_info.recog == MAT_KK) {

	// KK

	return true;
	}

	if (mat_info.recog == MAT_KBK) {

	// KBK (white)

	return true;
	}

	if (mat_info.recog == MAT_KKB) {

	// KBK (black)

	return true;
	}

	if (mat_info.recog == MAT_KNK) {

	// KNK (white)

	return true;
	}

	if (mat_info.recog == MAT_KKN) {

	// KNK (black)

	return true;
	}

	if (mat_info.recog == MAT_KPK) {

	// KPK (white)

	me = White;
	opp = COLOUR_OPP(me);

	wp = board.pawn[me][0];
	wk = KING_POS(board,me);
	bk = KING_POS(board,opp);

	if (SQUARE_FILE(wp) >= FileE) {
	wp = SQUARE_FILE_MIRROR(wp);
	wk = SQUARE_FILE_MIRROR(wk);
	bk = SQUARE_FILE_MIRROR(bk);
	}

	if (kpk_draw(wp,wk,bk,board.turn)) {
	return true;
	}
	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KKP) {

	// KPK (black)

	me = Black;
	opp = COLOUR_OPP(me);

	wp = SQUARE_RANK_MIRROR(board.pawn[me][0]);
	wk = SQUARE_RANK_MIRROR(KING_POS(board,me));
	bk = SQUARE_RANK_MIRROR(KING_POS(board,opp));

	if (SQUARE_FILE(wp) >= FileE) {
	wp = SQUARE_FILE_MIRROR(wp);
	wk = SQUARE_FILE_MIRROR(wk);
	bk = SQUARE_FILE_MIRROR(bk);
	}

	if (kpk_draw(wp,wk,bk,COLOUR_OPP(board.turn))) {
	return true;
	}
	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KBKB) {

	// KBKB

	wb = board.piece[White][1];
	bb = board.piece[Black][1];

	if (SQUARE_COLOUR(wb) == SQUARE_COLOUR(bb)) {   // bishops on same colour
	return true;
	}
	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KBPK) {

	// KBPK (white)

	me = White;
	opp = COLOUR_OPP(me);

	wp = board.pawn[me][0];
	wb = board.piece[me][1];
	bk = KING_POS(board,opp);

	if (SQUARE_FILE(wp) >= FileE) {
	wp = SQUARE_FILE_MIRROR(wp);
	wb = SQUARE_FILE_MIRROR(wb);
	bk = SQUARE_FILE_MIRROR(bk);
	}

	if (kbpk_draw(wp,wb,bk)) {
	return true;
	}
	ifelse = false;
	}

	if (ifelse && mat_info.recog == MAT_KKBP) {

	// KBPK (black)

	me = Black;
	opp = COLOUR_OPP(me);

	wp = SQUARE_RANK_MIRROR(board.pawn[me][0]);
	wb = SQUARE_RANK_MIRROR(board.piece[me][1]);
	bk = SQUARE_RANK_MIRROR(KING_POS(board,opp));

	if (SQUARE_FILE(wp) >= FileE) {
	wp = SQUARE_FILE_MIRROR(wp);
	wb = SQUARE_FILE_MIRROR(wb);
	bk = SQUARE_FILE_MIRROR(bk);
	}

	if (kbpk_draw(wp,wb,bk)) {
	return true;
	}
	ifelse = false;
	}

	if (ifelse) {
	//ASSERT(623, false);
	}

	return false;
	}

	// kpk_draw()

	private boolean kpk_draw( int wp, int wk, int bk, int turn ) {

	int wp_file  = 0;   // int
	int wp_rank  = 0;   // int
	int wk_file  = 0;   // int
	int bk_file  = 0;   // int
	int bk_rank  = 0;   // int
	boolean ifelse = false;

	//ASSERT(624, SQUARE_IS_OK(wp));
	//ASSERT(625, SQUARE_IS_OK(wk));
	//ASSERT(626, SQUARE_IS_OK(bk));
	//ASSERT(627, COLOUR_IS_OK(turn));

	//ASSERT(628, SQUARE_FILE(wp)<=FileD);

	wp_file = SQUARE_FILE(wp);
	wp_rank = SQUARE_RANK(wp);

	wk_file = SQUARE_FILE(wk);

	bk_file = SQUARE_FILE(bk);
	bk_rank = SQUARE_RANK(bk);

	ifelse = true;
	if (ifelse && (bk == wp+16)) {

	if (wp_rank <= Rank6) {

	return true;

	} else { 

	//ASSERT(629, wp_rank==Rank7);

	if (COLOUR_IS_WHITE(turn)) {
	if (wk == wp-15  ||  wk == wp-17) {
	return true;
	}
	} else { 
	if (wk != wp-15  &&  wk != wp-17) {
	return true;
	}
	}
	}
	ifelse = false;
	}

	if (ifelse && (bk == wp+32)) {

	if (wp_rank <= Rank5) {

	return true;

	} else { 

	//ASSERT(630, wp_rank==Rank6);

	if (COLOUR_IS_WHITE(turn)) {
	if (wk != wp-1  &&  wk != wp+1) {
	return true;
	}
	} else { 
	return true;
	}
	}

	ifelse = false;
	}

	if (ifelse && (wk == wp-1  ||  wk == wp+1)) {

	if (bk == wk+32  &&  COLOUR_IS_WHITE(turn)) {    // opposition
	return true;
	}

	ifelse = false;
	}

	if (ifelse && (wk == wp+15  ||  wk == wp+16  ||  wk == wp+17)) {

	if (wp_rank <= Rank4) {
	if (bk == wk+32  &&  COLOUR_IS_WHITE(turn)) {   // opposition
	return true;
	}
	}
	ifelse = false;
	}

	// rook pawn

	if (wp_file == FileA) {

	if (DISTANCE(bk,A8) <= 1) {
	return true;
	}

	if (wk_file == FileA) {
	if (wp_rank == Rank2) {
	wp_rank++; // HACK
	}
	if (bk_file == FileC  &&  bk_rank > wp_rank) {
	return true;
	}
	}
	}

	return false;

	}

	// kbpk_draw()

	private boolean kbpk_draw ( int wp, int wb, int bk ) {

	//ASSERT(631, SQUARE_IS_OK(wp));
	//ASSERT(632, SQUARE_IS_OK(wb));
	//ASSERT(633, SQUARE_IS_OK(bk));

	if (SQUARE_FILE(wp) == FileA
	&&  DISTANCE(bk,A8) <= 1
	&&  SQUARE_COLOUR(wb) != SQUARE_COLOUR(A8)) {
	return true;
	}

	return false;
	}

	// end of recog.cpp



	// search.cpp



	// depth_is_ok()

	private boolean depth_is_ok( int depth ) { 

	return (depth > -128)  &&  (depth < DepthMax);
	}

	// height_is_ok()

	private boolean height_is_ok( int height ) { 

	return (height >= 0)  &&  (height < HeightMax);
	}

	// search_clear()

	private void search_clear( boolean inpSearch )  {

	if(inpSearch) this.SearchInput = new search_input_t();
	this.SearchInfo = new search_info_t();
	this.SearchRoot = new search_root_t();
	this.SearchCurrent = new search_current_t();
	this.SearchBest = new search_best_t();
	
	}

	// search()

	private void search()  {

	int move  = MoveNone;    // int
	int depth  = 0;          // int

	//ASSERT(634, board_is_ok(this.SearchInput.board));

	// opening book

	if (option_get_bool("OwnBook")  && (! this.SearchInput.infinite)) {

	// no book here
	// move = book_move(this.SearchInput.board);

	if (move != MoveNone) {

	// play book move

	this.SearchBest.move = move;
	this.SearchBest.value = 1;
	this.SearchBest.flags = SearchExact;
	this.SearchBest.depth = 1;
	this.SearchBest.pv[0] = move;
	this.SearchBest.pv[1] = MoveNone;

	search_update_best();

	return;
	}
	}

	// this.SearchInput

	gen_legal_moves(this.SearchInput.list,this.SearchInput.board);
	my_timer_reset(this.SearchCurrent.timer);
	my_timer_start(this.SearchCurrent.timer);
	
	if ( this.SearchInput.list.size <= 1) {
	this.SearchInput.depth_is_limited = true;
	this.SearchInput.depth_limit = 4;        // was 1
	}

	// this.SearchInfo

	this.setjmp = false;
	
	while(true)
	{	// setjmp loop
		

	if (this.setjmp) {
	this.setjmp = false;
	//ASSERT(635, this.SearchInfo.can_stop);
	//ASSERT(636, this.SearchBest.move!=MoveNone);
	search_update_current();
	return;
	}

	// this.SearchRoot

	list_copy(this.SearchRoot.list,this.SearchInput.list);

	// this.SearchCurrent

	board_copy(this.SearchCurrent.board,this.SearchInput.board);
	
	// init

	trans_inc_date();

	sort_init1();
	search_full_init(this.SearchRoot.list,this.SearchCurrent.board);

	// iterative deepening

	for (depth = 1; depth< DepthMax; depth++ ) {

	if (DispDepthStart) {
	send("info depth " + string_from_int(depth));
	}

	this.SearchRoot.bad_1 = false;
	this.SearchRoot.change = false;

	board_copy(this.SearchCurrent.board,this.SearchInput.board);

	if (UseShortSearch  &&  depth <= ShortSearchDepth) {
	search_full_root(this.SearchRoot.list,this.SearchCurrent.board,depth,SearchShort);
	if (this.setjmp) {
	break;
	}
	} else { 
	search_full_root(this.SearchRoot.list,this.SearchCurrent.board,depth,SearchNormal);
	if (this.setjmp) {
	break;
	}
	}

	search_update_current();

	if (DispDepthEnd) {
	send_ndtm(6);
	}

	// update search info

	if (depth >= 1) {
	this.SearchInfo.can_stop = true;
	}

	if (depth == 1
	&&  this.SearchRoot.list.size >= 2
	&&  this.SearchRoot.list.value[0] >= this.SearchRoot.list.value[1] + EasyThreshold) {
	this.SearchRoot.easy = true;
	}

	if (UseBad  &&  depth > 1) {
	this.SearchRoot.bad_2 = this.SearchRoot.bad_1;
	this.SearchRoot.bad_1 = false;
	//ASSERT(637, this.SearchRoot.bad_2==(this.SearchBest.value<=this.SearchRoot.last_value-BadThreshold));
	}

	this.SearchRoot.last_value = this.SearchBest.value;

	// stop search?

	if (this.SearchInput.depth_is_limited
	&&  depth >= this.SearchInput.depth_limit) {
	this.SearchRoot.flag = true;
	}

	if (this.SearchInput.time_is_limited
	&&  this.SearchCurrent.time >= this.SearchInput.time_limit_1
	&&  (! this.SearchRoot.bad_2)) {
	this.SearchRoot.flag = true;
	}

	if (UseEasy
	&&  this.SearchInput.time_is_limited
	&&  this.SearchCurrent.time >= this.SearchInput.time_limit_1 * EasyRatio/100
	&&  this.SearchRoot.easy) {
	//ASSERT(638, ! this.SearchRoot.bad_2);
	//ASSERT(639, ! this.SearchRoot.change);
	this.SearchRoot.flag = true;
	}

	if (UseEarly
	&&  this.SearchInput.time_is_limited
	&&  this.SearchCurrent.time >= this.SearchInput.time_limit_1 * EarlyRatio/100
	&&  (! this.SearchRoot.bad_2)
	&&  (! this.SearchRoot.change)) {
	this.SearchRoot.flag = true;
	}

	if (this.SearchInfo.can_stop
	&&  (this.SearchInfo.stop  ||  (this.SearchRoot.flag  &&  (! this.SearchInput.infinite)))) {
	return;
	}
	}

	}	// setjmp loop

	}

	// search_update_best()

	private void search_update_best()  {

	int move = 0;   // int
	int value = 0;   // int
	int flags = 0;   // int

	int[] pv = null;   // *int[]

	int mate  = 0;                     // int
	string_t move_string = new string_t();     // string
	string_t pv_string = new string_t();       // string

	search_update_current();

	if (DispBest) {

	move = this.SearchBest.move;
	value = this.SearchBest.value;
	flags = this.SearchBest.flags;

	pv = this.SearchBest.pv;

	move_to_string(move,move_string);
	pv_to_string(pv,pv_string);

	mate = value_to_mate(value);
	this.SearchCurrent.mate = mate;

	if (mate == 0) {

	// normal evaluation

	if (flags == SearchExact) {
	send_ndtm(10);
	} else { 
	if (flags == SearchLower) {
	send_ndtm(11);
	} else { 
	if (flags == SearchUpper) {
	send_ndtm(12);
	}
	}
	}

	} else { 

	// mate announcement

	if (flags == SearchExact) {
	send_ndtm(20);
	} else { 
	if (flags == SearchLower) {
	send_ndtm(21);
	} else { 
	if (flags == SearchUpper) {
	send_ndtm(22);
	}
	}
	}

	}
	}

	// update time-management info

	if (UseBad  &&  this.SearchBest.depth > 1) {
	if (this.SearchBest.value <= this.SearchRoot.last_value - BadThreshold) {
	this.SearchRoot.bad_1 = true;
	this.SearchRoot.easy = false;
	this.SearchRoot.flag = false;
	} else { 
	this.SearchRoot.bad_1 = false;
	}
	}

	}

	// search_update_root()

	private void search_update_root()  {

	int move  = 0;       // int
	int move_pos  = 0;   // int

	string_t move_string = new string_t();  // string

	if (DispRoot) {

	search_update_current();

	if (this.SearchCurrent.time >= 1000) {

	move = this.SearchRoot.move;
	move_pos = this.SearchRoot.move_pos;

	move_to_string(move,move_string);

	send("info currmove " + move_string.v + " currmovenumber "+ string_from_int(move_pos+1));
	}

	}
	}

	// search_update_current()

	private void search_update_current()  {

	my_timer_t timer = new my_timer_t();
	int node_nb = 0;
	long etime = 0;
	long speed = 0;

	timer = this.SearchCurrent.timer;
	node_nb = this.SearchCurrent.node_nb;

	etime = my_timer_elapsed_real(timer);
	speed = (etime >= 1000 ? node_nb * 1000 / etime : 0 );

	this.SearchCurrent.time = etime;
	this.SearchCurrent.speed = speed;

	}

	// search_check()

	private void search_check()   {

	 search_send_stat();

	// event();

	if (this.SearchInput.depth_is_limited
	&&  this.SearchRoot.depth > this.SearchInput.depth_limit) {
	this.SearchRoot.flag = true;
	}

	if (this.SearchInput.time_is_limited
	&&  this.SearchCurrent.time >= this.SearchInput.time_limit_2) {
	this.SearchRoot.flag = true;
	}

	if (this.SearchInput.time_is_limited
	&&  this.SearchCurrent.time >= this.SearchInput.time_limit_1
	&&  (! this.SearchRoot.bad_1)
	&&  (! this.SearchRoot.bad_2)
	&&  ((! UseExtension ) ||  this.SearchRoot.move_pos == 0)) {
	this.SearchRoot.flag = true;
	}

	if (this.SearchInfo.can_stop
	&&  (this.SearchInfo.stop  ||  (this.SearchRoot.flag  &&  (! this.SearchInput.infinite)))) {
	this.setjmp = true;  // the same as  longjmp(this.SearchInfo.buf,1);
	}

	}

	// search_send_stat()


	private void search_send_stat()   {

	search_update_current();

	if (DispStat  &&  this.SearchCurrent.time >= this.SearchInfo.last_time + 1000) {  // at least one-second gap

	this.SearchInfo.last_time = this.SearchCurrent.time;

	send_ndtm(3);

	if(this.DispStats) trans_stats(this.Trans);
	}

	}

	// end of search.cpp



	// search_full.cpp



	// search_full_init()

	private void search_full_init( list_t list, board_t board ) { 

	String str1 = "";     // string
	int tmove  = 0;     // int

	//ASSERT(640, list_is_ok(list));
	//ASSERT(641, board_is_ok(board));

	// null-move options

	str1 = option_get_string("nullMove Pruning");

	if (string_equal(str1,"Always")) {
	this.Usenull = true;
	this.UsenullEval = false;
	} else { 
	if (string_equal(str1,"Fail High")) {
	this.Usenull = true;
	this.UsenullEval = true;
	} else { 
	if (string_equal(str1,"Never")) {
	this.Usenull = false;
	this.UsenullEval = false;
	} else { 
	//ASSERT(642, false);
	this.Usenull = true;
	this.UsenullEval = true;
	}
	}
	}

	this.nullReduction = option_get_int("nullMove Reduction");

	str1 = option_get_string("Verification Search");

	if (string_equal(str1,"Always")) {
	this.UseVer = true;
	this.UseVerEndgame = false;
	} else { 
	if (string_equal(str1,"endgame")) {
	this.UseVer = true;
	this.UseVerEndgame = true;
	} else { 
	if (string_equal(str1,"Never")) {
	this.UseVer = false;
	this.UseVerEndgame = false;
	} else { 
	//ASSERT(643, false);
	this.UseVer = true;
	this.UseVerEndgame = true;
	}
	}
	}

	this.VerReduction = option_get_int("Verification Reduction");

	// history-pruning options

	this.UseHistory = option_get_bool("History Pruning");
	this.HistoryValue = (option_get_int("History Threshold") * 16384 + 50) / 100;

	// futility-pruning options

	this.UseFutility = option_get_bool("Futility Pruning");
	this.FutilityMargin = option_get_int("Futility Margin");

	// delta-pruning options

	this.UseDelta = option_get_bool("Delta Pruning");
	this.DeltaMargin = option_get_int("Delta Margin");

	// quiescence-search options

	this.CheckNb = option_get_int("Quiescence Check Plies");
	this.CheckDepth = 1 - this.CheckNb;

	// standard sort

	list_note(list);
	list_sort(list);

	// basic sort

	tmove = MoveNone;
	if (UseTrans) {
	trans_retrieve(this.Trans, board.key, this.TransRv);
	tmove = this.TransRv.trans_move;
	}

	note_moves(list,board,0,tmove);
	list_sort(list);
	}

	// search_full_root()

	private int search_full_root( list_t list, board_t board, int depth, int search_type ) {

	int value  = 0;   // int

	//ASSERT(644, list_is_ok(list));
	//ASSERT(645, board_is_ok(board));
	//ASSERT(646, depth_is_ok(depth));
	//ASSERT(647, search_type==SearchNormal || search_type==SearchShort);

	//ASSERT(648, list==this.SearchRoot.list);
	//ASSERT(649, ! (list.size==0));
	//ASSERT(650, board==this.SearchCurrent.board);
	//ASSERT(651, board_is_legal(board));
	//ASSERT(652, depth>=1);

	value = full_root(list,board,-ValueInf, ValueInf,depth,0,search_type);
	if( this.setjmp ) {
	return 0;
	}

	//ASSERT(653, value_is_ok(value));
	//ASSERT(654, list.value[0]==value);

	return value;
	}

	// full_root()

	private int full_root( list_t list, board_t board, int alpha, int beta, int depth, int height, int search_type ) {

	int old_alpha  = 0;    // int
	int value  = 0;        // int
	int best_value  = 0;   // int
	int i  = 0;            // int
	int move  = 0;         // int
	int new_depth ;        // int
	undo_t undo = new undo_t();  // undo_t[1]
	int[] new_pv = new int[HeightMax];      // int[HeightMax];

	//ASSERT(655, list_is_ok(list));
	//ASSERT(656, board_is_ok(board));
	//ASSERT(657, range_is_ok(alpha,beta));
	//ASSERT(658, depth_is_ok(depth));
	//ASSERT(659, height_is_ok(height));
	//ASSERT(660, search_type==SearchNormal || search_type==SearchShort);

	//ASSERT(661, list.size==this.SearchRoot.list.size);
	//ASSERT(662, ! (list.size==0));
	//ASSERT(663, board.key==this.SearchCurrent.board.key);
	//ASSERT(664, board_is_legal(board));
	//ASSERT(665, depth>=1);

	// init

	this.SearchCurrent.node_nb++;
	this.SearchInfo.check_nb--;

	for (i = 0; i< list.size; i++ ) {
	list.value[i] = ValueNone;
	}

	old_alpha = alpha;
	best_value = ValueNone;

	// move loop

	for (i = 0; i< list.size; i++ ) {

	move = list.move[i];

	this.SearchRoot.depth = depth;
	this.SearchRoot.move = move;
	this.SearchRoot.move_pos = i;
	this.SearchRoot.move_nb = list.size;

	search_update_root();

	new_depth = full_new_depth(depth,move,board,board_is_check(board) && list.size==1,true);

	move_do(board,move,undo);

	if (search_type == SearchShort  ||  best_value == ValueNone) {   // first move
	value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NodePV);
	if( this.setjmp ) {
	return 0;
	}
	} else { // other moves
	value = -full_search(board,-alpha-1,-alpha,new_depth,height+1,new_pv,NodeCut);
	if( this.setjmp ) {
	return 0;
	}
	if (value > alpha) {   //  &&  value < beta
	this.SearchRoot.change = true;
	this.SearchRoot.easy = false;
	this.SearchRoot.flag = false;
	search_update_root();
	value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NodePV);
	if( this.setjmp ) {
	return 0;
	}
	}
	}

	move_undo(board,move,undo);

	if (value <= alpha) {    // upper bound
	list.value[i] = old_alpha;
	} else { 
	if (value >= beta) {    // lower bound
	list.value[i] = beta;
	} else {      // alpha < value < beta => exact value
	list.value[i] = value;
	}
	}

	if (value > best_value  &&  (best_value == ValueNone  ||  value > alpha)) {

	this.SearchBest.move = move;
	this.SearchBest.value = value;
	if (value <= alpha) {    // upper bound
	this.SearchBest.flags = SearchUpper;
	} else { 
	if (value >= beta) {    // lower bound
	this.SearchBest.flags = SearchLower;
	} else {   // alpha < value < beta => exact value
	this.SearchBest.flags = SearchExact;
	}
	}
	this.SearchBest.depth = depth;

	//unshift is faster, but not used here
	pv_cat(this.SearchBest.pv, new_pv, move);

	search_update_best();
	}

	if (value > best_value) {
	best_value = value;
	if (value > alpha) {
	if (search_type == SearchNormal) {
	alpha = value;
	}
	if (value >= beta) {
	break;
	}
	}
	}
	}

	//ASSERT(666, value_is_ok(best_value));

	list_sort(list);

	//ASSERT(667, this.SearchBest.move==list.move[0]);
	//ASSERT(668, this.SearchBest.value==best_value);

	if (UseTrans  &&  best_value > old_alpha  &&  best_value < beta) {
	pv_fill(this.SearchBest.pv, 0, board);
	}

	return best_value;

	}

	// full_search()

	private int full_search( board_t board, int alpha, int beta, int depth, int height, int[] pv, int node_type ) {

	boolean in_check = false;       // bool
	boolean single_reply = false;   // bool
	int tmove  = 0;       // int
	int tdepth  = 0;      // int

	int min_value  = 0;   // int
	int max_value  = 0;   // int
	int old_alpha  = 0;   // int
	int value  = 0;       // int
	int best_value  = 0;  // int

	int_t bmove = new int_t(); // int
	int move  = 0;        // int

	int best_move  = 0;   // int
	int new_depth  = 0;   // int
	int played_nb  = 0;   // int
	int i  = 0;           // int
	int opt_value  = 0;   // int
	boolean reduced = false;      // bool
	attack_t attack = new attack_t();  // attack_t[1]
	sort_t sort = new sort_t();      // sort_t[1]
	undo_t undo = new undo_t();      // undo_t[1]
	int[] new_pv = new int[HeightMax];     // int[HeightMax]
	int[] played = new int[256];          // int[256]
	boolean gotocut = false;
	boolean cont = false;

	//ASSERT(670, range_is_ok(alpha,beta));
	//ASSERT(671, depth_is_ok(depth));
	//ASSERT(672, height_is_ok(height));

	//ASSERT(674, node_type==NodePV || node_type==NodeCut || node_type==NodeAll);

	//ASSERT(675, board_is_legal(board));

	// horizon?

	if (depth <= 0) {
	return full_quiescence(board,alpha,beta,0,height,pv);
	}

	// init

	this.SearchCurrent.node_nb++;
	this.SearchInfo.check_nb--;
	pv[0] = MoveNone;

	if (height > this.SearchCurrent.max_depth) {
	this.SearchCurrent.max_depth = height;
	}

	if (this.SearchInfo.check_nb <= 0) {
	this.SearchInfo.check_nb = this.SearchInfo.check_nb + this.SearchInfo.check_inc;
	search_check();
	if( this.setjmp ) {
	return 0;
	}
	}

	// draw?

	if (board_is_repetition(board)  ||  recog_draw(board)) {
	return ValueDraw;
	}

	// mate-distance pruning

	if (UseDistancePruning) {

	// lower bound

	value = (height+2-ValueMate); // does not work if the current position is mate
	if (value > alpha  &&  board_is_mate(board)) {
	value = (height-ValueMate);
	}

	if (value > alpha) {
	alpha = value;
	if (value >= beta) {
	return value;
	}
	}

	// upper bound

	value = -(height+1-ValueMate);

	if (value < beta) {
	beta = value;
	if (value <= alpha) {
	return value;
	}
	}
	}

	// transposition table

	tmove = MoveNone;

	if (UseTrans  &&  depth >= TransDepth) {

	if( trans_retrieve(this.Trans, board.key, this.TransRv)) {

	tmove = this.TransRv.trans_move;

	// trans_move is now updated

	if (node_type != NodePV) {

	if (UseMateValues) {

	if (this.TransRv.trans_min_value > ValueEvalInf  &&  this.TransRv.trans_min_depth < depth) {
	this.TransRv.trans_min_depth = depth;
	}

	if (this.TransRv.trans_max_value < -ValueEvalInf  &&  this.TransRv.trans_max_depth < depth) {
	this.TransRv.trans_max_depth = depth;
	}
	}

	min_value = -ValueInf;

	if ( this.TransRv.trans_min_depth >= depth ) {
	min_value = value_from_trans(this.TransRv.trans_min_value,height);
	if (min_value >= beta) {
	return min_value;
	}
	}

	max_value = ValueInf;

	if ( this.TransRv.trans_max_depth >= depth ) {
	max_value = value_from_trans(this.TransRv.trans_max_value,height);
	if (max_value <= alpha) {
	return max_value;
	}
	}

	if (min_value == max_value) {
	return min_value; // exact match
	}
	}
	}
	}

	// height limit

	if (height >= HeightMax-1) {
	return evalpos(board);
	}

	// more init

	old_alpha = alpha;
	best_value = ValueNone;
	best_move = MoveNone;
	played_nb = 0;

	attack_set(attack,board);
	in_check = ATTACK_IN_CHECK(attack);

	// null-move pruning

	if (this.Usenull  &&  depth >=nullDepth  &&  node_type != NodePV) {

	if ((! in_check)
	&&  (! value_is_mate(beta))
	&&  do_null(board)
	&&  ((! this.UsenullEval ) ||  depth <=nullReduction+1  ||  evalpos(board) >= beta)) {

	// null-move search

	new_depth = depth - this.nullReduction - 1;

	move_do_null(board,undo);
	value = -full_search(board,-beta,-beta+1,new_depth,height+1,new_pv,-node_type);
	if( this.setjmp ) {
	return 0;
	}
	move_undo_null(board,undo);

	// verification search

	if (this.UseVer  &&  depth > this.VerReduction) {

	if (value >= beta  &&  ((! this.UseVerEndgame)  ||  do_ver(board))) {

	new_depth = depth - this.VerReduction;
	//ASSERT(676, new_depth>0);

	value = full_no_null(board,alpha,beta,new_depth,height,new_pv,NodeCut,tmove, bmove);
	move = bmove.v;

	if( this.setjmp ) {
	return 0;
	}
	if (value >= beta) {
	//ASSERT(677, move==new_pv[0]);
	played[played_nb] = move;
	played_nb++;
	best_move = move;
	best_value = value;

	pv_copy(pv,new_pv);

	gotocut = true;
	}
	}
	}

	// pruning

	if ((! gotocut) && value >= beta) {

	if (value > ValueEvalInf) {
	value = ValueEvalInf; // do not return unproven mates
	}
	//ASSERT(678, ! value_is_mate(value));

	// pv_cat(pv,new_pv,Movenull);

	best_move = MoveNone;
	best_value = value;
	gotocut = true;
	}
	}
	}

	if(! gotocut) {  // [1]

	// Internal Iterative Deepening

	if (UseIID  &&  depth >= IIDDepth  &&  node_type == NodePV  &&  tmove == MoveNone) {

	new_depth = depth - IIDReduction;
	//ASSERT(679, new_depth>0);

	value = full_search(board,alpha,beta,new_depth,height,new_pv,node_type);
	if( this.setjmp ) {
	return 0;
	}
	if (value <= alpha) {
	value = full_search(board,-ValueInf,beta,new_depth,height,new_pv,node_type);
	if( this.setjmp ) {
	return 0;
	}
	}

	tmove = new_pv[0];
	}

	// move generation

	sort_init2(sort,board,attack,depth,height,tmove);

	single_reply = false;
	if (in_check  &&  sort.list.size == 1) {
	single_reply = true; // HACK
	}

	// move loop

	opt_value = ValueInf;

	while(true) {

	move = sort_next(sort);
	if(move == MoveNone) {
	break;
	}

	// extensions

	new_depth = full_new_depth(depth,move,board,single_reply,node_type==NodePV);

	// history pruning

	reduced = false;

	if (this.UseHistory  &&  depth >= HistoryDepth  &&  node_type != NodePV) {
	if ((! in_check)  &&  played_nb >= HistoryMoveNb  &&  new_depth < depth) {
	//ASSERT(680, best_value!=ValueNone);
	//ASSERT(681, played_nb>0);
	//ASSERT(682, sort.pos>0 && move==sort.list.move[sort.pos-1]);
	value = sort.value; // history score
	if (value < this.HistoryValue) {
	//ASSERT(683, value>=0 && value<16384);
	//ASSERT(684, move!=tmove);
	//ASSERT(685, ! move_is_tactical(move,board));
	//ASSERT(686, ! move_is_check(move,board));
	new_depth--;
	reduced = true;
	}
	}
	}

	// futility pruning

	if (this.UseFutility  &&  depth == 1  &&  node_type != NodePV) {

	if ((! in_check)  &&  new_depth == 0  &&  (! move_is_tactical(move,board))
	&&  (! move_is_dangerous(move,board))) {

	//ASSERT(687, ! move_is_check(move,board));

	// optimistic evaluation

	if (opt_value == ValueInf) {
	opt_value = evalpos(board) + this.FutilityMargin;
	//ASSERT(688, opt_value<ValueInf);
	}

	value = opt_value;

	// pruning

	if (value <= alpha) {

	if (value > best_value) {
	best_value = value;
	pv[0] = MoveNone;
	}

	cont = true;
	}
	}
	}

	if(cont) {  // continue [1]
	cont = false;
	} else { 

	// recursive search

	move_do(board,move,undo);

	if (node_type != NodePV  ||  best_value == ValueNone) {    // first move
	value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,-node_type);
	if( this.setjmp ) {
	return 0;
	}
	} else {    // other moves
	value = -full_search(board,-alpha-1,-alpha,new_depth,height+1,new_pv,NodeCut);
	if( this.setjmp ) {
	return 0;
	}
	if (value > alpha) {    //  &&  value < beta
	value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,NodePV);
	if( this.setjmp ) {
	return 0;
	}
	}
	}

	// history-pruning re-search

	if (HistoryReSearch  &&  reduced  &&  value >= beta) {

	//ASSERT(689, node_type!=NodePV);

	new_depth++;
	//ASSERT(690, new_depth==depth-1);

	value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,-node_type);
	if( this.setjmp ) {
	return 0;
	}
	}

	move_undo(board,move,undo);

	played[played_nb] = move;
	played_nb++;

	if (value > best_value) {
	best_value = value;
	pv_cat(pv,new_pv,move);
	if (value > alpha) {
	alpha = value;
	best_move = move;
	if (value >= beta) {
	gotocut = true;
	break;
	}
	}
	}

	if (node_type == NodeCut) {
	node_type = NodeAll;
	}

	}  // continue [1]

	}


	if(! gotocut) {  // [2]

	// ALL node

	if (best_value == ValueNone) {    // no legal move
	if (in_check) {
	//ASSERT(691, board_is_mate(board));
	return (height-ValueMate);
	} else { 
	//ASSERT(692, board_is_stalemate(board));
	return ValueDraw;
	}
	}

	} // goto cut [2]
	} // goto cut [1]

	// cut:

	//ASSERT(693, value_is_ok(best_value));

	// move ordering

	if (best_move != MoveNone) {

	good_move(best_move,board,depth,height);

	if (best_value >= beta  &&  (! move_is_tactical(best_move,board))) {

	//ASSERT(694, played_nb>0 && played[played_nb-1]==best_move);

	for (i = 0; i<= played_nb-2; i++ ) {
	move = played[i];
	//ASSERT(695, move!=best_move);
	history_bad(move,board);
	}

	history_good(best_move,board);
	}
	}

	// transposition table

	if (UseTrans  &&  depth >= TransDepth) {

	tmove = best_move;
	tdepth = depth;
	this.TransRv.trans_min_value = ( best_value > old_alpha ?  value_to_trans(best_value,height) : -ValueInf );
	this.TransRv.trans_max_value = ( best_value < beta ? value_to_trans(best_value,height) : ValueInf );

	trans_store(this.Trans,board.key, tmove, tdepth, this.TransRv);
	}

	return best_value;

	}


	// full_no_null()

	private int full_no_null( board_t board, int alpha, int beta, int depth, int height, int[] pv, int node_type, int tmove,  int_t b_move ) {

	int value  = 0;            // int
	int best_value  = 0;       // int
	int move  = 0;             // int
	int new_depth  = 0;        // int

	attack_t attack = new attack_t();  // attack_t[1]
	sort_t sort = new sort_t();      // sort_t[1]
	undo_t undo = new undo_t();      // undo_t[1]
	int[] new_pv = new int[HeightMax];    // int[HeightMax]
	boolean gotocut = false;

	//ASSERT(697, range_is_ok(alpha,beta));
	//ASSERT(698, depth_is_ok(depth));
	//ASSERT(699, height_is_ok(height));
	//ASSERT(701, node_type==NodePV || node_type==NodeCut || node_type==NodeAll);
	//ASSERT(702, tmove==MoveNone || move_is_ok(tmove));

	//ASSERT(704, board_is_legal(board));
	//ASSERT(705, ! board_is_check(board));
	//ASSERT(706, depth>=1);

	// init

	this.SearchCurrent.node_nb++;
	this.SearchInfo.check_nb--;
	pv[0] = MoveNone;

	if (height > this.SearchCurrent.max_depth) {
	this.SearchCurrent.max_depth = height;
	}

	if (this.SearchInfo.check_nb <= 0) {
	this.SearchInfo.check_nb+=this.SearchInfo.check_inc;
	search_check();
	if( this.setjmp ) {
	return 0;
	}
	}

	attack_set(attack,board);
	//ASSERT(707, ! ATTACK_IN_CHECK(attack));

	b_move.v = MoveNone;
	best_value = ValueNone;

	// move loop

	sort_init2(sort,board,attack,depth,height,tmove);


	while(true) {

	move = sort_next(sort);
	if(move == MoveNone) {
	break;
	}

	new_depth = full_new_depth(depth,move,board,false,false);

	move_do(board,move,undo);
	value = -full_search(board,-beta,-alpha,new_depth,height+1,new_pv,-node_type);
	if( this.setjmp ) {
	return 0;
	}
	move_undo(board,move,undo);

	if (value > best_value) {
	best_value = value;
	pv_cat(pv,new_pv,move);
	if (value > alpha) {
	alpha = value;
	b_move.v = move;
	if (value >= beta) {
	gotocut = true;
	break;
	}
	}
	}

	}

	if(! gotocut) {  // [1]

	// ALL node

	if (best_value == ValueNone) {     // no legal move => stalemate
	//ASSERT(708, board_is_stalemate(board));
	best_value = ValueDraw;
	}

	} // goto cut [1]

	// cut:

	//ASSERT(709, value_is_ok(best_value));

	return best_value;

	}

	// full_quiescence()

	private int full_quiescence( board_t board, int alpha, int beta, int depth, int height, int[] pv ) { 

	boolean in_check = false;     // bool
	int old_alpha  = 0;        // int

	int value  = 0;            // int
	int best_value  = 0;       // int
//	int best_move  = 0;       // int
	int opt_value  = 0;        // int
	int move  = 0;             // int

	int to  = 0;               // int
	int capture  = 0;          // int

	attack_t attack = new attack_t();  // attack_t[1]
	sort_t sort = new sort_t();      // sort_t[1]
	undo_t undo = new undo_t();      // undo_t[1]
	int[] new_pv = new int[HeightMax];    // int[HeightMax]

	boolean gotocut = false;
	boolean cont = false;

	//ASSERT(711, range_is_ok(alpha,beta));
	//ASSERT(712, depth_is_ok(depth));
	//ASSERT(713, height_is_ok(height));

	//ASSERT(715, board_is_legal(board));
	//ASSERT(716, depth<=0);

	// init

	this.SearchCurrent.node_nb++;
	this.SearchInfo.check_nb--;
	pv[0] = MoveNone;

	if (height > this.SearchCurrent.max_depth) {
	this.SearchCurrent.max_depth = height;
	}

	if (this.SearchInfo.check_nb <= 0) {
	this.SearchInfo.check_nb+=this.SearchInfo.check_inc;
	search_check();
	if( this.setjmp ) {
	return 0;
	}
	}

	// draw?

	if (board_is_repetition(board)  ||  recog_draw(board)) {
	return ValueDraw;
	}

	// mate-distance pruning

	if (UseDistancePruning) {

	// lower bound

	value = (height+2-ValueMate); // does not work if the current position is mate
	if (value > alpha  &&  board_is_mate(board)) {
	value = (height-ValueMate);
	}

	if (value > alpha) {
	alpha = value;
	if (value >= beta) {
	return value;
	}
	}

	// upper bound

	value = -(height+1-ValueMate);

	if (value < beta) {
	beta = value;
	if (value <= alpha) {
	return value;
	}
	}
	}

	// more init

	attack_set(attack,board);
	in_check = ATTACK_IN_CHECK(attack);

	if (in_check) {
	// not.sure //ASSERT(717, depth<0);
	depth++; // in-check extension
	}

	// height limit

	if (height >= HeightMax-1) {
	return evalpos(board);
	}

	// more init

	old_alpha = alpha;
	best_value = ValueNone;
//	best_move = MoveNone;

	// if (this.UseDelta)
	opt_value = ValueInf;

	if (! in_check) {

	// lone-king stalemate?

	if (simple_stalemate(board)) {
	return ValueDraw;
	}

	// stand pat

	value = evalpos(board);

	//ASSERT(718, value>best_value);
	best_value = value;
	if (value > alpha) {
	alpha = value;
	if (value >= beta) {
	gotocut = true;
	}
	}

	if ((! gotocut) && this.UseDelta) {
	opt_value +=  this.DeltaMargin;
	//ASSERT(719, opt_value<ValueInf);
	}
	}

	if(! gotocut) {  // [1]

	// move loop

	sort_init_qs(sort,board,attack,depth>=this.CheckDepth);


	while(true) {

	move = sort_next_qs(sort);
	if(move == MoveNone) {
	break;
	}


	// delta pruning

	if (this.UseDelta  &&  beta == old_alpha+1) {

	if ((! in_check) && (! move_is_check(move,board)) && (! capture_is_dangerous(move,board))) {

	//ASSERT(720, move_is_tactical(move,board));

	// optimistic evaluation

	value = opt_value;

	to = MOVE_TO(move);
	capture = board.square[to];

	if (capture != Empty) {
	value +=  this.ValuePiece[capture];
	} else { 
	if (MOVE_IS_EN_PASSANT(move)) {
	value +=  ValuePawn;
	}
	}

	if (MOVE_IS_PROMOTE(move)) {
	value +=  ValueQueen - ValuePawn;
	}

	// pruning

	if (value <= alpha) {

	if (value > best_value) {
	best_value = value;
	pv[0] = MoveNone;
	}

	cont = true;
	}
	}
	}

	if(cont) {  // continue [1]
	cont = false;
	} else { 

	move_do(board,move,undo);
	value = -full_quiescence(board,-beta,-alpha,depth-1,height+1,new_pv);
	if( this.setjmp ) {
	return 0;
	}
	move_undo(board,move,undo);

	if (value > best_value) {
	best_value = value;
	pv_cat(pv,new_pv,move);
	if (value > alpha) {
	alpha = value;
//	best_move = move;
	if (value >= beta) {
	gotocut = true;
	break;
	}
	}
	}

	}  // continue [1]

	}

	if(! gotocut) {  // [2]

	// ALL node

	if (best_value == ValueNone) {        // no legal move
	//ASSERT(721, board_is_mate(board));
	return (height-ValueMate);
	}

	} // goto cut [2]
	} // goto cut [1]

	// cut:

	//ASSERT(722, value_is_ok(best_value));

	return best_value;

	}

	// full_new_depth()

	private int full_new_depth( int depth, int move, board_t board, boolean single_reply, boolean in_pv ) {
	int new_depth  = 0;   // int
	boolean b = false;       // bool

	//ASSERT(723, depth_is_ok(depth));
	//ASSERT(724, move_is_ok(move));

	//ASSERT(728, depth>0);

	new_depth--;

	b = b || (single_reply  &&  ExtendSingleReply);
	b = b || (in_pv  &&  MOVE_TO(move) == board.cap_sq &&  see_move(move,board) > 0);  // recapture
	b = b || (in_pv  &&  PIECE_IS_PAWN(MOVE_PIECE(move,board))
	&&  PAWN_RANK(MOVE_TO(move),board.turn) == Rank7
	&&  see_move(move,board) >= 0);
	b = b || move_is_check(move,board);
	if(b) {
	new_depth++;
	}

	// not.sure //ASSERT(729, new_depth>=0 && new_depth<=depth);

	return new_depth;
	}

	// do_null()

	private boolean do_null( board_t board ) {


	// use null move if the side-to-move has at least one piece

	return (board.piece_size[board.turn] >= 2); // king + one piece
	}

	// do_ver()

	private boolean do_ver( board_t board ) {

	// use verification if the side-to-move has at most one piece

	return (board.piece_size[board.turn] <= 2); // king + one piece
	}

	// pv_fill()

	private void pv_fill( int[] pv, int at, board_t board )  {

	int move  = 0;   // int
	int tmove  = 0;  // int
	int tdepth  = 0; // int

	undo_t undo = new undo_t();      // undo_t[1]

	//ASSERT(734, UseTrans);

	move = pv[at];

	if (move != MoveNone  &&  move != Movenull) {

	move_do(board,move,undo);
	pv_fill(pv, at+1,board);
	move_undo(board,move,undo);

	tmove = move;
	tdepth = -127; // HACK
	this.TransRv.trans_min_value = -ValueInf;
	this.TransRv.trans_max_value = ValueInf;

	trans_store(this.Trans, board.key, tmove, tdepth, this.TransRv);
	}
	}

	// move_is_dangerous()

	private boolean move_is_dangerous( int move, board_t board ) {

	int piece  = 0;   // int

	//ASSERT(735, move_is_ok(move));

	//ASSERT(737, ! move_is_tactical(move,board));

	piece = MOVE_PIECE(move,board);

	if (PIECE_IS_PAWN(piece) &&  PAWN_RANK(MOVE_TO(move),board.turn) >= Rank7) {
	return true;
	}

	return false;
	}

	// capture_is_dangerous()

	private boolean capture_is_dangerous( int move, board_t board ) {

	int piece  = 0;     // int
	int capture  = 0;   // int

	//ASSERT(738, move_is_ok(move));

	//ASSERT(740, move_is_tactical(move,board));

	piece = MOVE_PIECE(move,board);

	if (PIECE_IS_PAWN(piece) &&  PAWN_RANK(MOVE_TO(move),board.turn) >= Rank7) {
	return true;
	}

	capture = move_capture(move,board);

	if (PIECE_IS_QUEEN(capture)) {
	return true;
	}

	if (PIECE_IS_PAWN(capture) &&  PAWN_RANK(MOVE_TO(move),board.turn) <= Rank2) {
	return true;
	}

	return false;
	}

	// simple_stalemate()

	private boolean simple_stalemate( board_t board ) {

	int me  = 0;         // int
	int opp  = 0;        // int
	int king  = 0;       // int
	int opp_flag  = 0;   // int
	int from  = 0;       // int
	int to  = 0;         // int
	int capture  = 0;    // int
	int inc_ptr  = 0;    // int
	int inc  = 0;        // int

	//ASSERT(742, board_is_legal(board));
	//ASSERT(743, ! board_is_check(board));

	// lone king?

	me = board.turn;
	if (board.piece_size[me] != 1  ||  board.pawn_size[me] != 0) {
	return false; // no
	}

	// king in a corner?

	king = KING_POS(board,me);
	if (king != A1  &&  king != H1  &&  king != A8  &&  king != H8) {
	return false; // no
	}

	// init

	opp = COLOUR_OPP(me);
	opp_flag = COLOUR_FLAG(opp);

	// king can move?

	from = king;

	inc_ptr = 0;
	while(true) {
	inc = KingInc[inc_ptr];
	if( inc == IncNone ) {
	break;
	}

	to = from + inc;
	capture = board.square[to];
	if (capture == Empty  ||  FLAG_IS(capture,opp_flag)) {
	if (! is_attacked(board,to,opp)) {
	return false; // legal king move
	}
	}

	inc_ptr++;
	}


	// no legal move

	//ASSERT(744, board_is_stalemate( board ));

	return true;
	}

	// end of search_full.cpp



	// see.cpp

	// types



	// see_move()

	private int see_move( int move, board_t board ) {

	int att  = 0;              // int
	int def  = 0;              // int
	int from  = 0;             // int
	int to  = 0;               // int
	int value  = 0;            // int
	int piece_value  = 0;      // int
	int piece  = 0;            // int
	int capture  = 0;          // int
	int pos  = 0;              // int
	alists_t alists = new alists_t();  // alists_t[1]
	alist_t alist = new alist_t();          // alist_t *

	//ASSERT(745, move_is_ok(move));

	// init

	from = MOVE_FROM(move);
	to = MOVE_TO(move);

	// move the piece

	piece_value = 0;

	piece = board.square[from];
	//ASSERT(747, piece_is_ok(piece));

	att = PIECE_COLOUR(piece);
	def = COLOUR_OPP(att);

	// promote

	if (MOVE_IS_PROMOTE(move)) {
	//ASSERT(748, PIECE_IS_PAWN(piece));
	piece = move_promote(move);
	//ASSERT(749, piece_is_ok(piece));
	//ASSERT(750, COLOUR_IS(piece,att));
	}

	piece_value += this.ValuePiece[piece];

	// clear attacker lists

	alist_clear(alists.alist[Black]);
	alist_clear(alists.alist[White]);

	// find hidden attackers

	alists_hidden(alists,board,from,to);

	// capture the piece

	value = 0;

	capture = board.square[to];

	if (capture != Empty) {

	//ASSERT(751, piece_is_ok(capture));
	//ASSERT(752, COLOUR_IS(capture,def));

	value +=  this.ValuePiece[capture];
	}

	// promote

	if (MOVE_IS_PROMOTE(move)) {
	value +=  this.ValuePiece[piece] - ValuePawn;
	}

	// en-passant

	if (MOVE_IS_EN_PASSANT(move)) {
	//ASSERT(753, value==0);
	//ASSERT(754, PIECE_IS_PAWN(board.square[SQUARE_EP_DUAL(to)]));
	value +=  ValuePawn;
	alists_hidden(alists,board,SQUARE_EP_DUAL(to),to);
	}

	// build defender list

	alist = alists.alist[def];

	alist_build(alist,board,to,def);
	if (alist.size == 0) {
	return value; // no defender => stop SEE
	}

	// build attacker list

	alist = alists.alist[att];

	alist_build(alist,board,to,att);

	// remove the moved piece (if it's an attacker)

	pos = 0;
	while( pos<alist.size  &&  alist.square[pos] != from ) {
	pos++;
	}

	if (pos < alist.size) {
	alist_remove(alist,pos);
	}

	// SEE search

	value -=  see_rec(alists,board,def,to,piece_value);

	return value;

	}

	// see_square()
/*
	private int see_square( board_t board, int to, int colour ) {

	int att  = 0;              // int
	int def  = 0;              // int
	int piece_value  = 0;      // int
	int piece  = 0;            // int
	alists_t alists = new alists_t();  // alists_t[1]
	alist_t alist = new alist_t();          // alist_t *

	//ASSERT(756, SQUARE_IS_OK(to));
	//ASSERT(757, COLOUR_IS_OK(colour));

	//ASSERT(758, COLOUR_IS(board.square[to],COLOUR_OPP(colour)));

	// build attacker list

	att = colour;

	alist = alists.alist[att];

	alist_clear(alist);

	alist_build(alist,board,to,att);

	if (alist.size == 0) {
	return 0; // no attacker => stop SEE
	}

	// build defender list

	def = COLOUR_OPP(att);
	alist = alists.alist[def];

	alist_clear(alist);

	alist_build(alist,board,to,def);

	// captured piece

	piece = board.square[to];
	//ASSERT(759, piece_is_ok(piece));
	//ASSERT(760, COLOUR_IS(piece,def));

	piece_value = this.ValuePiece[piece];

	// SEE search

	return see_rec(alists,board,att,to,piece_value);

	}

*/
	// see_rec()

	private int see_rec( alists_t alists, board_t board, int colour, int to, int piece_value )  {

	int from  = 0;    // int
	int piece  = 0;   // int
	int value  = 0;   // int

	//ASSERT(763, COLOUR_IS_OK(colour));
	//ASSERT(764, SQUARE_IS_OK(to));
	//ASSERT(765, piece_value>0);

	// find the least valuable attacker

	from = alist_pop(alists.alist[colour],board);
	if (from == SquareNone) {
	return 0; // no more attackers
	}

	// find hidden attackers

	alists_hidden(alists,board,from,to);

	// calculate the capture value

	value = piece_value; // captured piece
	if (value == ValueKing) {
	return value; // do not allow an answer to a king capture
	}

	piece = board.square[from];
	//ASSERT(766, piece_is_ok(piece));
	//ASSERT(767, COLOUR_IS(piece,colour));
	piece_value = this.ValuePiece[piece];

	// promote

	if (piece_value == ValuePawn  &&  this.SquareIsPromote[to]) {    // HACK: PIECE_IS_PAWN(piece)
	//ASSERT(768, PIECE_IS_PAWN(piece));
	piece_value = ValueQueen;
	value +=  ValueQueen - ValuePawn;
	}

	value -=  see_rec(alists,board,COLOUR_OPP(colour),to,piece_value);

	if (value < 0) {
	value = 0;
	}

	return value;

	}

	// alist_build()

	private void alist_build( alist_t alist, board_t board, int to, int colour )  {

	int ptr  = 0;    // int
	int from  = 0;   // int
	int piece  = 0;  // int
	int delta  = 0;  // int
	int inc  = 0;    // int
	int sq  = 0;     // int
	int pawn  = 0;   // int

	//ASSERT(771, SQUARE_IS_OK(to));
	//ASSERT(772, COLOUR_IS_OK(colour));

	// piece attacks

	ptr = 0;
	while(true) {

	from = board.piece[colour][ptr];

	if(from==SquareNone) {
	break;
	}

	piece = board.square[from];
	delta = to - from;

	if (PSEUDO_ATTACK(piece,delta)) {

	inc = DELTA_INC_ALL(delta);
	//ASSERT(773, inc!=IncNone);

	sq = from;
	while(true) {

	sq+=inc;
	if (sq == to) {  // attack
	alist_add(alist,from,board);
	break;
	}

	if(board.square[sq] != Empty) {
	break;
	}

	}
	}

	ptr++;
	}

	// pawn attacks

	inc = PawnMoveInc[colour];
	pawn = PawnMake[colour];

	from = to - (inc-1);
	if (board.square[from] == pawn) {
	alist_add(alist,from,board);
	}

	from = to - (inc+1);
	if (board.square[from] == pawn) {
	alist_add(alist,from,board);
	}

	}

	// alists_hidden()

	private void alists_hidden( alists_t alists, board_t board, int from, int to )  {

	int inc  = 0;     // int
	int sq  = 0;      // int
	int piece  = 0;   // int

	//ASSERT(776, SQUARE_IS_OK(from));
	//ASSERT(777, SQUARE_IS_OK(to));

	inc = DELTA_INC_LINE(to-from);

	if (inc != IncNone)  {  // line

	sq = from;

	while(true) {
	sq-=inc;
	piece = board.square[sq];
	if ( piece!= Empty) {
	break;
	}
	}

	if (SLIDER_ATTACK(piece,inc)) {

	//ASSERT(778, piece_is_ok(piece));
	//ASSERT(779, PIECE_IS_SLIDER(piece));

	alist_add(alists.alist[PIECE_COLOUR(piece)],sq,board);
	}
	}

	}

	// alist_clear()

	private void alist_clear( alist_t alist ) {

	alist = new alist_t();

	//alist.size = 0;
	//alist.square = [];

	}


	// alist_add()

	private void alist_add( alist_t alist, int square, board_t board )  {

	int piece  = 0;   // int
	int size  = 0;    // int
	int pos  = 0;     // int


	//ASSERT(782, SQUARE_IS_OK(square));


	// insert in MV order

	piece = board.square[square];

	alist.size++; // HACK
	size = alist.size;

	//ASSERT(784, size>0 && size<16);

	pos = size-1;
	while( pos > 0  &&  piece > board.square[alist.square[pos-1]]) {    // HACK
	//ASSERT(785, pos>0 && pos<size);
	alist.square[pos] = alist.square[pos-1];
	pos--;
	}

	//ASSERT(786, pos>=0 && pos<size);
	alist.square[pos] = square;

	}

	// alist_remove()

	private void alist_remove( alist_t alist, int pos )  {

	int size  = 0;  // int
	int i  = 0;     // int

	//ASSERT(788, pos>=0 && pos<alist.size);

	size = alist.size;
	alist.size--;     // HACK

	//ASSERT(789, size>=1);

	//ASSERT(790, pos>=0 && pos<size);

	for (i = pos; i<=size-2; i++ ) {
	//ASSERT(791, i>=0 && i<size-1);
	alist.square[i] = alist.square[i+1];
	}

	}

	// alist_pop()

	private int alist_pop( alist_t alist, board_t board ) {

	int sq  = 0;     // int
	int size  = 0;   // int


	sq = SquareNone;

	size = alist.size;

	if (size != 0) {
	size--;
	//ASSERT(794, size>=0);
	sq = alist.square[size];
	alist.size = size;
	}

	return sq;

	}

	// end of see.cpp



	// sort.cpp





	// sort_init()

	private void sort_init1()  {

	int i  = 0;        // int
	int height  = 0;   // int
	int pos  = 0;      // int

	// killer

	for (height = 0; height<HeightMax; height++ ) {
	//this.Killer[height] = [];
	for (i = 0; i<=1; i++ ) {
	this.Killer[height][i] = MoveNone;
	}
	}

	// history

	for (i = 0; i<HistorySize; i++ ) {
	this.History[i] = 0;
	this.HistHit[i] = 1;
	this.HistTot[i] = 1;
	}

	// Code[]

	for (pos = 0; pos< CODE_SIZE; pos++ ) {
	this.Code[pos] = GEN_ERROR;
	}

	pos = 0;

	// main search

	this.PosLegalEvasion = pos;
	this.Code[0] = GEN_LEGAL_EVASION;
	this.Code[1] = GEN_END;

	this.PosSEE = 2;
	this.Code[2] = GEN_TRANS;
	this.Code[3] = GEN_GOOD_CAPTURE;
	this.Code[4] = GEN_KILLER;
	this.Code[5] = GEN_QUIET;
	this.Code[6] = GEN_BAD_CAPTURE;
	this.Code[7] = GEN_END;

	// quiescence search

	this.PosEvasionQS = 8;
	this.Code[8] = GEN_EVASION_QS;
	this.Code[9] = GEN_END;

	this.PosCheckQS = 10;
	this.Code[10] = GEN_CAPTURE_QS;
	this.Code[11] = GEN_CHECK_QS;
	this.Code[12] = GEN_END;

	this.PosCaptureQS = 13;
	this.Code[13] = GEN_CAPTURE_QS;
	this.Code[14] = GEN_END;

	pos = 15;

	//ASSERT(795, pos<CODE_SIZE);

	}

	// sort_init()

	private void sort_init2( sort_t sort, board_t board, attack_t attack, int depth, int height, int trans_killer )  {

	//ASSERT(799, depth_is_ok(depth));
	//ASSERT(800, height_is_ok(height));
	//ASSERT(801, trans_killer==MoveNone || move_is_ok(trans_killer));

	sort.board = board;
	sort.attack = attack;

	sort.depth = depth;
	sort.height = height;

	sort.trans_killer = trans_killer;
	sort.killer_1 = this.Killer[sort.height][0];
	sort.killer_2 = this.Killer[sort.height][1];

	if (ATTACK_IN_CHECK(sort.attack)) {

	gen_legal_evasions(sort.list,sort.board,sort.attack);
	note_moves(sort.list,sort.board,sort.height,sort.trans_killer);
	list_sort(sort.list);

	sort.gen = this.PosLegalEvasion + 1;
	sort.test = TEST_NONE;

	} else {  // not in check

	//sort.list.size = 0;
	LIST_CLEAR(sort.list);
	
	sort.gen = this.PosSEE;

	}

	sort.pos = 0;
	}

	// sort_next()

	private int sort_next( sort_t sort ) {

	int move  = 0;   // int
	int gen  = 0;    // int
	boolean nocont = false;
	boolean ifelse = false;

	while (true) {

	while (sort.pos < sort.list.size) {

	nocont = true;

	// next move

	move = sort.list.move[sort.pos];
	sort.value = 16384; // default score
	sort.pos++;

	//ASSERT(803, move!=MoveNone);

	// test

	ifelse = true;
	if (ifelse && (sort.test == TEST_NONE)) {
			    ifelse = false;
	}

	if (ifelse && (sort.test == TEST_TRANS_KILLER)) {

	if (nocont && (! move_is_pseudo(move,sort.board))) {
	nocont = false;
	}
	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	ifelse = false;
	}

	if (ifelse && (sort.test == TEST_GOOD_CAPTURE)) {

	//ASSERT(804, move_is_tactical(move,sort.board));

	if (nocont && move == sort.trans_killer) {
	nocont = false;
	}

	if (nocont && (! capture_is_good(move,sort.board))) {
	LIST_ADD(sort.bad,move);
	nocont = false;
	}

	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	ifelse = false;
	}

	if (ifelse && (sort.test == TEST_BAD_CAPTURE)) {

	//ASSERT(805, move_is_tactical(move,sort.board));
	//ASSERT(806, (! capture_is_good(move,sort.board)));

	//ASSERT(807, move!=sort.trans_killer);
	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	ifelse = false;
	}

	if (ifelse && (sort.test == TEST_KILLER)) {

	if (nocont && move == sort.trans_killer) {
	nocont = false;
	}
	if (nocont && (! quiet_is_pseudo(move,sort.board))) {
	nocont = false;
	}
	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	//ASSERT(808, (! nocont) || (! move_is_tactical(move,sort.board)));

	ifelse = false;
	}

	if (ifelse && (sort.test == TEST_QUIET)) {

	//ASSERT(809, ! move_is_tactical(move,sort.board));

	if (nocont && move == sort.trans_killer) {
	nocont = false;
	}
	if (nocont && move == sort.killer_1) {
	nocont = false;
	}
	if (nocont && move == sort.killer_2) {
	nocont = false;
	}
	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	if (nocont) {
	sort.value = history_prob(move,sort.board);
	}

	ifelse = false;
	}

	if (ifelse) {

	//ASSERT(810, false);

	return MoveNone;
	}

	if (nocont) {

	//ASSERT(811, pseudo_is_legal(move,sort.board));
	return move;

	} // otherwise continue

	}

	// next stage

	gen = this.Code[sort.gen];
	sort.gen++;

	ifelse = true;

	if (ifelse && (gen == GEN_TRANS)) {

	LIST_CLEAR(sort.list);
	if (sort.trans_killer != MoveNone) {
	LIST_ADD(sort.list,sort.trans_killer);
	}

	sort.test = TEST_TRANS_KILLER;

	ifelse = false;
	}

	if (ifelse && (gen == GEN_GOOD_CAPTURE)) {

	gen_captures(sort.list,sort.board);
	note_mvv_lva(sort.list,sort.board);
	list_sort(sort.list);

	LIST_CLEAR(sort.bad);

	sort.test = TEST_GOOD_CAPTURE;

	ifelse = false;
	}

	if (ifelse && (gen == GEN_BAD_CAPTURE)) {

	list_copy(sort.list,sort.bad);

	sort.test = TEST_BAD_CAPTURE;

	ifelse = false;
	}

	if (ifelse && (gen == GEN_KILLER)) {

	LIST_CLEAR(sort.list);
	if (sort.killer_1 != MoveNone) {
	LIST_ADD(sort.list,sort.killer_1);
	}
	if (sort.killer_2 != MoveNone) {
	LIST_ADD(sort.list,sort.killer_2);
	}

	sort.test = TEST_KILLER;

	ifelse = false;
	}

	if (ifelse && (gen == GEN_QUIET)) {

	gen_quiet_moves(sort.list,sort.board);
	note_quiet_moves(sort.list,sort.board);
	list_sort(sort.list);

	sort.test = TEST_QUIET;

	ifelse = false;
	}

	if (ifelse) {

	//ASSERT(812, gen==GEN_END);

	return MoveNone;
	}

	sort.pos = 0;

	}

	//return MoveNone;
	}

	// sort_init_qs()

	private void sort_init_qs( sort_t sort, board_t board, attack_t attack, boolean check )  {

	sort.board = board;
	sort.attack = attack;

	if (ATTACK_IN_CHECK(sort.attack)) {
	sort.gen = this.PosEvasionQS;
	} else { 
	if (check) {
	sort.gen = this.PosCheckQS;
	} else { 
	sort.gen = this.PosCaptureQS;
	}
	}

	LIST_CLEAR(sort.list);
	sort.pos = 0;

	}

	// sort_next_qs()

	private int sort_next_qs( sort_t sort ) {

	int move  = 0;   // int
	int gen  = 0;    // int
	boolean nocont = false;
	boolean ifelse = false;

	while (true) {

	while (sort.pos < sort.list.size) {

	nocont = true;

	// next move

	move = sort.list.move[sort.pos];
	sort.pos++;

	//ASSERT(818, move!=MoveNone);

	// test

	ifelse = true;

	if (ifelse && (sort.test == TEST_LEGAL)) {

	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	ifelse = false;
	}

	if (ifelse && (sort.test == TEST_CAPTURE_QS)) {

	//ASSERT(819, move_is_tactical(move,sort.board));

	if (nocont && (! capture_is_good(move,sort.board))) {
	nocont = false;
	}
	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	ifelse = false;
	}

	if (ifelse && (sort.test == TEST_CHECK_QS)) {

	//ASSERT(820, ! move_is_tactical(move,sort.board));
	//ASSERT(821, move_is_check(move,sort.board));

	if (nocont && see_move(move,sort.board) < 0) {
	nocont = false;
	}
	if (nocont && (! pseudo_is_legal(move,sort.board))) {
	nocont = false;
	}

	ifelse = false;
	}

	if (ifelse) {

	//ASSERT(822, false);
	return MoveNone;

	}

	if (nocont) {

	//ASSERT(823, pseudo_is_legal(move,sort.board));
	return move;

	}

	}

	// next stage

	gen = this.Code[sort.gen];
	sort.gen++;

	ifelse = true;

	if (ifelse && (gen == GEN_EVASION_QS)) {

	gen_pseudo_evasions(sort.list,sort.board,sort.attack);
	note_moves_simple(sort.list,sort.board);
	list_sort(sort.list);

	sort.test = TEST_LEGAL;

	ifelse = false;
	}

	if (ifelse && (gen == GEN_CAPTURE_QS)) {

	gen_captures(sort.list,sort.board);
	note_mvv_lva(sort.list,sort.board);
	list_sort(sort.list);

	sort.test = TEST_CAPTURE_QS;

	ifelse = false;
	}

	if (ifelse && (gen == GEN_CHECK_QS)) {

	gen_quiet_checks(sort.list,sort.board);

	sort.test = TEST_CHECK_QS;

	ifelse = false;
	}

	if (ifelse) {

	//ASSERT(824, gen==GEN_END);

	return MoveNone;
	}

	sort.pos = 0;
	}

	////ASSERT(1824, false);
	//return MoveNone;
	}

	// good_move()

	private void good_move( int move, board_t board, int depth, int height )  {

	int index  = 0;   // int
	int i  = 0;       // int

	//ASSERT(825, move_is_ok(move));
	//ASSERT(827, depth_is_ok(depth));
	//ASSERT(828, height_is_ok(height));

	if (move_is_tactical(move,board)) {
	return;
	}

	// killer

	if (this.Killer[height][0] != move) {
	this.Killer[height][1] = this.Killer[height][0];
	this.Killer[height][0] = move;
	}

	//ASSERT(829, this.Killer[height][0]==move);
	//ASSERT(830, this.Killer[height][1]!=move);

	// history

	index = history_index(move,board);

	this.History[index] = this.History[index] + ( depth * depth );          // HISTORY_INC()

	if (this.History[index] >= HistoryMax) {
	for (i = 0; i<HistorySize; i++ ) {
	this.History[i] = (this.History[i] + 1) / 2;
	}
	}

	}

	// history_good()

	private void history_good( int move, board_t board )   {

	int index  = 0;   // int

	//ASSERT(831, move_is_ok(move));

	if (move_is_tactical(move,board)) {
	return;
	}

	// history

	index = history_index(move,board);

	this.HistHit[index]++;
	this.HistTot[index]++;

	if (this.HistTot[index] >= HistoryMax) {
	this.HistHit[index] = (this.HistHit[index] + 1) / 2;
	this.HistTot[index] = (this.HistTot[index] + 1) / 2;
	}

	//ASSERT(833, this.HistHit[index]<=this.HistTot[index]);
	//ASSERT(834, this.HistTot[index]<HistoryMax);
	}

	// history_bad()

	private void history_bad( int move, board_t board )   {

	int index  = 0;   // int

	//ASSERT(835, move_is_ok(move));

	if (move_is_tactical(move,board)) {
	return;
	}

	// history

	index = history_index(move,board);

	this.HistTot[index]++;

	if (this.HistTot[index] >= HistoryMax) {
	this.HistHit[index] = (this.HistHit[index] + 1) / 2;
	this.HistTot[index] = (this.HistTot[index] + 1) / 2;
	}

	//ASSERT(837, this.HistHit[index]<=this.HistTot[index]);
	//ASSERT(838, this.HistTot[index]<HistoryMax);

	}

	// note_moves()

	private void note_moves( list_t list, board_t board, int height,  int trans_killer )   {

	int size  = 0;   // int
	int i  = 0;      // int
	int move  = 0;   // int

	//ASSERT(839, list_is_ok(list));
	//ASSERT(841, height_is_ok(height));
	//ASSERT(842, trans_killer==MoveNone || move_is_ok(trans_killer));

	size = list.size;

	if (size >= 2) {
	for (i = 0; i<size; i++ ) {
	move = list.move[i];
	list.value[i] = move_value(move,board,height,trans_killer);
	}
	}

	}

	// note_captures()

/*
	private void note_captures( list_t list, board_t board )  {

	int size  = 0;   // int
	int i  = 0;      // int
	int move  = 0;   // int

	//ASSERT(843, list_is_ok(list));

	size = list.size;

	if (size >= 2) {
	for (i = 0; i< size; i++ ) {
	move = list.move[i];
	list.value[i] = capture_value(move,board);
	}
	}

	}

*/
	// note_quiet_moves()

	private void note_quiet_moves( list_t list, board_t board )  {

	int size  = 0;   // int
	int i  = 0;      // int
	int move  = 0;   // int

	//ASSERT(845, list_is_ok(list));

	size = list.size;

	if (size >= 2) {
	for (i = 0; i< size; i++ ) {
	move = list.move[i];
	list.value[i] = quiet_move_value(move,board);
	}
	}

	}

	// note_moves_simple()

	private void note_moves_simple( list_t list, board_t board )  {

	int size  = 0;   // int
	int i  = 0;      // int
	int move  = 0;   // int

	//ASSERT(847, list_is_ok(list));

	size = list.size;

	if (size >= 2) {
	for (i = 0; i< size; i++ ) {
	move = list.move[i];
	list.value[i] = move_value_simple(move,board);
	}
	}

	}

	// note_mvv_lva()

	private void note_mvv_lva( list_t list, board_t board )  {

	int size  = 0;   // int
	int i  = 0;      // int
	int move  = 0;   // int

	//ASSERT(849, list_is_ok(list));

	size = list.size;

	if (size >= 2) {
	for (i = 0; i< size; i++ ) {
	move = list.move[i];
	list.value[i] = mvv_lva(move,board);
	}
	}

	}

	// move_value()

	private int move_value( int move, board_t board, int height, int trans_killer )  {

	int value  = 0;   // int

	//ASSERT(851, move_is_ok(move));
	//ASSERT(853, height_is_ok(height));
	//ASSERT(854, trans_killer==MoveNone || move_is_ok(trans_killer));

	if (move == trans_killer) {    // transposition table killer
	value = TransScore;
	} else { 
	if (move_is_tactical(move,board)) {   // capture || promote
	value = capture_value(move,board);
	} else { 
	if (move == this.Killer[height][0]) {   // killer 1
	value = KillerScore;
	} else { 
	if (move == this.Killer[height][1]) {  // killer 2
	value = KillerScore - 1;
	} else {   // quiet move
	value = quiet_move_value(move,board);
	}
	}
	}
	}

	return value;

	}

	// capture_value()

	private int capture_value( int move, board_t board ) {

	int value  = 0;   // int

	//ASSERT(855, move_is_ok(move));

	//ASSERT(857, move_is_tactical(move,board));

	value = mvv_lva(move,board);

	if (capture_is_good(move,board)) {
	value +=  GoodScore;
	} else { 
	value +=  BadScore;
	}

	//ASSERT(858, value>=-30000 && value<=30000);

	return value;

	}

	// quiet_move_value()

	private int quiet_move_value( int move, board_t board ) {

	int value  = 0;   // int
	int index  = 0;   // int

	//ASSERT(859, move_is_ok(move));

	//ASSERT(861, ! move_is_tactical(move,board));

	index = history_index(move,board);

	value = HistoryScore + this.History[index];
	//ASSERT(862, value>=HistoryScore && value<=KillerScore-4);

	return value;

	}

	// move_value_simple()

	private int move_value_simple( int move, board_t board ) {

	int value  = 0;   // int

	//ASSERT(863, move_is_ok(move));

	value = HistoryScore;
	if (move_is_tactical(move,board)) {
	value = mvv_lva(move,board);
	}

	return value;

	}

	// history_prob()

	private int history_prob( int move, board_t board ) {

	int value  = 0;   // int
	int index  = 0;   // int

	//ASSERT(865, move_is_ok(move));

	//ASSERT(867, ! move_is_tactical(move,board));

	index = history_index(move,board);

	//ASSERT(868, this.HistHit[index]<=this.HistTot[index]);
	//ASSERT(869, this.HistTot[index]<HistoryMax);

	value = (this.HistHit[index] * 16384) / this.HistTot[index];
	//ASSERT(870, value>=0 && value<=16384);

	return value;

	}

	// capture_is_good()

	private boolean capture_is_good( int move, board_t board ) {

	int piece  = 0;     // int
	int capture  = 0;   // int

	//ASSERT(871, move_is_ok(move));

	//ASSERT(873, move_is_tactical(move,board));

	// special cases

	if (MOVE_IS_EN_PASSANT(move)) {
	return true;
	}
	if (move_is_under_promote(move)) {
	return false; // REMOVE ME?
	}

	// captures && queen promotes

	capture = board.square[MOVE_TO(move)];

	if (capture != Empty) {

	// capture

	//ASSERT(874, move_is_capture(move,board));

	if (MOVE_IS_PROMOTE(move)) {
	return true; // promote-capture
	}

	piece = board.square[MOVE_FROM(move)];
	if (this.ValuePiece[capture] >= this.ValuePiece[piece]) {
	return true;
	}
	}

	return (see_move(move,board) >= 0);

	}

	// mvv_lva()

	private int mvv_lva( int move, board_t board ) {

	int piece  = 0;     // int
	int capture  = 0;   // int
	int promote  = 0;   // int
	int value  = 0;     // int

	//ASSERT(875, move_is_ok(move));

	//ASSERT(877, move_is_tactical(move,board));

	if (MOVE_IS_EN_PASSANT(move)) {   // en-passant capture

	value = 5; // PxP

	} else { 

	capture = board.square[MOVE_TO(move)];

	if (capture!= Empty) {   // normal capture

	piece = board.square[MOVE_FROM(move)];

	value = (this.PieceOrder[capture] * 6) - this.PieceOrder[piece] + 5;
	//ASSERT(878, value>=0 && value<30);

	} else {// promote

	//ASSERT(879, MOVE_IS_PROMOTE(move));

	promote = move_promote(move);

	value = this.PieceOrder[promote] - 5;
	//ASSERT(880, value>=-4 && value<0);
	}
	}

	//ASSERT(881, value>=-4 && value<30);

	return value;

	}

	// history_index()

	private int history_index( int move, board_t board ) {

	int index  = 0;   // int

	//ASSERT(882, move_is_ok(move));

	//ASSERT(884, ! move_is_tactical(move,board));

	index = (this.PieceTo12[board.square[MOVE_FROM(move)]] * 64) + this.SquareTo64[MOVE_TO(move)];

	//ASSERT(885, index>=0 && index<HistorySize);

	return index;

	}

	// end of sort.cpp


	// square.cpp



	// square_init()

	private void square_init()  {

	int sq  = 0;   // int

	// this.SquareTo64[]

	for (sq = 0; sq< SquareNb; sq++ ) {
	this.SquareTo64[sq] = -1;
	}

	for (sq = 0; sq<=63; sq++ ) {
	this.SquareTo64[SquareFrom64[sq]] = sq;
	}

	// this.SquareIsPromote[]

	for (sq = 0; sq< SquareNb; sq++ ) {
	this.SquareIsPromote[sq] = SQUARE_IS_OK(sq)  &&  (SQUARE_RANK(sq) == Rank1  ||  SQUARE_RANK(sq) == Rank8);
	}

	}

	// file_from_char()

	private int file_from_char( char c ) {

	//ASSERT(886, c>='a' && c<='h');

	return FileA + (char)(c - 'a');
	}

	// rank_from_char()

	private int rank_from_char( char c ) {

	//ASSERT(887, c>='1' && c<='8');

	return Rank1 + (char)(c - '1');
	}

	// file_to_char()

	private char file_to_char( int file ) { 

	//ASSERT(888, file>=FileA && file<=FileH);

	return (char)( 'a' + (file - FileA) );

	}

	// rank_to_char()

	private char rank_to_char( int rank ) {

	//ASSERT(889, rank>=Rank1 && rank<=Rank8);

	return (char)('1'+ (rank - Rank1) );

	}

	// square_to_string()

	private boolean square_to_string( int square, string_t str1 ) {

	//ASSERT(890, SQUARE_IS_OK(square));

	str1.v = "";
	str1.v += file_to_char(SQUARE_FILE(square));
	str1.v += rank_to_char(SQUARE_RANK(square));

	return true;
	}

	// square_from_string()

	private int square_from_string( string_t str1 )  {

	int file  = 0;   // int
	int rank  = 0;   // int
	char c1;   // char
	char c2;   // char

	c1 = str1.v.charAt(0);
	if (c1 < 'a'  ||  c1 > 'h') {
	return SquareNone;
	}
	c2 = str1.v.charAt(1);
	if (c2 < '1'  ||  c2 > '8') {
	return SquareNone;
	}

	file = file_from_char(c1);
	rank = rank_from_char(c2);

	return SQUARE_MAKE(file,rank);
	}

	// end of square.cpp


	// trans.cpp



	// trans_is_ok()

	private boolean trans_is_ok( trans_t trans ) {

	int date  = 0;   // int

	if ( trans.size == 0 ) {
	return false;
	}

	if ((trans.mask == 0)  ||  (trans.mask >= trans.size)) {
	return false;
	}

	if (trans.date >= DateSize) {
	return false;
	}

	for (date = 0; date<DateSize; date++ ) {
	if (trans.age[date] != trans_age(date)) {
	return false;
	}
	}

	return true;

	}


	// trans_alloc()

	private void trans_alloc() { 

	trans_clear();

	//ASSERT(900, trans_is_ok(this.Trans));
	}


	// trans_clear()

	private void trans_clear()  {

	this.Trans.size = TransSize;
	this.Trans.mask = this.Trans.size - 1;   // 2^x -1
	trans_set_date(0);
	//trans.table = [];            // will define objects while searching
	}

	// trans_cl_I()

	private void trans_cl_I( trans_t trans, int index )  {

	trans.table[index] = new entry_t();

	entry_t clear_entry = trans.table[index];

	clear_entry.lock = 0;
	clear_entry.move = MoveNone;
	clear_entry.depth = DepthNone;
	clear_entry.date = trans.date;
	clear_entry.move_depth = DepthNone;
	clear_entry.flags = 0;
	clear_entry.min_depth = DepthNone;
	clear_entry.max_depth = DepthNone;
	clear_entry.min_value = -ValueInf;
	clear_entry.max_value = ValueInf;

	//ASSERT(903, entry_is_ok(clear_entry));

	}


	// trans_inc_date()

	private void trans_inc_date()   {

	trans_set_date((this.Trans.date+1)%DateSize);
	}

	// trans_set_date()

	private void trans_set_date( int date )   {

	int date1 = 0;

	//ASSERT(906, date>=0 && date<DateSize);

	this.Trans.date = date;

	for (date1 = 0; date1<DateSize; date1++ ) {
	this.Trans.age[date1] = trans_age(date1);
	}

	}

	// trans_age()

	private int trans_age( int date ) {

	int age  = 0;   // int

	//ASSERT(908, date>=0 && date<DateSize);

	age = this.Trans.date - date;
	if (age < 0) {
	age += DateSize;
	}

	//ASSERT(909, age>=0 && age<DateSize);

	return age;

	}

	// trans_store()

	private void trans_store( trans_t trans, int key, int move, int depth, trans_rtrv Tset )   {

	entry_t entry = null;        // entry_t *
	entry_t best_entry = null;   // entry_t *
	int ei  = 0;             // int
	int i  = 0;              // int
	int score  = 0;          // int
	int best_score  = 0;     // int
	boolean nw_rc = false;

	//ASSERT(910, trans_is_ok(trans));
	//ASSERT(911, move>=0 && move<65536);
	//ASSERT(912, depth>=-127 && depth<=127);
	//ASSERT(913, Tset.trans_min_value>=-ValueInf && Tset.trans_min_value<=ValueInf);
	//ASSERT(914, Tset.trans_max_value>=-ValueInf && Tset.trans_max_value<=ValueInf);
	//ASSERT(915, Tset.trans_min_value<=Tset.trans_max_value);

	// init

	trans.write_nb++;

	// probe

	best_score = -32767;

	ei = trans_entry(trans,key);

	for (i = 0; i< ClusterSize; i++ ) {

	entry = trans.table[ei+i];

	if (entry!=null) {

	if (entry.lock == KEY_LOCK(key)) {

	// hash hit => update existing entry

	trans.write_hit++;
	if (entry.date != trans.date) {
	trans.used++;
	}

	entry.date = trans.date;

	if (depth > entry.depth) {
	entry.depth = depth; // for replacement scheme
	}

	if (move != MoveNone  &&  depth >= entry.move_depth) {
	entry.move_depth = depth;
	entry.move = move;
	}

	if (Tset.trans_min_value > -ValueInf  &&  depth >= entry.min_depth) {
	entry.min_depth = depth;
	entry.min_value = Tset.trans_min_value;
	}

	if (Tset.trans_max_value < ValueInf  &&  depth >= entry.max_depth) {
	entry.max_depth = depth;
	entry.max_value = Tset.trans_max_value;
	}

	//ASSERT(916, entry_is_ok(entry));

	return;
	}

	} else { 

	trans_cl_I( trans, ei+i );   // create a new entry record
	nw_rc = true;

	entry = trans.table[ei+i];

	}

	// evaluate replacement score

	score = (trans.age[entry.date] * 256) - entry.depth;
	//ASSERT(917, score>-32767);

	if (score > best_score) {
	best_entry = entry;
	best_score = score;
	}

	if(nw_rc) {
	break;
	}

	}

	// "best" entry found

	entry = best_entry;

	//ASSERT(919, entry.lock!=KEY_LOCK(key));

	if (entry.lock != 0) {     // originally entry.date == trans.date
	trans.write_collision++;
	} else { 
	trans.used++;
	}

	// store

	entry.lock = KEY_LOCK(key);
	entry.date = trans.date;

	entry.depth = depth;

	entry.move_depth = ( move != MoveNone ? depth : DepthNone );
	entry.move = move;

	entry.min_depth = (Tset.trans_min_value > -ValueInf ? depth : DepthNone );
	entry.max_depth = (Tset.trans_max_value < ValueInf ? depth : DepthNone );
	entry.min_value = Tset.trans_min_value;
	entry.max_value = Tset.trans_max_value;

	//ASSERT(921, entry_is_ok(entry));

	}

	// trans_retrieve()

	private boolean trans_retrieve( trans_t trans, int key, trans_rtrv Ret ) {

	entry_t entry = null;   // entry_t *
	int ei  = 0;        // int
	int i  = 0;         // int

	//ASSERT(922, trans_is_ok(trans));

	// init

	trans.read_nb++;

	// probe

	ei = trans_entry(trans,key);

	//for (i = 0; i< ClusterSize; i++ )
	{

	entry = trans.table[ei+i];

	if ( entry!=null && entry.lock == KEY_LOCK(key)) {

	// found

	trans.read_hit++;
	if (entry.date != trans.date) {
	entry.date = trans.date;
	}

	Ret.trans_move = entry.move;

	Ret.trans_min_depth = entry.min_depth;
	Ret.trans_max_depth = entry.max_depth;
	Ret.trans_min_value = entry.min_value;
	Ret.trans_max_value = entry.max_value;

	return true;
	} else {
	return false;
	}
	}

	// not found

	//return false;
	}

	// trans_stats()

	private void trans_stats( trans_t trans )  {

	long full = 0;       // double
	long hit  = 0;        // double
	long collision = 0;  // double
	String s = "";

	//ASSERT(928, trans_is_ok(trans));

	full = (trans.size>0 ? 100 * trans.used / trans.size : 0) ;
	hit = (trans.read_nb>0 ? 100 * trans.read_hit / trans.read_nb : 0);
	collision = (trans.write_nb>0 ? 100 * trans.write_collision / trans.write_nb : 0);

	s += "\n" + "hash trans info";
	s += " hashfull " + Long.toString(full) + "%";
	s += " hits " + Long.toString(hit) + "%";
	s += " collisions " + Long.toString(collision) + "%";

	full = (this.Material.size>0 ? 100 * this.Material.used / this.Material.size : 0);
	hit = (this.Material.read_nb>0 ? 100 * this.Material.read_hit / this.Material.read_nb : 0);
	collision = (this.Material.write_nb>0 ? 100 * this.Material.write_collision / this.Material.write_nb : 0);

	s += "\n" + "hash material info";
	s += " hashfull " + Long.toString(full) + "%";
	s += " hits " + Long.toString(hit) + "%";
	s += " collisions " + Long.toString(collision) + "%";

	full = (this.Pawn.size>0 ? 100 * this.Pawn.used / this.Pawn.size : 0);
	hit = (this.Pawn.read_nb>0 ? 100 * this.Pawn.read_hit /this.Pawn.read_nb : 0);
	collision = (this.Pawn.write_nb>0 ? 100 * this.Pawn.write_collision / this.Pawn.write_nb : 0);

	s += "\n" + "hash pawn info";
	s += " hashfull " + Long.toString(full) + "%";
	s += " hits " + Long.toString(hit) + "%";
	s += " collisions " + Long.toString(collision) + "%";
	s += "\n";

	send( s );
	}

	// trans_entry()

	private int trans_entry( trans_t trans, int key ) {  // index to entry_t  

	int index  = 0;  // uint32

	//ASSERT(929, trans_is_ok(trans));

	index =  ( KEY_INDEX(key) & trans.mask);

	//ASSERT(930, index<=trans.mask);

	return index;

	}

	// entry_is_ok()

	private boolean entry_is_ok( entry_t entry ) {

	if (entry.date >= DateSize) {
	return false;
	}

	if (entry.move == MoveNone  &&  entry.move_depth != DepthNone) {
	return false;
	}
	if (entry.move != MoveNone  &&  entry.move_depth == DepthNone) {
	return false;
	}

	if (entry.min_value == -ValueInf  &&  entry.min_depth != DepthNone) {
	return false;
	}
	if (entry.min_value >  -ValueInf  &&  entry.min_depth == DepthNone) {
	return false;
	}

	if (entry.max_value == ValueInf  &&  entry.max_depth != DepthNone) {
	return false;
	}
	if (entry.max_value <  ValueInf  &&  entry.max_depth == DepthNone) {
	return false;
	}

	return true;
	}

	// end of trans.cpp



	// util.cpp

	// my_timer_reset()

	private void my_timer_reset( my_timer_t timer ) { 

	timer.start_real = 0;
	timer.elapsed_real = 0;
	timer.running = false;

	}

	// my_timer_start()

	private void my_timer_start( my_timer_t timer ) { 

	//ASSERT(946, timer.start_real==0);
	//ASSERT(948, ! timer.running);

	timer.running = true;
	timer.start_real = os_clock();

	}

	// my_timer_stop()

/*	
	private void my_timer_stop( my_timer_t timer ) { 

	//ASSERT(950, timer.running);

	timer.elapsed_real = timer.elapsed_real + os_clock() - timer.start_real;
	timer.start_real = 0;
	timer.running = false;

	}
*/
	
	// my_timer_elapsed_real()

	private long my_timer_elapsed_real( my_timer_t timer ) { 

	if (timer.running) {
		timer.elapsed_real = (os_clock()-timer.start_real);
	}

	return timer.elapsed_real;
	}


	// end of util.cpp



	// value.cpp



	// value_init()

	private void value_init() { 

	int piece  = 0;   // int

	// ValuePiece[]

	for (piece = 0; piece<=1; piece++) {
	this.ValuePiece[piece] = -1;
	}

	this.ValuePiece[Empty] = 0; // needed?
	this.ValuePiece[Edge]  = 0; // needed?

	this.ValuePiece[WP] = ValuePawn;
	this.ValuePiece[WN] = ValueKnight;
	this.ValuePiece[WB] = ValueBishop;
	this.ValuePiece[WR] = ValueRook;
	this.ValuePiece[WQ] = ValueQueen;
	this.ValuePiece[WK] = ValueKing;

	this.ValuePiece[BP] = ValuePawn;
	this.ValuePiece[BN] = ValueKnight;
	this.ValuePiece[BB] = ValueBishop;
	this.ValuePiece[BR] = ValueRook;
	this.ValuePiece[BQ] = ValueQueen;
	this.ValuePiece[BK] = ValueKing;
	}

	// value_is_ok()

	private boolean value_is_ok( int value ) {

	if (value < -ValueInf  ||  value > ValueInf) {
	return false;
	}

	return true;
	}

	// range_is_ok()

	private boolean range_is_ok( int min, int max ) {

	if (! value_is_ok(min)) {
	return false;
	}
	if (! value_is_ok(max)) {
	return false;
	}

	if (min >= max) {
	return false; // alpha-beta-like ranges cannot be null
	}

	return true;
	}

	// value_is_mate()

	private boolean value_is_mate( int value ) {

	//ASSERT(954, value_is_ok(value));

	if (value < -ValueEvalInf  ||  value > ValueEvalInf) {
	return true;
	}

	return false;
	}

	// value_to_trans()

	private int value_to_trans( int value, int height ) {

	//ASSERT(955, value_is_ok(value));
	//ASSERT(956, height_is_ok(height));

	if (value < -ValueEvalInf) {
	value -=  height;
	} else { 
	if (value > ValueEvalInf) {
	value +=  height;
	}
	}

	//ASSERT(957, value_is_ok(value));

	return value;

	}

	// value_from_trans()

	private int value_from_trans( int value, int height ) {

	//ASSERT(958, value_is_ok(value));
	//ASSERT(959, height_is_ok(height));

	if (value < -ValueEvalInf) {
	value +=  height;
	} else { 
	if (value > ValueEvalInf) {
	value -=  height;
	}
	}

	//ASSERT(960, value_is_ok(value));

	return value;

	}

	// value_to_mate()

	private int value_to_mate( int value ) {

	int dist  = 0;   // int

	//ASSERT(961, value_is_ok(value));

	if (value < -ValueEvalInf) {

	dist = (ValueMate + value) / 2;
	//ASSERT(962, dist>0);

	return -dist;

	} else { 
	if (value > ValueEvalInf) {

	dist = (ValueMate - value + 1) / 2;
	//ASSERT(963, dist>0);

	return dist;
	}
	}

	return 0;
	}

	// end of value.cpp



	// vector.cpp



	private void vector_init()  {

	int delta  = 0;   // int
	int x  = 0;       // int
	int y  = 0;       // int
	int dist  = 0;    // int
	int tmp  = 0;     // int

	// Distance[]

	for (delta = 0; delta<DeltaNb; delta++ ) {
	this.Distance[delta] = -1;
	}

	for (y = -7; y<=7; y++) {

	for (x = -7; x<=7; x++) {

	delta = y * 16 + x;
	//ASSERT(964, delta_is_ok(delta));

	dist = 0;

	tmp = x;
	if (tmp < 0) {
	tmp = -tmp;
	}
	if (tmp > dist) {
	dist = tmp;
	}

	tmp = y;
	if (tmp < 0) {
	tmp = -tmp;
	}
	if (tmp > dist) {
	dist = tmp;
	}

	this.Distance[DeltaOffset+delta] = dist;
	}
	}

	}


	// delta_is_ok()

	private boolean delta_is_ok( int delta ) {

	if (delta < -119  ||  delta > 119) {
	return false;
	}

	if ( (delta & 0xF) == 8) {
	return false;     // HACK: delta % 16 would be ill-defined for negative numbers
	}

	return true;
	}


	// inc_is_ok()

	private boolean inc_is_ok( int inc ) {

	int dir  = 0;   // int

	for (dir = 0; dir<8 ; dir++ ) {
	if (KingInc[dir] == inc) {
	return true;
	}
	}

	return false;
	}

	// end of vector.cpp





	// main.cpp


	// main()

	private void main() { 

	// init

	print2out( VERSION );

	option_init();

	square_init();
	piece_init();
	pawn_init_bit();
	value_init();
	vector_init();
	attack_init();

	move_do_init();

	random_init();
	hash_init();

	inits();
	setstartpos();

	}

	// end of main.cpp

	private void ClearAll() {
	    // just clear all to be sure that nothing left

	search_clear(false);
	
	this.Trans = new trans_t();
	trans_clear();
	
	this.Pawn = new pawn_t();
	pawn_clear();
	
	this.Material = new material_t();
	material_clear();

	System.gc();		// Garbage collector
	}

	// randomized first move
	private boolean  random1stmv( String mvlist ) {

		   String m = "";
		   int i = 0;
		   int j = 0;
		   int l = mvlist.length();
		   Random rand = new Random();
		   boolean f;

		   String[][] fmv = { {"e2-e4", "d2-d4", "Ng1-f3", "Nb1-c3", "c2-c4", "g2-g3", "e2-e4", "c2-c3", "e2-e4", "d2-d4" },
		                   { "e7-e5", "d7-d5", "Ng8-f6", "Nb8-c6", "c7-c5", "g7-g6", "c7-c5", "c7-c6", "e7-e6", "g7-g6" } };

		   String[] bad = { "d2d4e7e5", "e2e4d7d5", "g1f3e7e5" };

		   
		   if(l<6) {
			   
			for(f=false;!f;)
			{	
		     m = string_from_int(rand.nextInt());
		     i = (int)( m.charAt(m.length()-1) - '0');
		     m = fmv[ (  l == 0 ? 0 : 1 ) ] [ i ];
		     j = ( m.length() >5 ? 1 : 0 );
		     m = m.substring(j,j+2) + m.substring(3+j, 3+j+2);
			 for(f=true,i=0; i<bad.length;i++)
				 if( bad[i].equals(mvlist+m) ) f=false;
			}
		     this.bestmv = m;
		     bestmv2 = m;
		     return true;
		   }

		   return false;
		}

	// Get MOVES NOW of board interface
	public String getGENMOVES()
	{
	int i;
	int move;
	string_t ms = new string_t();
	list_t list = new list_t();
    board_t Bo = this.SearchInput.board;

	String s = "";
	String sv_bestmv = this.bestmv;
	String sv_bestmv2 = this.bestmv2;

    string_t save_board = new string_t();
    
	board_to_fen(Bo, save_board);   // save board
	
	gen_legal_moves(list,Bo);
	
	for(i=0;i<list.size;i++)
	{	
		move = list.move[i];
		move_to_string(move,ms);
		this.bestmv = ms.v;
		format_best_mv2( move );
		s += ms.v + " (" + this.bestmv2 + ") ";
	}
	this.bestmv = sv_bestmv;	
	this.bestmv2 = sv_bestmv2;
	board_from_fen(Bo, save_board.v); // && restore
    
	return s;
	}
	
	// Get FEN of board interface
	public String getFEN()
	{
	string_t fs = new string_t();
	board_to_fen( SearchInput.board, fs);
	return fs.v;
	}
	
	// Constructor
	Fruit21()
	{
	    main();  // initialize and set up starting position
	}
	
	// This plays AI vs AI
	
	private void AutoGame()
	{
		boolean auto_over = false;
		String auto_mlist = "";
		String auto_pgn = "";
		
		int auto_mc = 0;
		
		while(!auto_over)
		{
			do_input( "go");

			if( auto_mc % 2 == 0) {
				auto_pgn += string_from_int((int)(auto_mc/2)+1)+".";
			}
			auto_pgn += this.bestmv2 + " ";

			auto_mlist += " " + this.bestmv;

			do_input( "position moves" + auto_mlist );

			printboard();

			print2out(auto_pgn);

			if( board_is_mate(  this.SearchInput.board ) ) {
			print2out("Checkmate! " + ( this.SearchInput.board.turn == White ? "0-1" : "1-0" ));
			auto_over = true;
			}
			if( board_is_stalemate( this.SearchInput.board ) ) {
			print2out("Stalemate  1/2-1/2");
			auto_over = true;
			}

			auto_mc++;
		}	
	}
	
	//---------------------------------
	// call Samples from outside...
	//---------------------------------
	public void Samples()
	{
	//  do_input( "help" );

	//  this.ShowInfo = true;
		
	AutoGame();

	//  do_input( "position moves e2e4 e7e5 g1f3 g8f6 f1c4 f8c5 e1g1 e8g8" );
        
	//  do_input( "position fen 7k/Q7/2P2K2/8/8/8/8/8 w - - 70 1" );
	//  do_input( "go movetime 10");
	//  do_input( "go depth 6");
		 
	//  printboard();
	//  print2out(bestmv2);		// disp all moves made
	}


}

